(* Package Name: salvador.m
   Author: Qi Zheng, Department of Epidemiology and Biostatistics
           School of Rural Public Health, Texas A&M Health Science Center
   Starting date: March 14, 2000, National Center for Toxicological Research
   Major revision: March 5, 2001
   Refinement started: May 11, 2001
   Revisions made for Version 5.0 Mathematica: April 8, 2004
   Major revision for version 2 started April 27, 2004 
     at School of Rural Public Health, Bryan, Texas *)

BeginPackage["salvador`", {"Statistics`DiscreteDistributions`",
                           "Statistics`ContinuousDistributions`",
                           "Statistics`Common`DistributionsCommon`"}]

Options[profileIntervalM]=Options[profileIntervalR]=
    {Alpha -> 0.05, MaxIterations -> 50, 
     InitialM -> -1, InitialR -> -1, InitialLowerM -> -1, 
     InitialUpperM -> -1, Tolerance -> 1/1000000000,
     InitialLowerR -> -1, InitialUpperR -> -1, 
     ShowIterations -> False}

Options[LRIntervalLD] = {Alpha -> 0.05, Tolerance -> 10^-8,
     ShowIterations->False, MaxIterations -> 50, Phi -> 1.0, InitialM->-1,
     InitialLowerM->-1, InitialUpperM->-1}

Options[newtonLD] = {ShowIterations -> False, Phi -> 1,
     InitialM -> -1, MaxIterations -> 50, Tolerance -> 10^-8}

Options[newtonM] = {ShowIterations -> False, MaxIterations -> 50, 
     Tolerance -> 10^-8, InitialM -> -1, InitialR -> -1}
 
Options[plotFit] = {logPlot -> False, DifferentialGrowth -> False, 
     dataStyle -> {RGBColor[1, 0, 0], PointSize[0.015]},
     fitStyle -> {RGBColor[0, 1, 0], Thickness[0.006]},
     InitialM -> -1, InitialR->-1}

Newcombe::usage="Newcombe[data] estimates m using m=(max y-mean y)/n."

JonesMedian::usage="JonesMedian[data] estimates m under the LD(m) model using
the method of the median by Jones et al."

FisherInfoMatrix::usage="FisherInfoMatrix[m,r,n] computes the expected Fisher
information matrix for the M(m,r) model, using n terms."

profileIntervalM::usage="profileIntervalM[data,opt] computes profile likelihood
based confidence intervals for m under the M(m,r) model."

profileIntervalR::usage="profileIntervalR[data,opt] computes profile likelihood
based confidence intervals for r(=1/\!\(\[Rho]\)) under the M(m,r) model."

LDLikelihood::usage="LDLikelihood[m,phi,data] gives minus log likelihood under
the LD(m,phi) distribution."

logLikelihoodM::usage="logLikelihoodM[m,r,data] gives the log likelihood function
under the M(m,r) model."

scoreInfoM::usage="scoreInfoM[data,m,r] is a temporary function ...."

newtonLD::usage="newtonLD[data,opts] uses the Newton-Raphson method to compute
mle of m for the LD model."

newtonM::usage="newtonM[data,opts] uses the Newton-Raphson method to compute
MLEs of m and r for the M(m,r) model."
LRIntervalLD::usage="LRIntervalLD[X,opts] uses data given in X to construct
a likelihood ratio (LR) based confidence interval for m that has asymptotic
confidence coefficient \!\(1-\[Alpha]\); \!\(\[Alpha]\) is specified by
Alpha->\!\(\[Alpha]\). Data in X is assumed to be generated by an
LD(m,\!\(\[Phi]\)) distribution; \!\(\[Phi]\) is specified by Phi->\!\(\[Phi]\)."

pmfLD::usage="pmfLD[\!\(m,\[Phi],k\)] returns a list of probabilities
\!\({p\_0,p\_1,..., p\_k}\) according to a Luria-Delbr\!\(\[UDoubleDot]\)uck
distribution, LD(m,\!\(\[Phi]\)). Note pmfLD[m,k] is the same as pmfLD[m,1,k]."

pmfM::usage="pmfM[m,r,k] returns a list of probabilities
\!\({p\_0,p\_1,...,p\_k}\) according to an \!\(M(m,r,1)\) distribution;
pmfM[m,\!\(\[Rho],\[Phi]\),n] returns the same list according to an
\!\(M(m,r,\[Phi])\) distribution. Note m is the mean number of mutations,
\!\(r=\[Beta]\_1/\[Beta]\_2\) is the ratio of mutant growth rate to nonmutant
growth rate, and \!\(\[Phi]=1-N\_0/N\_T\)."

pmfMKoch::usage="pmfMKoch[m,r,k] computes \!\({p\_0,p\_1,...,p\_k}\)
for an \!\(M(m,r,1)\) distribution, using Koch's algorithm. Note that r is the
reciprocal of Koch's original parameter b (called \!\(\[Rho]\) in SALVADOR v.1).
(This function is not recommended for large k.)"

pmfPoisson::usage="pmfPoisson[\!\(\[Lambda]\),k] returns a list of probabilities
\!\({p\_0,p\_1,...,p\_k}\) according to a Poisson(\!\(\[Lambda]\)) distribution."

pmfCairns::usage="pmfCairns[\!\(m,\[Phi],\[Lambda],k\)] returns a list of
probabilities \!\({p\_0,...,p\_k}\) according to the convolution of
an LD(\!\(m,\[Phi]\)) distribution and a Poisson(\!\(\[Lambda]\)) distribution."

pmf2cdf::usage="pmf2cdf[\!\({p\_0,p\_1,...,p\_k}\)] returns a list of cumulative
probabilities according to the given list of probabilities
\!\({p\_0,p\_1,...,p\_k}\)."

simuMutants::usage="simuMutants[\!\(\[Mu],\[Beta]\_1,\[Beta]\_2, N\_0,N\_T\)]
simulates the number of mutants in a sister culture. Note \!\(\[Mu]\) is the
mutation rate per cell per unit time; \!\(\[Beta]\_1\) is growth rate for
nonmutants; \!\(\[Beta]\_2\) is growth rate of mutants; \!\(N\_0\) is the initial
population size; \!\(N\_T\) is the population size prior to plating."

MeanOfMutants::usage="MeanOfMutants[\!\(\[Mu],\[Beta]\_1,\[Beta]\_2,N\_0,N\_T\)]
computes the mean number of mutants on each sister culture."

VarOfMutants::usage="VarOfMutants[\!\(\[Mu],\[Beta]\_1,\[Beta]\_2,N\_0,N\_T\)]
computes the variance of the number of mutants on each sister culture."

mleLD::usage="mleLD[data,\!\(\[Phi]:=1\),\!\(M\_\(min\)\):=0.001,
\!\(M\_\(max\)\):=50] searches in the interval
[\!\(M\_\(min\), M\_\(max\)\)] for the maximum likelihood estimate of m, 
when data is assumed to be generated by an LD(m,\!\(\[Phi]\)) distribution. Note
\!\(\[Phi]\) is considered as known." 

mleM::usage="mleM[data,M:50,R:5] searches for the maximum likelihood estimates
of both m and r by assuming that the data is generated by an
M(m,r,1) distribution. The search is confined to
\!\(m\[Element](0.001,M)\) and \!\(r\[Element](0.01,R)\). Note \!\(r=1/\[Rho]\)."

FisherInfoLD::usage="FisherInfoLD[m,\!\(\[Phi]\),k] computes the Fisher
information for m according to an LD(m,\!\(\[Phi]\)) distribution; it retains
k terms in the series \!\(I(m)=\[Sum] f\^2\%j/p\_j\)."

FisherInfoM::usage="FisherInfoM[m,\!\(\[Rho],\[Phi]\),k] computes the Fisher
information for m, I(m), according to an M(m,\!\(\[Rho],\[Phi]\)) distribution;
it retains k terms in the series \!\(I(m)=\[Sum] f\^2\%j/p\_j\). Note that
FisherInfoM[m,\!\(\[Rho]\),k] assumes \!\(\[Phi]=1\)."

FisherInfoRho::usage="FisherInfoRho[m,\!\(\[Rho]\),k] computes the
Fisher information for \!\(\[Rho]\), I(\!\(\[Rho]\)), using an
M(m, \!\(\[Rho],1\)) distribution. It retains k terms in the series
\!\(I(\!\(\[Rho]\))=\[Sum] f\^2\%j/p\_j\)."

FisherInfoMLea::usage="FisherInfoMLea[m,\!\(\[Rho]\),k] computes the Fisher
information for both m and \!\(\[Rho]\) according to an M(m,\!\(\[Rho]\),1)
distribution by the algorithm proposed by Lea and Coulson. It uses
k terms to approximate I(m) and I(\!\(\[Rho]\)). (This function is not
recommended for large k.)"

MethodOfP0::usage="MethodOfP0[data] estimates m, the expected number of mutations,
using the \!\(P\_0\) method."

MethodOfMeans::usage="MethodOfMeans[data,\!\(w\_0:=10\)] estimates m (the mean
number of mutations) by the method of means as proposed by Luria and
Delbr\!\(\[UDoubleDot]\)ck. Note \!\(w\_0\) is an initial value needed
for numerically solving a transcendental equation."

MethodOfMedians::usage="MethodOfMedians[data,\!\(w\_0:=1\)] estimates m, the
expected number of mutations, using the method of medians proposed by Lea and
Coulson. Note \!\(w\_0\) is an initial value needed to solve a transcendental
equation by Newton's method.  If only the median of the data is known,
say med, MethodOfMedians[med] also returns an estimate of m."

semiMLE::usage="semiMLE[data,\!\(m\_0:=1\)] computes an estimate
of m by solving \!\(\[Sum]Y\_i=0\), where \!\(Y\_i\) are the
Lea-Coulson transforms of the original data. Note that
\!\(m\_0\) is a starting value used to solve the equation."

CIMutationRate::usage="CIMutationRate[data,\!\(\[Alpha],N\_T,k\)] constructs
a \!\(\[Alpha]\) level confidence interval for \!\(\[Mu]\_\[Beta]\) using an
LD(m,1) distribution. It uses k terms in computing the Fisher information for
m. It is important to choose a suitable k. Note \!\(N\_T\) is the number of
nonmutants prior to plating."

plotFit::usage="plotFit[data,opts] fits either an LD(m,1) or an
M(m,\!\(\[Rho]\),1) distribution to data, and then plots the fitted distribution
along with the data. Use options for different ploting styles."

Begin["`Private`"]  
(* Fisher's information I(m) for an LD(m,phi) *)
(* revised October 30, 2001 *)
FisherInfoLD[m_, phi_:1, n_] := Module[{l, p, f}, 
     l = Table[phi^(k-1)*(1/k - phi/(k + 1)), {k, 1, n}];
     l = Prepend[l, -1]; 
     p = pmfLD[m, phi, n];
     If[Last[p] == 0, Return[Print["n is too large \!\(p\_n=0\) encountered."]]]; 
     f = $luria`$pdfconv[p, N[l]];
     Plus @@ (f^2/p)]/;(m>0)&&(phi>0)&&(phi<=1)&&(n>0)

(* observed information J(m) for an LD(m,phi)  4/27/04 *)
ObservedInfoLD[m_, phi_:1, U_] := Module[{l, p, p1, p2}, 
     l = Table[phi^(k-1)*(1/k - phi/(k + 1)), {k, 1, U}];
     l = N[Prepend[l, -1]];
     p = pmfLD[m, phi, U];
     p1=$luria`$pdfconv[p,l]; 
     p2=$luria`$pdfconv[p1,l]; 
     Plus@@(p1^2/p-p2)
         ]/;(m>0)&&(phi>0)&&(phi<=1)&&(U>0)

crossInfo[m_, r_, n_] := Module[{Lm, Lr, be, et, p, p10, p01, f11, f12, f22}, 
     be = Table[(j - 1)/(j + r), {j, 2, n}];
     be = FoldList[Times, 1, be]; 
     be = be/(1 + r);
     Lm = Prepend[r*be, -1]; 
     et = Table[1/(j + r), {j, 1, n}];
     et = FoldList[Plus, First[et], Rest[et]];
     Lr = m*be*(1 - r*et);
     Lr = Prepend[Lr, 0];
     p = pmfM[m, r, 1, n];
     p10=$luria`$pdfconv[p,Lm];
     p01=$luria`$pdfconv[p,Lr];
     f11=p10*p10/p;
     f12=p10*p01/p;
     f22=p01*p01/p;
     {Plus@@f11, Plus@@f12, Plus@@f22}]
 
(* Fisher's information I(m) for an M(m,rho,phi) distribution *)
FisherInfoM[m, 1, phi_, n_Integer]:=FisherInfoLD[m,phi,n]

FisherInfoM[m_, r_, 1, n_Integer] := $luria`$kochinfo[N[m],N[r],n]

FisherInfoM[m_, r_, n_Integer] := FisherInfoM[m,r,1,n]

FisherInfoM[m_, r_, phi_, n_Integer] := Module[{l, p, f, omega}, 
     omega = 1 - (1 - phi)^r;
     l = (Beta[omega, #1, 1 + 1/r] & ) /@ Range[n]; 
     l = Prepend[l/r/phi, -1];
     p = pmfM[m, r, phi, n];
     f = $luria`$pdfconv[l, p]; 
     Plus @@ (f^2/p)]/;(m>0)&&(r>0)&&(phi>0)&&(phi<1)&&(n>0)

FisherInfoRho[m_, r_, n_Integer] := $luria`$iofrho[N[m],N[r],n]
 
(* FisherInfoLDLC[m_/;Positive[m],n_Integer]/;n>0:= $luria`$fishinfo[N[m],n]*)

FisherInfoMLea[m_,r_,n_]:=$luria`$fisherho[N[m],N[r],n] 

(* simulating the number of mutants in a sister culture *)
simuMutants[mu_, b1_, b2_, N0_, Nt_] := 
    Module[{meanMutation, noOfMutation, T, mutationEpoch},
      T = Log[Nt/N0]/b1;
      meanMutation = mu*N0*(Exp[b1*T] - 1)/b1; 
      noOfMutation = Random[PoissonDistribution[meanMutation]]; 
      mutationEpoch = Table[Log[1 + Random[]*(Exp[b1*T] - 1)]/b1,{noOfMutation}];
      Plus @@ (Random[GeometricDistribution[Exp[-b2*(T - #1)]]] + 1 & ) /@ 
        mutationEpoch]

(* mean number of mutants *)
MeanOfMutants[mu_, b1_, b2_, N0_, Nt_] := Module[{t},
   t = Log[Nt/N0]/b1;
   If[b1 == b2, mu*N0*t*Exp[b1*t], 
       mu*N0*(Exp[b1*t] - Exp[b2*t])/(b1 - b2)]]
 
(* variance of the number of mutants *) 
VarOfMutants[mu_, b1_, b2_, N0_, Nt_] := Module[{t},
    t = Log[Nt/N0]/b1;
    If[b1 == b2, 
       2*mu*N0*Exp[b1*t]*(Exp[b1*t] - 1)/b1 - mu*N0*t*Exp[b1*t], 
       If[b1 == 2*b2, mu*N0*Exp[b2*t]*(1 - Exp[b2*t] + 2*b2*t*Exp[b2*t])/b2, 
        mu*N0*Exp[b2*t]*(b1*(1 + Exp[(b1 - b2)*t] - 2*Exp[b2*t]) + 
            2*b2*(Exp[b2*t] - 1))/(b1 - b2)/(b1 - 2*b2)]]]
 
(* Poisson distribution *)
pmfPoisson[lambda_ /; Positive[lambda], k_Integer /; NonNegative[k]] := 
    Exp[-N[lambda]]*FoldList[Times, 1, N[lambda]/Range[k]]

(* Luria-Delbruck distribution *)
pmfLD[m_/;Positive[m], phi_/;(phi>0)&&(phi<=1),
  k_Integer/;NonNegative[k]] := $luria`$pdfluria[N[m],N[phi],k]

pmfLD[m_, k_Integer]:=pmfLD[m,1,k]/;(m>0)&&(k>=0)

(* Cairnsian distribution *)
pmfCairns[m_, phi_, poi_, k_Integer] := Module[{LDProb, PoiProb}, 
     LDProb = $luria`$pdfluria[N[m], N[phi], k];
     PoiProb = pmfPoisson[N[poi],k];
     $luria`$pdfconv[LDProb,PoiProb]
]/;(m>0)&&(phi>0)&&(phi<=1)&&(poi>0)&&(k>=0)

(* the M distribution *)
pmfM[m_,r__,0]:={Exp[-N[m]]}

(* Koch's algorithm for the M(m,r,1) distribution, slow for large n *)
pmfMKoch[m_, r_, 0]:={Exp[-m]}
pmfMKoch[m_, r_, n_Integer] := Module[{q},
q=$luria`$pdfkoch[N[m],N[1/r],n];  (* Koch uses rho=1/r, 5/14/04 *)
Exp[-N[m]] Prepend[q,1]
]/;(m>0)&&(r>0)&&(n>0)

(* pmf for an M(m,r,1) distribution, using an algorithm like in Stewart et al. *)
pmfM[m_,r_,n_Integer]:=$luria`$kochstew[N[m],N[r],n] (* from 1/r to r 5/13/04 *)

pmfM[m_,r_,1,n_Integer]:=$luria`$kochstew[N[m],N[r],n] (* from 1/r to r, 5/13/04 *)

(* exact algorithm of Stewart et al. *)
pmfM[m_, r_, phi_, n_Integer] := Module[
      {q, r1=1+r, omega=N[1-(1-phi)^(1/r)]},
       q = (Beta[omega, #1, r1] & ) /@ Range[n]; 
       q= q*m*r/phi;
       q = Prepend[q, -N[m]];
       $luria`$poisum[q]
]/;(phi>0)&&(phi<1)&&(m>0)&&(r>0)&&(n>0)

(* cumulative probabilities *)
pmf2cdf[l_List] := FoldList[Plus, First[l], Rest[l]] /; Length[l] > 0

(* the median method of Jones et al. 5/19/04 *)
JonesMedian[data_]:=Module[{r},
r=Median[data];
(r-0.693147)/(Log[r]+0.3665)]

(* the Newcombe method, 5/20/04 *)
Newcombe[data_]:=Module[{},(Max[data]-Mean[data])/Length[data]//N]

(* the P0 method *)
MethodOfP0[data_] := Module[{p0},
      p0 = Count[data, 0]/Length[data]; 
      If[p0 > 0, -Log[N[p0]], Return[ Print[
        "The \!\(P\_0 \) method is not applicable to this case."];Null]]
]

(* Luria and Delbruck's original method of means *) 
MethodOfMeans[data_List, y0_:10] := Module[{n, M, y, t}, 
     n = Length[data];
     M = Plus @@ data;
     y = FixedPoint[Function[t, t - (t*Log[t] - N[M])/(Log[t] + 1)], y0]; 
     y/n]

(* method of median *)
MethodOfMedians[med_?NumericQ,y0_:1] := Module[{y},
      If[med==0,Exp[-1.24], 
        y=FixedPoint[Function[t,t-(t+Log[t]-1.24-Log[med])/(1 + 1/t)],y0];
        med/y]
]

MethodOfMedians[data_List,y0_:1] := MethodOfMedians[Median[data],y0]

(* the semi-MLE of Lea and Coulson *)
semiMLE[data_,m0_:1] := Module[{m,root}, 
     root = FindRoot[leaLike[m, data], {m, m0}];
     root[[1]][[2]]
]

(* MLE for m using an LD(m,phi) distribution, phi is assumed to be known *)
mleLD[data_, phi_:1, minM_:0.001, maxM_:50] := Module[{m, opt}, 
     opt = FindMinimum[LDLikelihood[m, phi, data], {m, minM, maxM}]; 
     opt[[2]][[1]][[2]]
]

(* MLE for both m and r, using an M(m,r,1) distribution *)
mleM[data_, M_:50, R_:5] := Module[{m, r, opt}, 
     opt = FindMinimum[-logLikelihoodM[m, r, data], {{m, 0.001, M}, {r, 0.01, R}}];
    {opt[[2]][[1]][[2]], opt[[2]][[2]][[2]]}]

(* confidence interval for mutation rate with an LD(m,1) model *)
CIMutationRate[data_, level_, Nt_, k_:1000] := Module[{m, z, d, f0, f1}, 
     z = Quantile[
         Statistics`NormalDistribution`NormalDistribution[0, 1], (1 - level)/2]; 
     m = mleLD[data];
     f0 = FisherInfoLD[m, 1, k]; 
     f1 = FisherInfoLD[m, 1, k + 1000];
     If[f1 - f0 > 0.001, Return[Print["Specified k is too small."]]];
     d = z/Sqrt[Length[data]*f1]; 
     {m + d, m - d}/Nt]

(* plot the fitting, May 11, 2001 *)

plotFit[data_List, opts___] := Module[{distinctPt, freq, cumuFreq, g1, g2, m, 
      r, prob, koch, pointStyle, lineStyle, useLog, rescale, initM, initR}, 
     koch = DifferentialGrowth /. {opts} /. Options[plotFit]; 
      pointStyle = dataStyle /. {opts} /. Options[plotFit]; 
      lineStyle = fitStyle /. {opts} /. Options[plotFit]; 
      useLog = logPlot /. {opts} /. Options[plotFit];
      initM = InitialM /. {opts} /. Options[plotFit];
      initR = InitialR /. {opts} /. Options[plotFit];   (* 5-17-2004 *)
      rescale = If[useLog, (Log[#+1])&, Identity];  (* add "1" ... 5/18/04 *)
      distinctPt = Sort[Union[data]]; 
      freq = (Count[data, #1] & ) /@ distinctPt; 
      cumuFreq = FoldList[Plus, First[freq], Rest[freq]]/Length[data]; 
      g1 = ListPlot[Transpose[{rescale[distinctPt], cumuFreq}], 
        PlotStyle -> Flatten[{pointStyle}],
        DisplayFunction -> Identity];
      If[koch, {m, r} = newtonM[data, InitialM->initM, InitialR->initR],
         m = newtonLD[data, InitialM->initM]];  (* 4-30-04 *)
      If[koch, prob = pmfM[m, r, Max[data]], prob = pmfLD[m, 1, Max[data]]]; 
      prob = pmf2cdf[prob][[Range[1+Min[data], 1+Max[data]]]]; 
      g2 = ListPlot[Transpose[{rescale[Range[Min[data], Max[data]]], prob}], 
        PlotJoined -> True, PlotStyle -> Flatten[{lineStyle}],
        DisplayFunction -> Identity];
      Show[g1, g2, PlotRange -> All, Frame -> True, 
       DisplayFunction -> $DisplayFunction, Axes -> None]]


(* likelihood ratio-based confidence interval for m with LD distribution, 5/4/04 *)
LRIntervalLD[data_List, opts___] := Module[
     {mhat, la, qa, h, m0, m1, like, score, info, mL, mU, alpha, tol,
      initM, initLow, initUp, maxIter, phi, showIter}, 
     alpha = Alpha /. {opts} /. Options[LRIntervalLD]; 
     tol = Tolerance /. {opts} /. Options[LRIntervalLD]; 
     maxIter = MaxIterations /. {opts} /. Options[LRIntervalLD]; 
     phi = Phi /. {opts} /. Options[LRIntervalLD];
     initM= InitialM /. {opts} /. Options[LRIntervalLD];
     initLow= InitialLowerM /. {opts} /. Options[LRIntervalLD];
     initUp= InitialUpperM /. {opts} /. Options[LRIntervalLD];
     showIter= ShowIterations /. {opts} /. Options[LRIntervalLD];
     If[phi<0||phi>1, Return["\!\(\[Phi]\) out of range!"]];
     mhat = newtonLD[data, Phi->phi, InitialM->initM, Tolerance->tol];
     {like, score} = LDLoglikeScore[mhat, phi, data]; 
     {score, info} =LDScore$Info[mhat,phi,data];
     qa=Quantile[Statistics`NormalDistribution`ChiSquareDistribution[1], 1-alpha]; 
     h=Sqrt[qa/info];
     la = like - 0.5*qa;
     If[showIter, Print["Iterating for lower limit ..."]];
     If[initLow<=0, m0=mhat-0.5*h, m0=initLow];
      For[i = 1, i <= maxIter, i++, 
        {like, score} = LDLoglikeScore[m0, phi, data]; 
         m1 = m0 - (like - la)/score;
         If[showIter, Print[m0]];
         If[Abs[(m1 - m0)/m0] <= 10^(-5), mL=m1; Break[], m0 = m1]];
     If[showIter, Print["Iterating for upper limit ..."]];
     If[initUp<=0, m0=mhat+0.5*h, m0=initUp];
      For[i = 1, i <= maxIter, i++, 
        {like, score} = LDLoglikeScore[m0, phi, data]; 
         m1 = m0 - (like - la)/score;
         If[showIter, Print[m0]];
         If[Abs[(m1 - m0)/m0] <= tol, mU=m1; Break[], m0 = m1]];
      {mL, mU}]


(* computing MLE of m using Newton-Raphson method, for LD model, 5/5/04 *)
newtonLD[data_List, opts___] := Module[
      {m0, m1, score, info, showIter, phi, tol, maxIter, initEst},
      showIter = ShowIterations /. {opts} /. Options[newtonLD]; 
      initEst = InitialM /. {opts} /. Options[newtonLD]; 
      maxIter = MaxIterations /. {opts} /. Options[newtonLD]; 
      phi = Phi /. {opts} /. Options[newtonLD]; 
      tol = Tolerance /. {opts} /. Options[newtonLD]; 
      If[initEst>0, m0=initEst,
         If[Median[data]==0, m0=MethodOfP0[data], m0=JonesMedian[data]]];
      If[showIter, Print[m0]]; 
      For[i = 1, i <= maxIter, i++, 
        {score, info} = LDScore$Info[m0, phi, data]; 
        m1 = m0 + score/info;
        If[Abs[(m1 - m0)/m0] < tol, Return[m1], 
         If[showIter, Print[m1]]; m0 = m1]];
      Print["no convergence!"];  ]

(* Newton-Raphson for MLE of the M(m,r) model, 5/13/2004 *)
newtonM[data_, opts___] := Module[
     {showIter, maxIter, tol, init4m, init4r, m0, r0, m1, r1, score, info, i}, 
     showIter = ShowIterations /. {opts} /. Options[newtonM]; 
     maxIter = MaxIterations /. {opts} /. Options[newtonM]; 
     tol = Tolerance /. {opts} /. Options[newtonM]; 
     init4m = InitialM /. {opts} /. Options[newtonM]; 
     init4r = InitialR /. {opts} /. Options[newtonM]; 
     If[init4m < 0, m0 = newtonLD[data], m0 = init4m]; 
     If[init4r < 0, r0 = 1, r0 = init4r];
     {score, info} = scoreInfoM[data, N[m0], N[r0]];
     For[i = 1, i <= maxIter, i++, 
       {m1, r1} = {m0, r0} + score . Inverse[info]; 
       If[Abs[(m1 - m0)/m0] + Abs[(r1 - r0)/r0] < tol, Return[{m1, r1}], 
         If[showIter, Print[{m0, r0}]]; m0 = m1; r0 = r1; 
          {score, info} = scoreInfoM[data, m1, r1]; ]; ]; 
      Print["no convergence"]]

(* confidence interval for m based on profile likelihood, for M(m,r) model *)
(* 5/15/2004 *)
profileIntervalM[data_, opts___] := Module[
     {showIter, initM, initR, initLowM, initUpM, initLowR, initUpR, tol,
      alpha, maxIter, m0, m1, r0, r1, la, mhat, rhat, score, info, like,
      lowCL, upCL, i, venLowM, venLowR, venUpM, venUpR}, 
     alpha = Alpha /. {opts} /. Options[profileIntervalM]; 
     showIter = ShowIterations /. {opts} /. Options[profileIntervalM]; 
     maxIter = MaxIterations /. {opts} /. Options[profileIntervalM]; 
     tol = Tolerance /. {opts} /. Options[profileIntervalM]; 
     initM = InitialM /. {opts} /. Options[profileIntervalM]; 
     initLowM = InitialLowerM /. {opts} /. Options[profileIntervalM]; 
     initUpM = InitialUpperM /. {opts} /. Options[profileIntervalM]; 
     initR = InitialR /. {opts} /. Options[profileIntervalM]; 
     initLowR = InitialLowerR /. {opts} /. Options[profileIntervalM]; 
     initUpR = InitialUpperR /. {opts} /. Options[profileIntervalM]; 
     {mhat, rhat} = newtonM[data, InitialM -> initM, InitialR -> initR, 
        Tolerance -> tol];
     la = logLikelihoodM[mhat, rhat, data] - 
        0.5*Quantile[Statistics`NormalDistribution`ChiSquareDistribution[1],
        1 - alpha]; 
     {venLowM, venLowR, venUpM, venUpR}=venzon4m[data,mhat,rhat,alpha];
     If[initLowM < 0, m0 = venLowM, m0 = initLowM]; 
     If[initLowR < 0, r0 = venLowR, r0 = initLowR]; 
     If[showIter, Print["Iterating for lower limit ..."]]; 
     {score, info} = scoreInfoM[data, m0, r0]; 
     like = logLikelihoodM[m0, r0, data];
     For[i = 1, i <= maxIter, i++, 
       {m1, r1} = {m0, r0} + Inverse[{-score, {info[[1,2]], info[[2,2]]}}] . 
           {like - la, score[[2]]}; If[showIter, Print[{m0, r0}]]; 
       {score, info} = scoreInfoM[data, m1, r1];
       If[Abs[(m0 - m1)/m0] < tol, lowCL = m1; Break[]];
       like = logLikelihoodM[m1, r1, data]; m0 = m1; r0 = r1; ];
     (* upper confidence limit *)
      If[initUpM < 0, m0 = venUpM, m0 = initUpM]; 
      If[initUpR < 0, r0 = venUpR, r0 = initUpR]; 
      If[showIter, Print["Iterating for upper limit ..."]]; 
      {score, info} = scoreInfoM[data, m0, r0]; 
      like = logLikelihoodM[m0, r0, data];
      For[i = 1, i <= maxIter, i++, 
       {m1, r1} = {m0, r0} + Inverse[{-score, {info[[1,2]], info[[2,2]]}}] . 
           {like - la, score[[2]]};
       If[showIter, Print[{m0, r0}]]; 
       {score, info} = scoreInfoM[data, m1, r1];
       If[Abs[(m0 - m1)/m0] < tol, upCL = m1; Break[]];
       like = logLikelihoodM[m1, r1, data]; m0 = m1; r0 = r1; ];
     {lowCL, upCL}]

(* profile interval for r=1/rho under the M(m,r) model,  5/18/04 *)
profileIntervalR[data_, opts___] := Module[
     {showIter, initM, initR, initLowM, initUpM, initLowR, initUpR, tol,
      alpha, maxIter, m0, m1, r0, r1, la, mhat, rhat, score, info, like,
      lowCL, upCL, i, venLowM, venLowR, venUpM, venUpR}, 
     alpha = Alpha /. {opts} /. Options[profileIntervalR]; 
     showIter = ShowIterations /. {opts} /. Options[profileIntervalR]; 
     maxIter = MaxIterations /. {opts} /. Options[profileIntervalR]; 
     tol = Tolerance /. {opts} /. Options[profileIntervalR]; 
     initM = InitialM /. {opts} /. Options[profileIntervalR]; 
     initLowM = InitialLowerM /. {opts} /. Options[profileIntervalR]; 
     initUpM = InitialUpperM /. {opts} /. Options[profileIntervalR]; 
     initR = InitialR /. {opts} /. Options[profileIntervalR]; 
     initLowR = InitialLowerR /. {opts} /. Options[profileIntervalR]; 
     initUpR = InitialUpperR /. {opts} /. Options[profileIntervalR]; 
     {mhat, rhat} = newtonM[data, InitialM -> initM, InitialR -> initR, 
        Tolerance -> tol];
     la = logLikelihoodM[mhat, rhat, data] - 
        0.5*Quantile[Statistics`NormalDistribution`ChiSquareDistribution[1],
        1 - alpha]; 
     {venLowM, venLowR, venUpM, venUpR}=venzon4r[data,mhat,rhat,alpha];
     If[initLowM < 0, m0 = venLowM, m0 = initLowM]; 
     If[initLowR < 0, r0 = venLowR, r0 = initLowR]; 
     If[showIter, Print["Iterating for lower limit ..."]]; 
     {score, info} = scoreInfoM[data, m0, r0]; 
     like = logLikelihoodM[m0, r0, data];
     For[i = 1, i <= maxIter, i++, 
       {m1, r1} = {m0, r0} + Inverse[{-score, {info[[1,1]], info[[1,2]]}}] . 
           {like - la, score[[1]]}; If[showIter, Print[{m0, r0}]]; 
       {score, info} = scoreInfoM[data, m1, r1];
       If[Abs[(r0 - r1)/r0] < tol, lowCL = r1; Break[]];
       like = logLikelihoodM[m1, r1, data]; m0 = m1; r0 = r1; ];
     (* upper confidence limit *)
      If[initUpM < 0, m0 = venUpM, m0 = initUpM]; 
      If[initUpR < 0, r0 = venUpR, r0 = initUpR]; 
      If[showIter, Print["Iterating for upper limit ..."]]; 
      {score, info} = scoreInfoM[data, m0, r0]; 
      like = logLikelihoodM[m0, r0, data];
      For[i = 1, i <= maxIter, i++, 
       {m1, r1} = {m0, r0} + Inverse[{-score, {info[[1,1]], info[[1,2]]}}] . 
           {like - la, score[[1]]};
       If[showIter, Print[{m0, r0}]]; 
       {score, info} = scoreInfoM[data, m1, r1];
       If[Abs[(r0 - r1)/m0] < tol, upCL = r1; Break[]];
       like = logLikelihoodM[m1, r1, data]; m0 = m1; r0 = r1; ];
     {lowCL, upCL}]

(* expected Fisher Information matrix for the M(m,r) model, 5/17/2004 *)
FisherInfoMatrix[m_?NumericQ, r_?NumericQ, n_] := Module[
     {B, eta, h10, h01, h02, p, p10, p01, I11, I12, I22},
     B = N[Table[(j - 1)/(j + r), {j, 2, n}]]; 
     B = FoldList[Times, 1, B]/(1 + r); eta = Table[1/(j + r), {j, 1, n}]; 
     eta = FoldList[Plus, First[eta], Rest[eta]];
     h10 = Prepend[r*B, -1]; 
     h01 = Prepend[B*(1 - r*eta), 0];
     p = $luria`$kochstew[N[m], N[r], n]; 
     p10 = $luria`$pdfconv[p, h10];
     p01 = m*$luria`$pdfconv[p, h01]; 
      I11 = Plus@@ (p10^2/p);
      I12 = Plus@@ (p10*p01/p);
      I22 = Plus@@ (p01^2/p);
      {{I11, I12}, {I12, I22}}]
 
(* ============== internal functions ============== *)
LDLikelihood[m_?NumericQ,phi_,data_List] := Module[{prob, likelihood}, 
     prob = $luria`$pdfluria[N[m], N[phi], Max[data]];
     likelihood = (-Log[prob[[#1 + 1]]] & ) /@ data;
     Plus @@ likelihood]

(* loglikelihood and score,  5-3-04 *)
LDLoglikeScore[m_?NumericQ,phi_,data_List] := Module[
    {n, h, p, p1, loglike, score}, 
     n= Max[data];
     p = $luria`$pdfluria[N[m], N[phi], n];
     loglike = Log[p[[#1 + 1]]] &  /@ data;
     h = Table[phi^(k-1)*(1/k - phi/(k + 1)), {k, 1, n}];
     h = Prepend[h, -1]; 
     p1 = $luria`$pdfconv[p, N[h]];
     score = ((p1/p)[[#1 + 1]])& /@ data;
    {Plus @@ loglike, Plus @@ score}
    ]

(* Rao's score U and observed Fisher information J for LD model, 5-5-04 *) 
LDScore$Info[m_?NumericQ, phi_, data_List] := Module[
      {n, h, p, p1, p2, info, score},
      n = Max[data]; 
      p = $luria`$pdfluria[N[m], N[phi], n]; 
      h = Table[phi^(k - 1)*(1/k - phi/(k + 1)), {k, 1, n}]; 
      h = Prepend[h, -1];
      p1 = $luria`$pdfconv[p, N[h]];
      p2 = $luria`$pdfconv[p1, N[h]]; 
      score = ((p1/p)[[#1 + 1]] & ) /@ data; 
      info = (((p1/p)^2 - p2/p)[[#1 + 1]] & ) /@ data; 
      {Plus @@ score, Plus @@ info}]

(* all 1st and 2nd derivatives of the log likelihood function for M(m,r) model *)
(* 5/13/2004 *)
partials4M[m_, r_, n_] := Module[
     {B, eta, h10, h01, h02, p, p10, p01, p02, p20, p11},
     B = Table[(j - 1)/(j + r), {j, 2, n}]; 
     B = FoldList[Times, 1, B]/(1 + r); eta = Table[1/(j + r), {j, 1, n}]; 
     eta = FoldList[Plus, First[eta], Rest[eta]];
     h10 = Prepend[r*B, -1]; 
     h01 = Prepend[B*(1 - r*eta), 0];
     p = $luria`$kochstew[N[m], N[r], n]; 
     p10 = $luria`$pdfconv[p, h10];
     p01 = m*$luria`$pdfconv[p, h01]; 
     h02 = Table[j/(j + r)^2, {j, 1, n}];
     h02 = FoldList[Plus, First[h02], Rest[h02]];
     h02 = B*(r*eta*eta - eta - h02);
     h02 = Prepend[h02, 0]; 
     p02 = m*($luria`$pdfconv[h02, p] + $luria`$pdfconv[h01, p01]); 
     p20 = $luria`$pdfconv[h10, p10];
     p11 = (1*p01)/m + m*$luria`$pdfconv[h01, p10]; 
     {p10, p01, p20, p11, p02, p}]
 
(* score and observed information matrix for the M(m,r)  model, 5/13/04 *)  
scoreInfoM[data_List, m_, r_] := Module[
     {U1, U2, J11, J12, J22, p10, p01, p20, p11, p02, p, n},
      n = Max[data];
      {p10, p01, p20, p11, p02, p} = partials4M[m, r, n];
      U1 = ((p10/p)[[#1 + 1]] & ) /@ data; 
      U1 = Plus @@ U1;
      U2 = ((p01/p)[[#1 + 1]] & ) /@ data;
      U2 = Plus @@ U2; 
      J11 = (((p10/p)^2 - p20/p)[[#1 + 1]] & ) /@ data;
      J11 = Plus @@ J11; 
      J12 = (((p10*p01)/p^2 - p11/p)[[#1 + 1]] & ) /@ data; 
      J12 = Plus @@ J12;
      J22 = (((p01/p)^2 - p02/p)[[#1 + 1]] & ) /@ data; 
      J22 = Plus @@ J22;
      {{U1, U2}, {{J11, J12}, {J12, J22}}}]
 
logLikelihoodM[m_?NumericQ,r_,data_List] := Module[{prob, likelihood}, 
     prob = pmfM[N[m], N[r], Max[data]];
     likelihood = (Log[prob[[#1 + 1]]] & ) /@ data;
     Plus @@ likelihood]  (* adapted from MLikelihood, 5/14/04 *)

(* find initial estimate of confidence limits for m and r according to Venzon *)
(* output give a list of the form {lower m, lower r, upper m, upper r} *) 
(* see Venzon and Moolgavkar 1988, 5/15/04 *)
venzon4m[data_, mhat_, rhat_, alpha_:0.05] := Module[{U, J, q, h}, 
     {U, J} = scoreInfoM[data, mhat, rhat]; 
   q = Quantile[Statistics`NormalDistribution`ChiSquareDistribution[1], 1 - alpha]; 
      h = J[[1,1]] - J[[1,2]]^2/J[[2,2]];
      h = 0.5*Sqrt[q/h]; 
      {mhat - h, rhat + (h*J[[1,2]])/J[[2,2]], mhat + h, 
       rhat - (h*J[[1,2]])/J[[2,2]]} ]

venzon4r[data_, mhat_, rhat_, alpha_:0.05] := Module[{U, J, q, h}, 
     {U, J} = scoreInfoM[data, mhat, rhat]; 
   q = Quantile[Statistics`NormalDistribution`ChiSquareDistribution[1], 1 - alpha]; 
      h = J[[2,2]] - J[[1,2]]^2/J[[1,1]];
      h = 0.5*Sqrt[q/h]; 
      {mhat  + (h*J[[1,2]])/J[[1,1]], rhat-h,
       mhat  - (h*J[[1,2]])/J[[1,1]], rhat+h}]


(* -log of likelihood function using the Lea-Coulson normal approximation *)
leaLike[m_, data_] := Plus@@((11.6/(#1/m - Log[m] + 4.5) - 2.02)&/@data)

End[]  
EndPackage[]  
  
