/* file name: salvador.tm */
/* Author: Qi Zheng, National Center for Toxicological Research */
/* Starting date: May 8, 2000 */
/* Usage: for fluctuation analysis, to be called by salvador.m through MathLink */
/* July 12, 2001 replacing MLPutSymbol with MLPutFunction, for Abort.
/* Minor changes: October 23, 2001 and November 9, 2001 */
/* SALVADOR v.2 started May 14, 2004 at School of Rural Public Health, Bryan, TX */
/* compile: mcc salvador.tm -o salvador -lm */
/* mprep salvador.tm -o salvador.c   .... wcl386 salvador.c ml32i2m.lib */

#include <math.h>
#include "mathlink.h"

:Evaluate: BeginPackage["$luria`"]
:Begin:
:Function: pdfluria
:Pattern: $pdfluria[m_Real,phi_Real,k_Integer]
:Arguments: {m,phi,k}
:ArgumentTypes: {Real,Real,Integer}
:ReturnType: Manual 
:End:

:Evaluate: $pdfluria::usage="$pdfluria[m,phi,k] computes an LD(m,phi) dist."

/* revised October 30, 2001, m is the mean number of mutations, not theta */ 
void pdfluria(double m, double phi, int k){
double *problist, cumuphi;
int n,j;
problist=(double*) malloc(sizeof(double)*(k+1));
for(n=1;n<=k;n++) problist[n]=0;
problist[0]=exp(-m); 
for(n=1;n<=k;n++) {
  if (MLAbort){MLPutFunction(stdlink,"Abort",0);return;};
   cumuphi=1.0;
   for(j=1;j<=n;j++){
      if (MLAbort){MLPutFunction(stdlink,"Abort",0);return;};
      problist[n]=problist[n]+cumuphi*(1-j*phi/(j+1))*problist[n-j];
      cumuphi=cumuphi*phi;}
   problist[n]=problist[n]*m/n;};
MLPutRealList(stdlink,problist,k+1);
free(problist);
}


/* April 14, 2001: computing the M(m,r,1) distribution using a recursive */
/* algorithm as given in Stewart et al. */

:Begin: 
:Function: kochstew 
:Pattern: $kochstew[m_Real, r_Real, k_Integer] 
:Arguments: {m,r,k} 
:ArgumentTypes: {Real, Real, Integer} 
:ReturnType: Manual  
:End: 
 
:Evaluate: $kochstew::usage="$kochstew[m,r,k] is a MathLink function."

/* R is redefined to be b1/b2, the reciprocal of rho, 5-14-04 */
void kochstew(double m, double R, int k){ 
double *q, *p;
int n,j;
q=(double*) malloc(sizeof(double)*(k+1)); 
p=(double*) malloc(sizeof(double)*(k+1)); 

q[1]=m*R/(1+R);
for(j=2;j<=k;j++){
if (MLAbort){MLPutFunction(stdlink,"Abort",0);return;};
q[j]=(j-1)/(j+R)*q[j-1];}

for(n=1;n<=k;n++){
  if (MLAbort){MLPutFunction(stdlink,"Abort",0);return;};
  p[n]=0;}

p[0]=exp(-m);  

for(n=1;n<=k;n++) { 
if (MLAbort){MLPutFunction(stdlink,"Abort",0);return;};
   for(j=1;j<=n;j++){ 
     if (MLAbort){MLPutFunction(stdlink,"Abort",0);return;};
     p[n]+=j*q[j]*p[n-j];} 
     p[n]=p[n]/n;}; 
MLPutRealList(stdlink,p,k+1); 
free(q);
free(p); 
} 

/* April 16, 2001: I(m) with approximate M distribution */
/* computing the Fisher information for m using approximate form of M dist */

:Begin: 
:Function: kochinfo 
:Pattern: $kochinfo[m_Real, r_Real, k_Integer] 
:Arguments: {m,r,k} 
:ArgumentTypes: {Real, Real, Integer} 
:ReturnType: Real  
:End: 
 
:Evaluate: $kochinfo::usage="$kochinfo[m,r,k] is a MathLink function."

double kochinfo(double m, double r, int k){ 
double *l, *p, *f, R, fisher;
int n,i,j;
l=(double*) malloc(sizeof(double)*(k+1)); 
p=(double*) malloc(sizeof(double)*(k+1)); 
f=(double*) malloc(sizeof(double)*(k+1)); 

R=1/r;
l[0]=-1;
l[1]=R/(1+R);
for(j=2;j<=k;j++){
l[j]=(j-1)/(j+R)*l[j-1];}

for(n=1;n<=k;n++) p[n]=0; 

p[0]=exp(-m);  

for(n=1;n<=k;n++) { 
if (MLAbort){MLPutFunction(stdlink,"Abort",0);return -1;};
   for(j=1;j<=n;j++){ 
      p[n]+=j*l[j]*p[n-j];} 
   p[n]=p[n]*m/n;}; 

/* the f_k sequence */
for (i=0;i<=k;i++) f[i]=0;
for(j=1;j<=k+1;j++){
for(i=1;i<=j;i++){
if (MLAbort){MLPutFunction(stdlink,"Abort",0);return -1;};
f[j-1]+=l[i-1]*p[j-i];};
};

fisher=0;
for(j=0;j<=k;j++) fisher+=f[j]*f[j]/p[j];
return fisher;
free(f); 
free(l);
free(p); 
} 


/* April 17, 2001: finding I(rho) using Zheng's method */

:Begin: 
:Function: iofrho 
:Pattern: $iofrho[m_Real, r_Real, k_Integer] 
:Arguments: {m,r,k} 
:ArgumentTypes: {Real, Real, Integer} 
:ReturnType: Real  
:End: 
 
:Evaluate: $iofrho::usage="$iofrho[m,r,k] is a MathLink function."

double iofrho(double m, double r, int k){ 

double *b, *l, *p, *f, A, R, a, fisher;
int i,j,n;

b=(double*) malloc(sizeof(double)*(k+1)); 
l=(double*) malloc(sizeof(double)*(k+1)); 
p=(double*) malloc(sizeof(double)*(k+1)); 
f=(double*) malloc(sizeof(double)*(k+1)); 

R=1/r;
A=1/(1+R);
a=1/(1+R);
b[0]=0;
b[1]=m/(1+R)/r/r;
l[0]=0;
l[1]=(a/r-1)*b[1];

for(i=2;i<=k;i++){
 if (MLAbort){MLPutFunction(stdlink,"Abort",0);return -1;}; 
A=1/(i+R);
a+=A;
b[i]=b[i-1]*(i-1)*A;
l[i]=(a/r-1)*b[i];
}

for(n=1;n<=k;n++) p[n]=0; 

p[0]=exp(-m);  

for(n=1;n<=k;n++) { 
 if (MLAbort){MLPutFunction(stdlink,"Abort",0);return -1;}; 
   for(j=1;j<=n;j++){ 
      p[n]+=j*b[j]*p[n-j];} 
   p[n]=p[n]*r/n;}; 

/* the f_k sequence */
for (i=0;i<=k;i++) f[i]=0;

for(j=1;j<=k;j++){
for(i=1;i<=j;i++){
 if (MLAbort){MLPutFunction(stdlink,"Abort",0);return -1;}; 
f[j]+=l[i]*p[j-i];};
};

fisher=0;
for(j=0;j<=k;j++) fisher+=f[j]*f[j]/p[j];
return fisher; 

free(b);
free(f); 
free(l);
free(p); 
} 
 

/* March 6, 2001: to convolute 2 lists of PDFs */
 
:Begin: 
:Function: pdfconv 
:Pattern: $pdfconv[x_List,y_List]
:Arguments: {x,y} 
:ArgumentTypes: {RealList,RealList} 
:ReturnType: Manual  
:End: 
 
:Evaluate: $pdfconv::usage="$pdfconv[x,y] is a MathLink function."
  
void pdfconv(double* x, long xlen, double* y, long ylen){ 
double *z; 
int i, k; 
z=(double*) malloc(sizeof(double)*xlen); 
for (i=0;i<=xlen-1;i++) z[i]=0;
for(k=1;k<=xlen;k++){
for(i=1;i<=k;i++){
if (MLAbort){MLPutFunction(stdlink,"Abort",0);return;};
z[k-1]+=x[i-1]*y[k-i];};
};
MLPutRealList(stdlink,z,xlen); 
free(z); 
} 


/* March 6, 2001: Poisson sum distribution */
 
:Begin: 
:Function: poisum 
:Pattern: $poisum[q_List] 
:Arguments: {q} 
:ArgumentTypes: {RealList} 
:ReturnType: Manual  
:End: 
 
:Evaluate: $poisum::usage="$poisum[q] is a MathLink function."
  
void poisum(double* q, long qlen){ 
double *p;
int n,j; 
p=(double*) malloc(sizeof(double)*qlen); 
for(n=1;n<=qlen-1;n++) p[n]=0; 
p[0]=exp(q[0]);  
for(n=1;n<=qlen-1;n++) { 
if (MLAbort){MLPutFunction(stdlink,"Abort",0);return;};
   for(j=1;j<=n;j++){ 
      p[n]+=j*q[j]*p[n-j];} 
   p[n]=p[n]/n;}; 
MLPutRealList(stdlink,p,qlen); 
free(p); 
} 
 
/* March 13, 2001: to compute Fisher's Information */
/* Luria-Delbruck distribution, Lea-Coulson method, for I(m) */ 
:Begin: 
:Function: fishinfo
:Pattern: $fishinfo[m_Real,n_Integer]
:Arguments: {m,n} 
:ArgumentTypes: {Real,Integer} 
:ReturnType: Real  
:End: 
 
:Evaluate: $fishinfo::usage="$fishinfo[m,n] is a MathLink function."
  
double fishinfo(double m, int n){ 
int i,j,k;
double A, info;
double *p, *t;
double **z;  /* to save memory, define z[i][j]=c[j][i] */
/* a lower triangular matrix z is formed in place of a square matrix */

t=(double *) calloc(n,sizeof(double));
p=(double *) calloc(n,sizeof(double));
z=(double **) calloc(n,sizeof(double *));

for (i=1;i<=n;i++){
z[i-1]=(double *) calloc(i, sizeof(double));
}

for (i=1;i<=n;i++){
z[i-1][0]=1.0/i/(i+1); }

/* z[0][0]=1/2.0; */
for (i=2;i<=n;i++) {
z[i-1][i-1]=z[i-2][i-2]/2.0; }

for (j=2;j<=n;j++) {
  for (k=j+1;k<=n;k++) {
   z[k-1][j-1]=(j*z[k-2][j-2]+(k-1)*z[k-2][j-1])/(j+k);
  }
}

for (i=1;i<=n;i++) {
A=1;
for (j=1;j<=i;j++) {
  p[i-1]+=z[i-1][j-1]*A*m/j;
  t[i-1]+=z[i-1][j-1]*A;
  A*=m/j;}
}

/* computing Fisher's information */
info=1;  /* because t_0=0 */
for (i=1;i<=n;i++){
info+=(t[i-1]-p[i-1])*(t[i-1]-p[i-1])/p[i-1];}

for (i=1;i<=n;i++) {free(z[i-1]);}
free(z);
free(p);
free(t);
return exp(-m)*info;
}

/* computes exp(m)*{p1, p2, ..., pn} using the Koch algorithm */
/* m stands for the mean number of mutations on each sister culture */
/* r stands for rho, which is the ratio of b2 to b1 */
/* March 13, 2001 */
:Begin: 
:Function: pdfkoch
:Pattern: $pdfkoch[m_Real, r_Real,n_Integer]
:Arguments: {m,r,n} 
:ArgumentTypes: {Real,Real,Integer} 
:ReturnType: Manual
:End: 
 
:Evaluate: $pdfkoch::usage="$pdfkoch[m,r,n] is a MathLink function."
  
void pdfkoch(double m, double r, int n){ 
int i,j,k;
double A;
double *p;
double **z;  /* to save memory, define z[i][j]=c[j][i] */

p=(double *) calloc(n,sizeof(double));
z=(double **) calloc(n,sizeof(double *));

for (i=1;i<=n;i++){
z[i-1]=(double *) calloc(i, sizeof(double));
}

z[0][0]=1/(1+r);
for (i=2;i<=n;i++){
z[i-1][0]=z[i-2][0]*(i-1)*r/(1+i*r); }

for (i=2;i<=n;i++) {
z[i-1][i-1]=z[i-2][i-2]/(1+r); }

for (j=2;j<=n;j++) {
  for (k=j+1;k<=n;k++) {
   z[k-1][j-1]=(j*z[k-2][j-2]+(k-1)*r*z[k-2][j-1])/(j+k*r);
  }
}

for (i=1;i<=n;i++) {
A=1;
for (j=1;j<=i;j++) {
  A*=m/j;
  p[i-1]+=z[i-1][j-1]*A;}
}

MLPutRealList(stdlink,p,n);
for (i=1;i<=n;i++) {free(z[i-1]);}
free(z);
free(p);
}


/* Lea-Coulson method as modified by Zheng, for I(rho) */ 
:Begin: 
:Function: fisherho
:Pattern: $fisherho[m_Real, r_Real,n_Integer]
:Arguments: {m,r,n} 
:ArgumentTypes: {Real,Real,Integer} 
:ReturnType: Manual
:End: 
 
:Evaluate: $fisherho::usage="$fisherho[m,r,n] computes Fisher's info for rho."
  
void fisherho(double m, double r, int n){ 
int i,j,k;
double A, infoM, infoRho;
double info[2]={0,0};
double *p, *t, *s;
double **z, **d;  /* to save memory, define z[i][j]=c[j][i] */

p=(double *) calloc(n,sizeof(double));
t=(double *) calloc(n,sizeof(double));
s=(double *) calloc(n,sizeof(double));
z=(double **) calloc(n,sizeof(double *));
d=(double **) calloc(n,sizeof(double *));


for (i=1;i<=n;i++){
z[i-1]=(double *) calloc(i, sizeof(double));
d[i-1]=(double *) calloc(i, sizeof(double));
}

z[0][0]=1/(1+r);
d[0][0]=-1/(1+r)/(1+r);
for (i=2;i<=n;i++){
z[i-1][0]=z[i-2][0]*(i-1)*r/(1+i*r);
d[i-1][0]=(i-1)/(1+i*r)/(1+i*r)*z[i-2][0]+(i-1)*r/(1+i*r)*d[i-2][0];
 }

/* computing C[i][i] */
for (i=2;i<=n;i++) {
z[i-1][i-1]=z[i-2][i-2]/(1+r); }

/* computing D[i][i] */
A=1/(1+r)/(1+r);
for(i=2;i<=n;i++) {
A=A/(1+r);
d[i-1][i-1]=-i*A; }

/* computing C[i][j] */
for (j=2;j<=n;j++) {
  for (k=j+1;k<=n;k++) {
   z[k-1][j-1]=(j*z[k-2][j-2]+(k-1)*r*z[k-2][j-1])/(j+k*r);
  }
}

/* computing D[i][j] */
for (j=2;j<=n;j++) {
  for (k=j+1;k<=n;k++) {
   d[k-1][j-1]=(-k*j*z[k-2][j-2]/(j+k*r)+j*d[k-2][j-2]+\
    (k-1)*j/(j+k*r)*z[k-2][j-1]+(k-1)*r*d[k-2][j-1])/(j+k*r);
  }
}


/* computing p[j], t[j] and s[j] */
for (i=1;i<=n;i++) {
A=1;
for (j=1;j<=i;j++) {
  p[i-1]+=z[i-1][j-1]*A*m/j;
  s[i-1]+=d[i-1][j-1]*A*m/j;
  t[i-1]+=z[i-1][j-1]*A;
  A*=m/j;}
}

/* computing Fisher's information */
infoM=1;
infoRho=0;
for (i=1;i<=n;i++){
infoM+=(t[i-1]-p[i-1])*(t[i-1]-p[i-1])/p[i-1];
infoRho+=s[i-1]*s[i-1]/p[i-1]; }

info[0]=exp(-m)*infoM;
info[1]=exp(-m)*infoRho;

MLPutRealList(stdlink,info,2);

for (i=1;i<=n;i++) {free(z[i-1]); free(d[i-1]);}
free(z);
free(p);
free(t);
free(s);
}

:Evaluate: EndPackage[]
int main(int argc, char* argv[]){ 
return MLMain(argc,argv); } 
 
 
