(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    205546,       8754]*)
(*NotebookOutlinePosition[    209380,       8857]*)
(*  CellTagsIndexPosition[    208542,       8834]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["SALVADOR 2.0 Manual", "Title",
  FontSize->16],

Cell["\<\
Qi Zheng
Department of Epidemiology and Biostatistics
School of Rural Public Health,  Bryan, Texas 77802
qzheng@srph.tamhsc.edu
February, 2005\
\>", "Text"],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
  "SALVADOR  is a package intended as a tool for exploring  various \
quantitative aspects of fluctuation experiments. For reasons of computational \
efficiency, the package was written in a mixture of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and the C language. Therefore, before using SALVADOR, two programs must \
be install from within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Install["\<salvadorV2\>"]\)], "Input"],

Cell[BoxData[
    \(LinkObject["salvadorV2", 2, 2]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(<< salvadorV2.m\)], "Input"],

Cell["\<\
(To avoid being distracted by warning messages against use of similar \
variable names, e.g., var1 and var2, we suppress such warning messages.)\
\>", "Text"],

Cell["Off[General::spell1]", "Input"],

Cell["\<\
For notation, nomenclature and other background information, the user can \
consult Zheng(1999,2002,2005) and numerous additional references contained \
therein. The sole purpose of this manual is to demonstrate the various \
functionalities of SALVADOR 2.0.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simulation", "Section",
  FontSize->16],

Cell[TextData[{
  "Five basic parameters must be specified before you simulate the number of \
mutants in a  culture. These five parameters are: 1) mutation rate (per cell \
per unit time) \[Mu]; 2) cellular  birth rate for nonmutants ",
  Cell[BoxData[
      \(TraditionalForm\`\[Beta]\_1\)]],
  "; 3) cellular birth rate for mutants ",
  Cell[BoxData[
      \(TraditionalForm\`\[Beta]\_2\)]],
  "; 4) initial population size ",
  Cell[BoxData[
      \(TraditionalForm\`N\_0\)]],
  "; and 5) final population size ",
  Cell[BoxData[
      \(TraditionalForm\`N\_T\)]],
  ". It is strongly recommended that, before running a simulation,  you check \
the parameters of interest by computing the mean and variance of the number \
of mutants, for unrealistic parameters can lead to very time-consuming \
computational tasks and useless results. We now use the following five \
parameters to simulate a fluctuation experiment ."
}], "Text"],

Cell[BoxData[
    \(mu = 10^\(-8\); b1 = 1.2; b2 = 1.1; N0 = 100; Nt = 2.4*10^8;\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?MeanOfMutants\)\)], "Input"],

Cell[BoxData[
    \("MeanOfMutants[\!\(\[Mu],\[Beta]\_1,\[Beta]\_2,N\_0,N\_T\)] computes \
the mean number of mutants on each sister culture."\)], "Print",
  CellTags->"Info3317216010-4785616"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MeanOfMutants[mu, b1, b2, N0, Nt]\)], "Input"],

Cell[BoxData[
    \(16.94451339681247`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?VarOfMutants\)\)], "Input"],

Cell[BoxData[
    \("VarOfMutants[\!\(\[Mu],\[Beta]\_1,\[Beta]\_2,N\_0,N\_T\)] computes the \
variance of the number of mutants on each sister culture."\)], "Print",
  CellTags->"Info3317216011-1867243"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(VarOfMutants[mu, b1, b2, N0, Nt]\)], "Input"],

Cell[BoxData[
    \(995576.079641768`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?simuMutants\)\)], "Input"],

Cell[BoxData[
    \("simuMutants[\!\(\[Mu],\[Beta]\_1,\[Beta]\_2,\ N\_0,N\_T\)] simulates \
the number of mutants in a sister culture. Note \!\(\[Mu]\) is the mutation \
rate per cell per unit time; \!\(\[Beta]\_1\) is growth rate for nonmutants; \
\!\(\[Beta]\_2\) is growth rate of mutants; \!\(N\_0\) is the initial \
population size; \!\(N\_T\) is the population size prior to plating."\)], \
"Print",
  CellTags->"Info3317216011-7491825"]
}, Open  ]],

Cell["\<\
Now we simulate the numbers of mutants in 20 sister cultures.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[simuMutants[mu, b1, b2, N0, Nt], {20}]\)], "Input"],

Cell[BoxData[
    \({9, 6, 12, 2, 89, 20, 9, 2, 5, 6, 1, 3, 3, 7, 6, 3, 65, 5, 5, 
      1}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Important Distributions", "Section",
  FontSize->16],

Cell[TextData[{
  "SALVADOR can compute the probability mass function (p.m.f.) of four most \
important distributions in fluctuation analysis: the Poisson distribution, \
the LD or Luria-Delbruck  distribution, the M or Mandelbrot distribution, and \
the convolution of a Poisson and an LD distribution. The last distribution \
was  popularized in fluctuation analysis by Cairns et al. in 1988. Using \
pmf2cdf, you can transform a list of p.m.f. to a list of c.d.f. (cumulative \
distribution function).  As a demonstration we  first compute ",
  Cell[BoxData[
      \(TraditionalForm\`p\_1\)]],
  ", ..., ",
  Cell[BoxData[
      \(TraditionalForm\`p\_10\)]],
  " of a Poisson distribution having mean 10.5."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?pmfPoisson\)\)], "Input"],

Cell[BoxData[
    \("pmfPoisson[\!\(\[Lambda]\),k] returns a list of probabilities \
\!\({p\_0,p\_1,...,p\_k}\) according to a Poisson(\!\(\[Lambda]\)) \
distribution."\)], "Print",
  CellTags->"Info3317216011-4729632"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pmfPoisson[10.5, 10]\)], "Input"],

Cell[BoxData[
    \({0.000027536449349747158`, 0.00028913271817234516`, 
      0.001517946770404812`, 0.005312813696416842`, 0.013946135953094211`, 
      0.029286885501497845`, 0.051252049627621234`, 0.07687807444143184`, 
      0.1009024727043793`, 0.11771955148844249`, 
      0.12360552906286462`}\)], "Output"]
}, Open  ]],

Cell["\<\
Now we plot the c.d.f. of a Luria-Delbruck distribution, so its upper tail \
behavior can be more easily examined.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?pmfLD\)\)], "Input"],

Cell[BoxData[
    \("pmfLD[\!\(m,\[Phi],k\)] returns a list of probabilities \
\!\({p\_0,p\_1,...,\ p\_k}\) according to a Luria-Delbr\!\(\[UDoubleDot]\)uck \
distribution, LD(m,\!\(\[Phi]\)). Note pmfLD[m,k] is the same as \
pmfLD[m,1,k]."\)], "Print",
  CellTags->"Info3317216011-4887619"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(LDprob = pmfLD[5.7, 150]; \ LDprob[\([Range[10]]\)]\)], "Input"],

Cell[BoxData[
    \({0.003345965457471272`, 0.009536001553793125`, 0.016767469398752913`, 
      0.02355789717184977`, 0.02910035040826871`, 0.03312787122784504`, 
      0.03569563178469149`, 0.03701109059727024`, 0.037328993778887536`, 
      0.036895826091020184`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(LDcdf = pmf2cdf[LDprob]; \ LDcdf[\([Range[10]]\)]\)], "Input"],

Cell[BoxData[
    \({0.003345965457471272`, 0.012881967011264397`, 0.02964943641001731`, 
      0.05320733358186708`, 0.08230768399013579`, 0.11543555521798082`, 
      0.1511311870026723`, 0.18814227759994254`, 0.22547127137883008`, 
      0.26236709746985026`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[Transpose[{Range[0, 150], LDcdf}], PlotJoined \[Rule] True, 
      Frame \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00634921 0.0147151 0.616346 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.15079 -0.0125 -6 -9 ]
[.15079 -0.0125 6 0 ]
[.27778 -0.0125 -6 -9 ]
[.27778 -0.0125 6 0 ]
[.40476 -0.0125 -6 -9 ]
[.40476 -0.0125 6 0 ]
[.53175 -0.0125 -6 -9 ]
[.53175 -0.0125 6 0 ]
[.65873 -0.0125 -9 -9 ]
[.65873 -0.0125 9 0 ]
[.78571 -0.0125 -9 -9 ]
[.78571 -0.0125 9 0 ]
[.9127 -0.0125 -9 -9 ]
[.9127 -0.0125 9 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .13798 -18 -4.5 ]
[-0.0125 .13798 0 4.5 ]
[-0.0125 .26125 -18 -4.5 ]
[-0.0125 .26125 0 4.5 ]
[-0.0125 .38452 -18 -4.5 ]
[-0.0125 .38452 0 4.5 ]
[-0.0125 .50779 -18 -4.5 ]
[-0.0125 .50779 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.15079 0 m
.15079 .00625 L
s
[(20)] .15079 -0.0125 0 1 Mshowa
.27778 0 m
.27778 .00625 L
s
[(40)] .27778 -0.0125 0 1 Mshowa
.40476 0 m
.40476 .00625 L
s
[(60)] .40476 -0.0125 0 1 Mshowa
.53175 0 m
.53175 .00625 L
s
[(80)] .53175 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(100)] .65873 -0.0125 0 1 Mshowa
.78571 0 m
.78571 .00625 L
s
[(120)] .78571 -0.0125 0 1 Mshowa
.9127 0 m
.9127 .00625 L
s
[(140)] .9127 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.18254 0 m
.18254 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.34127 0 m
.34127 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.5 0 m
.5 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.81746 0 m
.81746 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .13798 m
.00625 .13798 L
s
[(0.2)] -0.0125 .13798 1 0 Mshowa
0 .26125 m
.00625 .26125 L
s
[(0.4)] -0.0125 .26125 1 0 Mshowa
0 .38452 m
.00625 .38452 L
s
[(0.6)] -0.0125 .38452 1 0 Mshowa
0 .50779 m
.00625 .50779 L
s
[(0.8)] -0.0125 .50779 1 0 Mshowa
.125 Mabswid
0 .04553 m
.00375 .04553 L
s
0 .07635 m
.00375 .07635 L
s
0 .10717 m
.00375 .10717 L
s
0 .1688 m
.00375 .1688 L
s
0 .19962 m
.00375 .19962 L
s
0 .23044 m
.00375 .23044 L
s
0 .29207 m
.00375 .29207 L
s
0 .32289 m
.00375 .32289 L
s
0 .35371 m
.00375 .35371 L
s
0 .41534 m
.00375 .41534 L
s
0 .44616 m
.00375 .44616 L
s
0 .47697 m
.00375 .47697 L
s
0 .53861 m
.00375 .53861 L
s
0 .56943 m
.00375 .56943 L
s
0 .60024 m
.00375 .60024 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.15079 .61178 m
.15079 .61803 L
s
.27778 .61178 m
.27778 .61803 L
s
.40476 .61178 m
.40476 .61803 L
s
.53175 .61178 m
.53175 .61803 L
s
.65873 .61178 m
.65873 .61803 L
s
.78571 .61178 m
.78571 .61803 L
s
.9127 .61178 m
.9127 .61803 L
s
.125 Mabswid
.05556 .61428 m
.05556 .61803 L
s
.0873 .61428 m
.0873 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.18254 .61428 m
.18254 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.24603 .61428 m
.24603 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.34127 .61428 m
.34127 .61803 L
s
.37302 .61428 m
.37302 .61803 L
s
.43651 .61428 m
.43651 .61803 L
s
.46825 .61428 m
.46825 .61803 L
s
.5 .61428 m
.5 .61803 L
s
.56349 .61428 m
.56349 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.62698 .61428 m
.62698 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.72222 .61428 m
.72222 .61803 L
s
.75397 .61428 m
.75397 .61803 L
s
.81746 .61428 m
.81746 .61803 L
s
.84921 .61428 m
.84921 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.94444 .61428 m
.94444 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .13798 m
1 .13798 L
s
.99375 .26125 m
1 .26125 L
s
.99375 .38452 m
1 .38452 L
s
.99375 .50779 m
1 .50779 L
s
.125 Mabswid
.99625 .04553 m
1 .04553 L
s
.99625 .07635 m
1 .07635 L
s
.99625 .10717 m
1 .10717 L
s
.99625 .1688 m
1 .1688 L
s
.99625 .19962 m
1 .19962 L
s
.99625 .23044 m
1 .23044 L
s
.99625 .29207 m
1 .29207 L
s
.99625 .32289 m
1 .32289 L
s
.99625 .35371 m
1 .35371 L
s
.99625 .41534 m
1 .41534 L
s
.99625 .44616 m
1 .44616 L
s
.99625 .47697 m
1 .47697 L
s
.99625 .53861 m
1 .53861 L
s
.99625 .56943 m
1 .56943 L
s
.99625 .60024 m
1 .60024 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01678 m
.03016 .02265 L
.03651 .03299 L
.04286 .04751 L
.04921 .06545 L
.05556 .08586 L
.0619 .10786 L
.06825 .13068 L
.0746 .15368 L
.08095 .17642 L
.0873 .19857 L
.09365 .21989 L
.1 .24024 L
.10635 .25956 L
.1127 .2778 L
.11905 .29498 L
.1254 .3111 L
.13175 .32622 L
.1381 .34037 L
.14444 .35361 L
.15079 .366 L
.15714 .37759 L
.16349 .38843 L
.16984 .39858 L
.17619 .40809 L
.18254 .417 L
.18889 .42536 L
.19524 .43321 L
.20159 .44059 L
.20794 .44753 L
.21429 .45406 L
.22063 .46021 L
.22698 .46602 L
.23333 .4715 L
.23968 .47668 L
.24603 .48158 L
.25238 .48622 L
.25873 .49062 L
.26508 .49479 L
.27143 .49876 L
.27778 .50252 L
.28413 .50611 L
.29048 .50952 L
.29683 .51277 L
.30317 .51587 L
.30952 .51882 L
.31587 .52165 L
.32222 .52435 L
.32857 .52694 L
.33492 .52941 L
Mistroke
.34127 .53178 L
.34762 .53405 L
.35397 .53623 L
.36032 .53833 L
.36667 .54034 L
.37302 .54227 L
.37937 .54413 L
.38571 .54592 L
.39206 .54764 L
.39841 .5493 L
.40476 .5509 L
.41111 .55244 L
.41746 .55393 L
.42381 .55537 L
.43016 .55676 L
.43651 .5581 L
.44286 .5594 L
.44921 .56066 L
.45556 .56187 L
.4619 .56305 L
.46825 .56419 L
.4746 .56529 L
.48095 .56637 L
.4873 .56741 L
.49365 .56841 L
.5 .56939 L
.50635 .57034 L
.5127 .57127 L
.51905 .57216 L
.5254 .57304 L
.53175 .57388 L
.5381 .57471 L
.54444 .57551 L
.55079 .57629 L
.55714 .57706 L
.56349 .5778 L
.56984 .57852 L
.57619 .57922 L
.58254 .57991 L
.58889 .58058 L
.59524 .58123 L
.60159 .58187 L
.60794 .58249 L
.61429 .5831 L
.62063 .58369 L
.62698 .58427 L
.63333 .58483 L
.63968 .58538 L
.64603 .58592 L
.65238 .58645 L
Mistroke
.65873 .58697 L
.66508 .58747 L
.67143 .58797 L
.67778 .58845 L
.68413 .58892 L
.69048 .58939 L
.69683 .58984 L
.70317 .59028 L
.70952 .59072 L
.71587 .59114 L
.72222 .59156 L
.72857 .59197 L
.73492 .59237 L
.74127 .59276 L
.74762 .59315 L
.75397 .59353 L
.76032 .5939 L
.76667 .59426 L
.77302 .59462 L
.77937 .59497 L
.78571 .59532 L
.79206 .59565 L
.79841 .59598 L
.80476 .59631 L
.81111 .59663 L
.81746 .59694 L
.82381 .59725 L
.83016 .59756 L
.83651 .59786 L
.84286 .59815 L
.84921 .59844 L
.85556 .59872 L
.8619 .599 L
.86825 .59927 L
.8746 .59954 L
.88095 .59981 L
.8873 .60007 L
.89365 .60032 L
.9 .60058 L
.90635 .60082 L
.9127 .60107 L
.91905 .60131 L
.9254 .60155 L
.93175 .60178 L
.9381 .60201 L
.94444 .60224 L
.95079 .60246 L
.95714 .60268 L
.96349 .6029 L
.96984 .60311 L
Mistroke
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{62, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`006P3oool2000001`0oooo100000030?ooo`8000006`3oool300000080oooo0P00000H0?oo
o`<00000103oool2000001X0oooo0P0000040?ooo`8000005P3oool500000080oooo0P0000040?oo
o`8000004P3oool5000000030?ooo`0000000000008000000`3oool2000001<0oooo1@0000030?oo
o`<000000P3oool200000180oooo000I0?ooo`040000003oool0oooo000001/0oooo00<000000?oo
o`3oool00`3oool010000000oooo0?ooo`00000K0?ooo`040000003oool0oooo00000080oooo00<0
00000?ooo`3oool05@3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000001P0
oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`00000G0?ooo`050000003oool0
oooo0?ooo`0000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0140oooo
00D000000?ooo`3oool0oooo000000050?ooo`040000003oool0oooo000001@0oooo00<000000?oo
o`3oool0103oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00l0oooo000I0?oo
o`040000003oool0oooo000001`0oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`00
000H0?ooo`D0000000D0oooo0000003oool0oooo0000000G0?ooo`040000003oool0oooo00000080
oooo00@000000?ooo`3oool00000603oool010000000oooo0?ooo`0000020?ooo`040000003oool0
oooo000001L0oooo00D000000?ooo`3oool0oooo000000020?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080
oooo00@000000?ooo`3oool00000503oool010000000oooo0?ooo`3oool5000000050?ooo`000000
oooo0?ooo`0000004@3oool001T0oooo00@000000?ooo`3oool000007@3oool01@000000oooo0?oo
o`3oool000000080oooo00<000000?ooo`3oool05P3oool010000000oooo0?ooo`0000020?ooo`04
0000003oool0oooo000001L0oooo0`0000030?ooo`040000003oool0oooo000001T0oooo0P000003
0?ooo`040000003oool0oooo000001L0oooo00D000000?ooo`3oool0oooo000000020?ooo`040000
003oool0oooo00000080oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`030?ooo`05
0000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`0B0?ooo`050000003oool0oooo
0?ooo`0000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00l0oooo000I
0?ooo`040000003oool0oooo000001/0oooo00@000000?ooo`3oool000000P3oool010000000oooo
0?ooo`00000I0?ooo`030000003oool000000080oooo00@000000?ooo`3oool00000603oool00`00
0000oooo0?ooo`020?ooo`040000003oool0oooo000001P0oooo00@000000?ooo`3oool000000P3o
ool010000000oooo0?ooo`00000G0?ooo`050000003oool0oooo0?ooo`0000000P3oool010000000
oooo0?ooo`0000020?ooo`030000003oool0oooo0140oooo00D000000?ooo`3oool0oooo00000002
0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`020?ooo`030000003oool000000080oooo00@000000?ooo`3oool000004@3oool001X0oooo0P00
000M0?ooo`800000103oool2000001/0oooo0P0000030?ooo`8000006@3oool3000000<0oooo0P00
000J0?ooo`800000103oool2000001L0oooo0P0000040?ooo`800000103oool2000001<0oooo0P00
00040?ooo`800000103oool2000001@0oooo0P0000050?ooo`8000000`3oool200000180oooo003o
0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo000E
0?ooool000002`0000010?ooo`005@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00<0oooo0@0000010?ooo`003P3oool2000000D0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07P3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07P3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01@0oooo0@0000010?ooo`003@3oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo0?l0oooo1`3oool100000040oooo000=0?ooo`040000
003oool0oooo000000@0oooo0P00003o0?ooo`L0oooo0P0000010?ooo`003@3oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool10?ooo`40
00000@3oool000d0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`040?ooo`80
0000o`3oool10?ooo`4000000@3oool000h0oooo0P0000050?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0o@3oool100000040oooo000E0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0o@3oool100000040oooo000E0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool0o03oool100000040oooo000E0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0o03oool100000040oooo000E0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0o03oool100000040oooo000E0?ooo`8000002@3oool00`000000oooo0?ooo`3j0?ooo`800000
0@3oool001D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3k0?ooo`4000000@3o
ool001D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3k0?ooo`4000000@3oool0
01D0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3j0?ooo`4000000@3oool001D0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3j0?ooo`4000000@3oool001D0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3i0?ooo`4000000@3oool001D0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`3i0?ooo`4000000@3oool001D0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`3i0?ooo`4000000@3oool001D0oooo0P00000<0?oo
o`030000003oool0oooo0?L0oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo0?P0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo0?L0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo0?L0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo0?L0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
0?L0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?L0
oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0?H0oooo
0@0000010?ooo`005@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0?H0oooo0@00
00010?ooo`005@3oool2000000h0oooo00<000000?ooo`3oool0m@3oool200000040oooo000E0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool0mP3oool100000040oooo000E0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0m@3oool100000040oooo000E0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0m@3oool100000040oooo000E0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0m@3oool100000040oooo000E0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool0m03oool100000040oooo00020?ooo`800000103oool20000
00<0oooo100000040?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0m03oool10000
0040oooo00001@3oool000000?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo0?@0oooo0@0000010?ooo`0000D0oooo0000003o
ool0oooo000000090?ooo`030000003oool0oooo00@0oooo0P00000A0?ooo`030000003oool0oooo
0?80oooo0P0000010?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3c0?ooo`4000000@3oool00005
0?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0l`3oool100000040oooo00020?ooo`8000002P3oool20000
00D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3b0?ooo`4000000@3oool001D0
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3b0?ooo`4000000@3oool001D0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3b0?ooo`4000000@3oool001D0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`3b0?ooo`4000000@3oool001D0oooo00<00000
0?ooo`3oool04@3oool00`000000oooo0?ooo`3b0?ooo`4000000@3oool001D0oooo0P00000C0?oo
o`030000003oool0oooo0?00oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0?40oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0B0?ooo`030000
003oool0oooo0?40oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo0?40oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo0?00oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
0?00oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0?00
oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0>l0oooo
0@0000010?ooo`005@3oool2000001D0oooo00<000000?ooo`3oool0kP3oool200000040oooo000E
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0k`3oool100000040oooo000E0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool0kP3oool100000040oooo000E0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool0kP3oool100000040oooo000E0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool0kP3oool100000040oooo000E0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool0k@3oool100000040oooo000E0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool0k@3oool100000040oooo000E0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0k@3oool100000040oooo000E0?ooo`8000005`3oool00`000000
oooo0?ooo`3/0?ooo`8000000@3oool001D0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`3/0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`3/0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`3/
0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`3/0?oo
o`4000000@3oool001D0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`3[0?ooo`40
00000@3oool001D0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`3[0?ooo`400000
0@3oool00080oooo0P0000040?ooo`8000001@3oool3000000<0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`3Z0?ooo`4000000@3oool000050?ooo`000000oooo0?ooo`0000002`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0jP3o
ool100000040oooo00001@3oool000000?ooo`3oool0000000P0oooo1@0000030?ooo`8000006P3o
ool00`000000oooo0?ooo`3Y0?ooo`8000000@3oool000050?ooo`000000oooo0?ooo`000000203o
ool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
j@3oool100000040oooo00001@3oool000000?ooo`3oool0000000T0oooo00<000000?ooo`000000
103oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0>T0oooo0@0000010?ooo`000P3o
ool2000000/0oooo0P0000040?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0j03o
ool100000040oooo000E0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0j03oool1
00000040oooo000E0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0j03oool10000
0040oooo000E0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0j03oool100000040
oooo000E0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0i`3oool100000040oooo
000E0?ooo`8000007@3oool00`000000oooo0?ooo`3V0?ooo`8000000@3oool001D0oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`3W0?ooo`4000000@3oool001D0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`3V0?ooo`4000000@3oool001D0oooo00<000000?ooo`3o
ool07@3oool00`000000oooo0?ooo`3V0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0
7P3oool00`000000oooo0?ooo`3U0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool07P3o
ool00`000000oooo0?ooo`3U0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool07P3oool0
0`000000oooo0?ooo`3U0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`3T0?ooo`4000000@3oool001D0oooo0P00000P0?ooo`030000003oool0oooo0><0
oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0><0oooo
0@0000010?ooo`005@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0><0oooo0@00
00010?ooo`005@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0><0oooo0@000001
0?ooo`005@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0>80oooo0@0000010?oo
o`005@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0>80oooo0@0000010?ooo`00
5@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0>40oooo0@0000010?ooo`005@3o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0>40oooo0@0000010?ooo`005@3oool2
000002@0oooo00<000000?ooo`3oool0g`3oool200000040oooo000E0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool0g`3oool100000040oooo000E0?ooo`030000003oool0oooo02@0
oooo00<000000?ooo`3oool0g`3oool100000040oooo000E0?ooo`030000003oool0oooo02D0oooo
00<000000?ooo`3oool0gP3oool100000040oooo000E0?ooo`030000003oool0oooo02H0oooo00<0
00000?ooo`3oool0g@3oool100000040oooo000E0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool0g@3oool100000040oooo00020?ooo`800000103oool2000000<0oooo0`0000050?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`3oool0g03oool100000040oooo00001@3oool0
00000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0W
0?ooo`030000003oool0oooo0=`0oooo0@0000010?ooo`0000D0oooo0000003oool0oooo00000008
0?ooo`040000003oool0oooo000000@0oooo0P00000Y0?ooo`030000003oool0oooo0=X0oooo0P00
00010?ooo`0000D0oooo0000003oool0oooo000000080?ooo`<000001@3oool00`000000oooo0?oo
o`0X0?ooo`030000003oool0oooo0=/0oooo0@0000010?ooo`0000D0oooo0000003oool0oooo0000
00090?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?oo
o`3J0?ooo`4000000@3oool00080oooo0P00000:0?ooo`<00000103oool00`000000oooo0?ooo`0Y
0?ooo`030000003oool0oooo0=X0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0Z0?oo
o`030000003oool0oooo0=T0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0[0?ooo`03
0000003oool0oooo0=P0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0/0?ooo`030000
003oool0oooo0=L0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0/0?ooo`030000003o
ool0oooo0=L0oooo0@0000010?ooo`005@3oool2000002h0oooo00<000000?ooo`3oool0e@3oool2
00000040oooo000E0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0eP3oool10000
0040oooo000E0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0e@3oool100000040
oooo000E0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0e03oool100000040oooo
000E0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0d`3oool100000040oooo000E
0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0dP3oool100000040oooo000E0?oo
o`030000003oool0oooo0340oooo00<000000?ooo`3oool0dP3oool100000040oooo000E0?ooo`03
0000003oool0oooo0380oooo00<000000?ooo`3oool0d@3oool100000040oooo000E0?ooo`030000
003oool0oooo03<0oooo00<000000?ooo`3oool0d03oool100000040oooo000E0?ooo`800000=@3o
ool00`000000oooo0?ooo`3>0?ooo`8000000@3oool001D0oooo00<000000?ooo`3oool0=@3oool0
0`000000oooo0?ooo`3>0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0=P3oool00`00
0000oooo0?ooo`3=0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0=`3oool00`000000
oooo0?ooo`3<0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0>03oool00`000000oooo
0?ooo`3;0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?oo
o`3:0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0>P3oool200000<X0oooo0@000001
0?ooo`005@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo0<P0oooo0@0000010?oo
o`005@3oool2000003d0oooo00<000000?ooo`3oool0aP3oool200000040oooo000E0?ooo`030000
003oool0oooo03d0oooo0P0000370?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0?`3o
ool200000<D0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`110?ooo`800000``3oool1
00000040oooo000E0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0`@3oool10000
0040oooo000E0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0`03oool100000040
oooo00020?ooo`800000103oool2000000@0oooo0P0000050?ooo`030000003oool0oooo04@0oooo
0P0000300?ooo`4000000@3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo04H0oooo0P00002n0?ooo`4000000@3oool000050?oo
o`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`800000B@3oool30000
0;X0oooo0P0000010?ooo`0000D0oooo0000003oool0oooo000000090?ooo`8000001@3oool00`00
0000oooo0?ooo`1;0?ooo`800000^@3oool100000040oooo00001@3oool000000?ooo`3oool00000
00P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`1=0?ooo`030000003oool0
oooo0;H0oooo0@0000010?ooo`000P3oool2000000X0oooo0P0000050?ooo`030000003oool0oooo
04h0oooo0P00002f0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0D03oool300000;<0
oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`1C0?ooo`800000/@3oool100000040oooo
000E0?ooo`030000003oool0oooo05D0oooo0`00002^0?ooo`4000000@3oool001D0oooo00<00000
0?ooo`3oool0F03oool200000:`0oooo0@0000010?ooo`005@3oool2000005/0oooo1000002W0?oo
o`8000000@3oool001D0oooo00<000000?ooo`3oool0GP3oool300000:D0oooo0@0000010?ooo`00
5@3oool00`000000oooo0?ooo`1Q0?ooo`<00000XP3oool100000040oooo000E0?ooo`030000003o
ool0oooo06@0oooo1@00002M0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0J@3oool4
000009T0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`1]0?ooo`D00000U03oool10000
0040oooo000E0?ooo`030000003oool0oooo0780oooo1@00002?0?ooo`4000000@3oool001D0oooo
00<000000?ooo`3oool0M`3oool5000008X0oooo0@0000010?ooo`005@3oool2000007d0oooo1`00
00220?ooo`8000000@3oool001D0oooo00<000000?ooo`3oool0P`3oool6000007d0oooo0@000001
0?ooo`005@3oool00`000000oooo0?ooo`290?ooo`T00000M03oool100000040oooo000E0?ooo`03
0000003oool0oooo0980oooo2P00001Z0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0
W03oool:00000600oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`2V0?ooo``00000E03o
ool100000040oooo000E0?ooo`030000003oool0oooo0;80oooo3`0000150?ooo`4000000@3oool0
01D0oooo00<000000?ooo`3oool0`@3ooolC00000380oooo0@0000010?ooo`005@3oool00`000000
oooo0?ooo`3D0?oooaD000007@3oool100000040oooo000E0?ooo`800000jP3ooolH000000@0oooo
0P0000010?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo0@0000010?ooo`005@3oool0
0`000000oooo0?ooo`3o0?ooo`L0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00<0oooo0@0000010?ooo`005@3ooooo000000/000000@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-16.3954, -0.100666, \
0.593102, 0.00610976}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
Now let's look at both an approximate form and the exact form of an M \
distribution.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?pmfM\)\)], "Input"],

Cell[BoxData[
    \("pmfM[m,r,k] returns a list of probabilities \!\({p\_0,p\_1,...,p\_k}\) \
according to an \!\(M(m,r,1)\) distribution; pmfM[m,\!\(\[Rho],\[Phi]\),n] \
returns the same list according to an \!\(M(m,r,\[Phi])\) distribution. Note \
m is the mean number of mutations, \!\(r=\[Beta]\_1/\[Beta]\_2\) is the ratio \
of mutant growth rate to nonmutant growth rate, and \
\!\(\[Phi]=1-N\_0/N\_T\)."\)], "Print",
  CellTags->"Info3317216011-5180669"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pmfM[10.5, 0.8, 10]\)], "Input"],

Cell[BoxData[
    \({0.000027536449349747158`, 0.00012850343029882007`, 
      0.00034573541961349213`, 0.0007047470220843672`, 0.001209956176818187`, 
      0.0018478624868760109`, 0.002593033991680133`, 0.0034141644273363568`, 
      0.0042789138833278706`, 0.005157198619975631`, 
      0.006023079521618202`}\)], "Output"]
}, Open  ]],

Cell["\<\
If we assume \[Phi]=1, we  can also use an algorithm devised by Koch  (based \
on an algorithm of Lea and Coulson) to compute the p.m.f. of an M \
distribution.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?pmfMKoch\)\)], "Input"],

Cell[BoxData[
    \("pmfMKoch[m,r,k] computes \!\({p\_0,p\_1,...,p\_k}\) for an \
\!\(M(m,r,1)\) distribution, using Koch's algorithm. Note that r is the \
reciprocal of Koch's original parameter b (called \!\(\[Rho]\) in SALVADOR \
v.1). (This function is not recommended for large k.)"\)], "Print",
  CellTags->"Info3317216011-5924977"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pmfMKoch[10.5, 0.8, 10]\)], "Input"],

Cell[BoxData[
    \({0.000027536449349747158`, 0.00012850343029882007`, 
      0.000345735419613492`, 0.0007047470220843671`, 0.0012099561768181867`, 
      0.0018478624868760109`, 0.0025930339916801328`, 0.0034141644273363563`, 
      0.00427891388332787`, 0.00515719861997563`, 
      0.006023079521618202`}\)], "Output"]
}, Open  ]],

Cell["\<\
To use the exact form of the M distribution, the parameter \[Phi]<1 must be \
specified.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pmfM[10.5, 0.8, 0.9999, 10]\)], "Input"],

Cell[BoxData[
    \({0.000027536449349747158`, 0.00012851628179849647`, 
      0.000345799986016298`, 0.0007049322303118314`, 0.0012103555467003163`, 
      0.0018485831774866113`, 0.0025941846925863032`, 0.003415844983222769`, 
      0.004281207952509993`, 0.005160169530586166`, 
      0.006026768867509342`}\)], "Output"]
}, Open  ]],

Cell["\<\
Finally let us take a look at the convolution of a Poisson distribution and \
an LD distribution .\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?pmfCairns\)\)], "Input"],

Cell[BoxData[
    \("pmfCairns[\!\(m,\[Phi],\[Lambda],k\)] returns a list of probabilities \
\!\({p\_0,...,p\_k}\) according to the convolution of an LD(\!\(m,\[Phi]\)) \
distribution and a Poisson(\!\(\[Lambda]\)) distribution."\)], "Print",
  CellTags->"Info3317216011-5580427"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pmfCairns[2.5, 1, 8.5, 10]\)], "Input"],

Cell[BoxData[
    \({0.00001670170079024566`, 0.00016284158270489518`, 
      0.0008008117576822997`, 0.0026513515064390232`, 0.006657036789692021`, 
      0.013540891094019243`, 0.023283755824621593`, 0.03488343821483244`, 
      0.04659431114081542`, 0.05652302198442524`, 
      0.06324871361500847`}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Estimation of Mutation Rates", "Section",
  FontSize->16],

Cell[TextData[{
  "SALVADOR concentrates on  estimation of the mean number of mutations \
denoted by m, because an estimate of the mutation rate ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\_\[Beta]\)]],
  "  can be easily obtained from that of m. The recommended estimation method \
is the maximum likelihood method. We use the experimental data of Demerec \
(1945) to illustrate the capabilities  of SALVADOR 2.0.  The following are \
the known experimental parameters of Demerec's experiment."
}], "Text"],

Cell[BoxData[
    \(N0 = 90; Nt = 1.9*10^8; phi = 1 - N0/Nt;\)], "Input"],

Cell["\<\
The experimental data of Demerec's experiment are given below.\
\>", "Text"],

Cell[BoxData[
    \(\(demerec = {33, 18, \ 839, 47, 13, 126, 48, 80, 9, 71, 196, 66, 28, 
          17, 27, 37, 126, 33, 12, 44, 28, 67, 730, 168, 44, 50, 583, 23, 17, 
          24};\)\)], "Input"],

Cell["\<\
We can fit an LD(m,1) distribution to the data by calling  newtonLD, which \
uses the Newton-Raphson methods to compute the maximum likelihood estimate \
(m.l.e.) of m.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?newtonLD\)\)], "Input"],

Cell[BoxData[
    \("newtonLD[data,opts] uses the Newton-Raphson method to compute mle of m \
for the LD model."\)], "Print",
  CellTags->"Info3317216011-4936587"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(newtonLD[demerec]\)], "Input"],

Cell[BoxData[
    \(10.843826994529076`\)], "Output"]
}, Open  ]],

Cell["\<\
Sometimes it is necessary to choose an initial guess, to set convergence \
critiria or to specify the maximum number of iterations. Thus it is helpful \
to familiarize yourselve with the options of newtonLD.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Options[newtonLD]", "Input"],

Cell[BoxData[
    \({ShowIterations \[Rule] False, Phi \[Rule] 1, InitialM \[Rule] \(-1\), 
      MaxIterations \[Rule] 50, Tolerance \[Rule] 1\/100000000}\)], "Output"]
}, Open  ]],

Cell["\<\
For example, we can choose two different starting points and see how the two \
converge to the same m.l.e. Note that if InitialM is set to a negative \
number, then SALVADOR 2.0 uses its own rules to deterimine an initial guess, \
which was documented in Zheng (2004).\
\>", "Text"],

Cell[CellGroupData[{

Cell["newtonLD[demerec,InitialM->3,ShowIterations->True]", "Input"],

Cell[BoxData[
    \(3\)], "Print"],

Cell[BoxData[
    \(5.9410741308071096`\)], "Print"],

Cell[BoxData[
    \(9.34988232782199`\)], "Print"],

Cell[BoxData[
    \(10.733970006389027`\)], "Print"],

Cell[BoxData[
    \(10.843274363697045`\)], "Print"],

Cell[BoxData[
    \(10.843826980617358`\)], "Print"],

Cell[BoxData[
    \(10.843826994529072`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["newtonLD[demerec,InitialM->15,ShowIterations->True]", "Input"],

Cell[BoxData[
    \(15\)], "Print"],

Cell[BoxData[
    \(10.171664609935714`\)], "Print"],

Cell[BoxData[
    \(10.822551997688214`\)], "Print"],

Cell[BoxData[
    \(10.843806356058554`\)], "Print"],

Cell[BoxData[
    \(10.843826994509673`\)], "Print"],

Cell[BoxData[
    \(10.843826994529072`\)], "Output"]
}, Open  ]],

Cell["\<\
If you believe that mutants grow at a rate different from that at which \
nonmutants grow, then you can try to fit a M(m,r) model to your data. Note \
that r in SALVADOR 2.0 is the reciporacol of \[Rho] in SALVADOR 1.0.\
\>", "Text"],

Cell[CellGroupData[{

Cell["?newtonM", "Input"],

Cell[BoxData[
    \("newtonM[data,opts] uses the Newton-Raphson method to compute MLEs of m \
and r for the M(m,r) model."\)], "Print",
  CellTags->"Info3317216012-4476652"]
}, Open  ]],

Cell[CellGroupData[{

Cell["newtonM[demerec]", "Input"],

Cell[BoxData[
    \({9.852618286896071`, 0.8938070840172462`}\)], "Output"]
}, Open  ]],

Cell["Options of newtonM are similar to those of newtonLD.", "Text"],

Cell[CellGroupData[{

Cell["Options[newtonM]", "Input"],

Cell[BoxData[
    \({ShowIterations \[Rule] False, MaxIterations \[Rule] 50, 
      Tolerance \[Rule] 1\/100000000, InitialM \[Rule] \(-1\), 
      InitialR \[Rule] \(-1\)}\)], "Output"]
}, Open  ]],

Cell["\<\
you can also use the slower melM to accomplish the same task.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Interval Estimation of Mutation Rates", "Section"],

Cell["\<\
The most important new feature of SALVADOR 2.0 is its capability of computing \
(profile) likelihood ratio-based confidence intervals.  For the equal growth \
model, the function for constructing a CI for m is LRInervalLD.\
\>", "Text"],

Cell[CellGroupData[{

Cell["?LRIntervalLD", "Input"],

Cell[BoxData[
    \("LRIntervalLD[X,opts] uses data given in X to construct a likelihood \
ratio (LR) based confidence interval for m that has asymptotic confidence \
coefficient \!\(1-\[Alpha]\); \!\(\[Alpha]\) is specified by Alpha->\!\(\
\[Alpha]\). Data in X is assumed to be generated by an LD(m,\!\(\[Phi]\)) \
distribution; \!\(\[Phi]\) is specified by Phi->\!\(\[Phi]\)."\)], "Print",
  CellTags->"Info3317216013-2836232"]
}, Open  ]],

Cell["\<\
A 95% asymptotic confidence interval for m is obtained by the command\
\>", "Text"],

Cell[CellGroupData[{

Cell["LRIntervalLD[demerec]", "Input"],

Cell[BoxData[
    \({8.650538086656935`, 13.194764931218957`}\)], "Output"]
}, Open  ]],

Cell["\<\
And a 90% confidence interval would be constructed by using the Alpha option. \
Note that if a (1-\[Alpha])100% CI is desired, then the options Alpah should \
be set to \[Alpha], not \[Alpha]/2, because critical values are determined by \
 2 chi-squared distribution.\
\>", "Text"],

Cell[CellGroupData[{

Cell["LRIntervalLD[demerec,Alpha->0.1]", "Input"],

Cell[BoxData[
    \({8.991839033899844`, 12.806736879987557`}\)], "Output"]
}, Open  ]],

Cell["In addition to Alpha, There are a few other options.", "Text"],

Cell[CellGroupData[{

Cell["Options[LRIntervalLD]", "Input"],

Cell[BoxData[
    \({Alpha \[Rule] 0.05`, Tolerance \[Rule] 1\/100000000, 
      ShowIterations \[Rule] False, MaxIterations \[Rule] 50, 
      Phi \[Rule] 1.`, InitialM \[Rule] \(-1\), InitialLowerM \[Rule] \(-1\), 
      InitialUpperM \[Rule] \(-1\)}\)], "Output"]
}, Open  ]],

Cell["\<\
InitialM sets an initial guess for computing the maximum likelihood estimate \
of m, which is needed for computing a CI for m. The two options, \
InitialLowerM and intialUpperM, are statrting points for computing the lower \
and upper boundary points of the desired CI for m. When these  options are \
set to a negative number, SALVADOR adopts its algorithms described in Zheng \
(2004) to determine reasosnalbe starting values.  For example, if we choose 3 \
as an initial guess for the lower boundary point, and 16 as an initial guess \
for the upper boudary point, the iteration process will proceed as follows.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
LRIntervalLD[demerec,InitialLowerM->3,InitialUpperM->16,ShowIterations->True]\
\
\>", "Input"],

Cell[BoxData[
    \("Iterating for lower limit ..."\)], "Print"],

Cell[BoxData[
    \(3\)], "Print"],

Cell[BoxData[
    \(5.699422221344831`\)], "Print"],

Cell[BoxData[
    \(7.603623983988979`\)], "Print"],

Cell[BoxData[
    \(8.454611596733105`\)], "Print"],

Cell[BoxData[
    \(8.64144232084982`\)], "Print"],

Cell[BoxData[
    \(8.65051691861597`\)], "Print"],

Cell[BoxData[
    \("Iterating for upper limit ..."\)], "Print"],

Cell[BoxData[
    \(16\)], "Print"],

Cell[BoxData[
    \(13.871854805032044`\)], "Print"],

Cell[BoxData[
    \(13.263291837867623`\)], "Print"],

Cell[BoxData[
    \(13.19564885862982`\)], "Print"],

Cell[BoxData[
    \(13.194765082626434`\)], "Print"],

Cell[BoxData[
    \(13.194764931218952`\)], "Print"],

Cell[BoxData[
    \({8.650538086541864`, 13.194764931218952`}\)], "Output"]
}, Open  ]],

Cell["\<\
You can also construct a Wald type CI for m by first computing expected \
Fisher information.\
\>", "Text"],

Cell[CellGroupData[{

Cell["?FisherInfoLD", "Input"],

Cell[BoxData[
    \("FisherInfoLD[m,\!\(\[Phi]\),k] computes the Fisher information for m \
according to an LD(m,\!\(\[Phi]\)) distribution; it retains k terms in the \
series \!\(I(m)=\[Sum]\ f\^2\%j/p\_j\)."\)], "Print",
  CellTags->"Info3317216015-6655442"]
}, Open  ]],

Cell[CellGroupData[{

Cell["mhat=newtonLD[demerec]", "Input"],

Cell[BoxData[
    \(10.843826994529076`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["info=FisherInfoLD[mhat,50000]", "Input"],

Cell[BoxData[
    \(0.025357094149261867`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["{mhat-1.96 Sqrt[1/(30 info)], mhat+1.96 Sqrt[1/(30 info)]}", "Input"],

Cell[BoxData[
    \({8.596606419965056`, 13.091047569093096`}\)], "Output"]
}, Open  ]],

Cell["\<\
This Wald type CI for m is very simular to the likelihood ratio based CI we \
just found.\
\>", "Text"],

Cell["\<\
When fitting a differential growth model to data, use the function \
profileIntervalM to construct a profile likelihood based CI for m, and use \
the function profileIntervalR to construct a CI for r.\
\>", "Text"],

Cell[CellGroupData[{

Cell["?profileIntervalM", "Input"],

Cell[BoxData[
    \("profileIntervalM[data,opt] computes profile likelihood based \
confidence intervals for m under the M(m,r) model."\)], "Print",
  CellTags->"Info3317216057-4785192"]
}, Open  ]],

Cell[CellGroupData[{

Cell["profileIntervalM[demerec]", "Input"],

Cell[BoxData[
    \({6.9830651233184415`, 13.007332373950865`}\)], "Output"]
}, Open  ]],

Cell["\<\
To get a Wald type CI for m, we have to compute the expected Fisher \
information matrix approximately. Since we know little about the converging \
speed of computing this informtion matrix,  it is strongly recommended that a \
Wald type CI be  corroborated by a likelihood ratio based CI.\
\>", "Text"],

Cell[CellGroupData[{

Cell["?FisherInfoMatrix", "Input"],

Cell[BoxData[
    \("FisherInfoMatrix[m,r,n] computes the expected Fisher information \
matrix for the M(m,r) model, using n terms."\)], "Print",
  CellTags->"Info3317216058-9081352"]
}, Open  ]],

Cell[CellGroupData[{

Cell["{mhatm,rhatm}=newtonM[demerec]", "Input"],

Cell[BoxData[
    \({9.852618286896071`, 0.8938070840172462`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["infoMat=FisherInfoMatrix[mhatm,rhatm,5000]", "Input"],

Cell[BoxData[
    \({{0.026539225837420636`, \(-0.2594358703137823`\)}, \
{\(-0.2594358703137823`\), 5.061358718741653`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["{{var11,var12},{var21,var22}}=Inverse[infoMat]", "Input"],

Cell[BoxData[
    \({{75.52286098136372`, 3.871161926289656`}, {3.8711619262896573`, 
        0.3960039931670454`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["{mhatm-1.96 Sqrt[var11/30],mhatm+1.96 Sqrt[var11/30]}", "Input"],

Cell[BoxData[
    \({6.742802522010061`, 12.96243405178208`}\)], "Output"]
}, Open  ]],

Cell["\<\
Again the above Wald type CI is similar to the likelihood ratio based CI we \
just found.\
\>", "Text"],

Cell["To get a 99% CI for M, set Alpha to 0.01, not 0.005.", "Text"],

Cell[CellGroupData[{

Cell["profileIntervalM[demerec,Alpha->0.01]", "Input"],

Cell[BoxData[
    \({6.158961230587444`, 14.043751024099189`}\)], "Output"]
}, Open  ]],

Cell["\<\
To see if the differential growth model is necessary, we can construct a 95% \
CI for r.\
\>", "Text"],

Cell[CellGroupData[{

Cell["profileIntervalR[demerec]", "Input"],

Cell[BoxData[
    \({0.6910073899881216`, 1.1268801153166552`}\)], "Output"]
}, Open  ]],

Cell["\<\
Because unit is not contained in the above CI, the equal growth model may be \
sufficient. \
\>", "Text"],

Cell["\<\
Sometimes you may want to use some of the options when constructing a \
likelihood based CI.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Options[profileIntervalM]", "Input"],

Cell[BoxData[
    \({Alpha \[Rule] 0.05`, MaxIterations \[Rule] 50, 
      InitialM \[Rule] \(-1\), InitialR \[Rule] \(-1\), 
      InitialLowerM \[Rule] \(-1\), InitialUpperM \[Rule] \(-1\), 
      Tolerance \[Rule] 1\/1000000000, InitialLowerR \[Rule] \(-1\), 
      InitialUpperR \[Rule] \(-1\), ShowIterations \[Rule] False}\)], "Output"]
}, Open  ]],

Cell["\<\
For example, occasionally you may have to try a few starting points in \
constructing a confidence interval.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
profileIntervalR[demerec,InitialLowerR->0.5,InitialUpperR->1.5,ShowIterations-\
>True]\
\>", "Input"],

Cell[BoxData[
    \("Iterating for lower limit ..."\)], "Print"],

Cell[BoxData[
    \({8.799956037182803`, 0.5`}\)], "Print"],

Cell[BoxData[
    \({2.2051609933373832`, 0.39316001239037357`}\)], "Print"],

Cell[BoxData[
    \({4.20025480491638`, 0.6105082612451901`}\)], "Print"],

Cell[BoxData[
    \({5.7695296742140645`, 0.5678451456137223`}\)], "Print"],

Cell[BoxData[
    \({7.23224482301614`, 0.6582003543179061`}\)], "Print"],

Cell[BoxData[
    \({7.692066492409306`, 0.6878755543876475`}\)], "Print"],

Cell[BoxData[
    \({7.73234888851331`, 0.690977939427495`}\)], "Print"],

Cell[BoxData[
    \({7.732677433440407`, 0.6910073874171986`}\)], "Print"],

Cell[BoxData[
    \({7.732677459362545`, 0.6910073899881213`}\)], "Print"],

Cell[BoxData[
    \("Iterating for upper limit ..."\)], "Print"],

Cell[BoxData[
    \({10.90528053660934`, 1.5`}\)], "Print"],

Cell[BoxData[
    \({13.335417373602336`, 1.337867887117464`}\)], "Print"],

Cell[BoxData[
    \({12.310028005062616`, 1.1633629546232844`}\)], "Print"],

Cell[BoxData[
    \({11.946647814278675`, 1.1289512043664727`}\)], "Print"],

Cell[BoxData[
    \({11.928254901550776`, 1.126887348033876`}\)], "Print"],

Cell[BoxData[
    \({11.928185096616975`, 1.1268801154076615`}\)], "Print"],

Cell[BoxData[
    \({0.6910073899881143`, 1.1268801153166508`}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Methods of Historical Interest", "Section",
  FontSize->16],

Cell["\<\
The maximum likelihood method is obviously the method of choice for \
estimating mutation rates. But there are several methods of great historical \
interest; these methods are still popular.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?MethodOfP0\)\)], "Input"],

Cell[BoxData[
    \("MethodOfP0[data] estimates m, the expected number of mutations, using \
the \!\(P\_0\) method."\)], "Print",
  CellTags->"Info3317216076-5808766"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MethodOfP0[demerec]\)], "Input"],

Cell[BoxData[
    \("The \!\(P\_0 \) method is not applicable to this case."\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?MethodOfMeans\)\)], "Input"],

Cell[BoxData[
    \("MethodOfMeans[data,\!\(w\_0:=10\)] estimates m (the mean number of \
mutations) by the method of means as proposed by Luria and Delbr\!\(\
\[UDoubleDot]\)ck. Note \!\(w\_0\) is an initial value needed for numerically \
solving a transcendental equation."\)], "Print",
  CellTags->"Info3317216076-9348376"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(MethodOfMeans[demerec]\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(18.940937803769877`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?MethodOfMedians\)\)], "Input"],

Cell[BoxData[
    \("MethodOfMedians[data,\!\(w\_0:=1\)] estimates m, the expected number \
of mutations, using the method of medians proposed by Lea and Coulson. Note \
\!\(w\_0\) is an initial value needed to solve a transcendental equation by \
Newton's method.  If only the median of the data is known, say med, \
MethodOfMedians[med] also returns an estimate of m."\)], "Print",
  CellTags->"Info3317216076-1494160"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MethodOfMedians[demerec]\)], "Input"],

Cell[BoxData[
    \(11.851891765263119`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?semiMLE\)\)], "Input"],

Cell[BoxData[
    \("semiMLE[data,\!\(m\_0:=1\)] computes an estimate of m by solving \!\(\
\[Sum]Y\_i=0\), where \!\(Y\_i\) are the Lea-Coulson transforms of the \
original data. Note that \!\(m\_0\) is a starting value used to solve the \
equation."\)], "Print",
  CellTags->"Info3317216076-2304299"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(semiMLE[demerec]\)], "Input"],

Cell[BoxData[
    \(11.553643621646893`\)], "Output"]
}, Open  ]],

Cell["\<\
It is noteworthy that Jones has proposed a method based on the median that \
does not require an initial guess.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\

JonesMedian[demerec]\
\>", "Input"],

Cell[BoxData[
    \(10.43365243358806`\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Another Example", "Section"],

Cell["\<\
Boe et al. (1994) meticulously conducted a large scale fluctuation \
experiment, producing the following 1104 observations.\
\>", "Text"],

Cell["\<\
boe={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,2,3,4,4,4,4,\
6,8,14,15,17,27,29,39,39,66,74,265,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,\
1,1,1,1,2,2,2,2,3,3,4,4,4,5,5,5,7,8,12,16,18,19,30,42,132,152,513,0,0,0,0,0,0,\
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,2,2,2,2,3,4,5,5,6,\
7,8,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,2,2,\
2,3,4,4,5,6,6,13,18,19,26,36,68,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,\
1,1,1,1,1,1,1,1,1,2,2,2,2,2,3,3,4,4,5,6,6,7,12,16,24,0,0,0,0,0,0,0,0,0,0,0,0,\
0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,2,2,2,3,4,4,5,5,6,10,11,12,14,24,27,29,30,\
73,482,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,\
1,1,1,1,2,3,3,5,11,16,30,31,49,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,\
1,1,1,1,2,2,2,2,2,2,2,2,2,4,5,5,6,6,6,7,12,12,12,16,513,0,0,0,0,0,0,0,0,0,0,0,\
0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,3,3,3,3,3,3,4,4,5,7,8,11,11,18,52,\
73,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,2,2,3,3,\
3,3,4,4,4,5,7,7,7,151,513,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,\
1,1,1,1,1,1,2,2,3,3,3,4,4,5,6,11,13,16,40,258,320,513,0,0,0,0,0,0,0,0,0,0,0,0,\
0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,3,3,3,3,4,4,4,11,11,17,107,\
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,2,2,2,2,2,2,2,2,3,3,\
3,4,4,6,6,16,18,20,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
0,1,1,1,1,1,1,1,2,2,3,3,5,8,8,9,21,21,26,37,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,2,3,3,3,3,4,5,5,6,6,7,7,8,10,12,14,0,0,0,0,0,0,\
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,2,3,3,4,5,5,6,13,16,\
19,26,41,105,116,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,\
1,1,1,1,2,2,2,3,6,6,6,11,13,19,21,34,140,146,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,2,2,3,3,3,4,9,10,10,11,13,19,20,23,59,69,0,0,\
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,2,2,2,3,3,3,4,4,4,4,4,5,5,\
8,8,9,10,21,26,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,2,\
2,2,2,2,2,3,3,4,4,6,9,9,10,14,25,26,29,57,137,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
0,0,0,0,0,1,1,1,1,1,1,1,1,2,2,2,2,2,3,3,3,3,3,4,5,6,7,8,10,14,16,28,0,0,0,0,0,\
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,3,4,4,6,6,7,\
7,18,32,35,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,2,2,2,2,2,\
2,3,3,3,3,4,4,5,5,8,10,11,12,19,21};\
\>", "Input"],

Cell["\<\
Assuming equal growth rate we have the maximum likelihood estimate of m  as \
follows.\
\>", "Text"],

Cell[CellGroupData[{

Cell["newtonLD[boe]", "Input"],

Cell[BoxData[
    \(0.7365908051572939`\)], "Output"]
}, Open  ]],

Cell["\<\
The fitting is less than perfect, but still an LD distribution catches the \
most prominent features of the data.\
\>", "Text"],

Cell[CellGroupData[{

Cell["plotFit[boe,logPlot->True]", "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.152571 -0.525882 1.1292 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.17638 -0.0125 -3 -9 ]
[.17638 -0.0125 3 0 ]
[.32895 -0.0125 -3 -9 ]
[.32895 -0.0125 3 0 ]
[.48152 -0.0125 -3 -9 ]
[.48152 -0.0125 3 0 ]
[.63409 -0.0125 -3 -9 ]
[.63409 -0.0125 3 0 ]
[.78666 -0.0125 -3 -9 ]
[.78666 -0.0125 3 0 ]
[.93923 -0.0125 -3 -9 ]
[.93923 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .03872 -18 -4.5 ]
[-0.0125 .03872 0 4.5 ]
[-0.0125 .15164 -18 -4.5 ]
[-0.0125 .15164 0 4.5 ]
[-0.0125 .26456 -18 -4.5 ]
[-0.0125 .26456 0 4.5 ]
[-0.0125 .37748 -18 -4.5 ]
[-0.0125 .37748 0 4.5 ]
[-0.0125 .4904 -18 -4.5 ]
[-0.0125 .4904 0 4.5 ]
[-0.0125 .60332 -6 -4.5 ]
[-0.0125 .60332 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.17638 0 m
.17638 .00625 L
s
[(1)] .17638 -0.0125 0 1 Mshowa
.32895 0 m
.32895 .00625 L
s
[(2)] .32895 -0.0125 0 1 Mshowa
.48152 0 m
.48152 .00625 L
s
[(3)] .48152 -0.0125 0 1 Mshowa
.63409 0 m
.63409 .00625 L
s
[(4)] .63409 -0.0125 0 1 Mshowa
.78666 0 m
.78666 .00625 L
s
[(5)] .78666 -0.0125 0 1 Mshowa
.93923 0 m
.93923 .00625 L
s
[(6)] .93923 -0.0125 0 1 Mshowa
.125 Mabswid
.05432 0 m
.05432 .00375 L
s
.08484 0 m
.08484 .00375 L
s
.11535 0 m
.11535 .00375 L
s
.14587 0 m
.14587 .00375 L
s
.20689 0 m
.20689 .00375 L
s
.23741 0 m
.23741 .00375 L
s
.26792 0 m
.26792 .00375 L
s
.29844 0 m
.29844 .00375 L
s
.35947 0 m
.35947 .00375 L
s
.38998 0 m
.38998 .00375 L
s
.42049 0 m
.42049 .00375 L
s
.45101 0 m
.45101 .00375 L
s
.51204 0 m
.51204 .00375 L
s
.54255 0 m
.54255 .00375 L
s
.57306 0 m
.57306 .00375 L
s
.60358 0 m
.60358 .00375 L
s
.66461 0 m
.66461 .00375 L
s
.69512 0 m
.69512 .00375 L
s
.72564 0 m
.72564 .00375 L
s
.75615 0 m
.75615 .00375 L
s
.81718 0 m
.81718 .00375 L
s
.84769 0 m
.84769 .00375 L
s
.87821 0 m
.87821 .00375 L
s
.90872 0 m
.90872 .00375 L
s
.96975 0 m
.96975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .03872 m
.00625 .03872 L
s
[(0.5)] -0.0125 .03872 1 0 Mshowa
0 .15164 m
.00625 .15164 L
s
[(0.6)] -0.0125 .15164 1 0 Mshowa
0 .26456 m
.00625 .26456 L
s
[(0.7)] -0.0125 .26456 1 0 Mshowa
0 .37748 m
.00625 .37748 L
s
[(0.8)] -0.0125 .37748 1 0 Mshowa
0 .4904 m
.00625 .4904 L
s
[(0.9)] -0.0125 .4904 1 0 Mshowa
0 .60332 m
.00625 .60332 L
s
[(1)] -0.0125 .60332 1 0 Mshowa
.125 Mabswid
0 .0613 m
.00375 .0613 L
s
0 .08389 m
.00375 .08389 L
s
0 .10647 m
.00375 .10647 L
s
0 .12905 m
.00375 .12905 L
s
0 .17422 m
.00375 .17422 L
s
0 .19681 m
.00375 .19681 L
s
0 .21939 m
.00375 .21939 L
s
0 .24197 m
.00375 .24197 L
s
0 .28714 m
.00375 .28714 L
s
0 .30973 m
.00375 .30973 L
s
0 .33231 m
.00375 .33231 L
s
0 .35489 m
.00375 .35489 L
s
0 .40006 m
.00375 .40006 L
s
0 .42265 m
.00375 .42265 L
s
0 .44523 m
.00375 .44523 L
s
0 .46781 m
.00375 .46781 L
s
0 .51298 m
.00375 .51298 L
s
0 .53557 m
.00375 .53557 L
s
0 .55815 m
.00375 .55815 L
s
0 .58073 m
.00375 .58073 L
s
0 .01613 m
.00375 .01613 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.17638 .61178 m
.17638 .61803 L
s
.32895 .61178 m
.32895 .61803 L
s
.48152 .61178 m
.48152 .61803 L
s
.63409 .61178 m
.63409 .61803 L
s
.78666 .61178 m
.78666 .61803 L
s
.93923 .61178 m
.93923 .61803 L
s
.125 Mabswid
.05432 .61428 m
.05432 .61803 L
s
.08484 .61428 m
.08484 .61803 L
s
.11535 .61428 m
.11535 .61803 L
s
.14587 .61428 m
.14587 .61803 L
s
.20689 .61428 m
.20689 .61803 L
s
.23741 .61428 m
.23741 .61803 L
s
.26792 .61428 m
.26792 .61803 L
s
.29844 .61428 m
.29844 .61803 L
s
.35947 .61428 m
.35947 .61803 L
s
.38998 .61428 m
.38998 .61803 L
s
.42049 .61428 m
.42049 .61803 L
s
.45101 .61428 m
.45101 .61803 L
s
.51204 .61428 m
.51204 .61803 L
s
.54255 .61428 m
.54255 .61803 L
s
.57306 .61428 m
.57306 .61803 L
s
.60358 .61428 m
.60358 .61803 L
s
.66461 .61428 m
.66461 .61803 L
s
.69512 .61428 m
.69512 .61803 L
s
.72564 .61428 m
.72564 .61803 L
s
.75615 .61428 m
.75615 .61803 L
s
.81718 .61428 m
.81718 .61803 L
s
.84769 .61428 m
.84769 .61803 L
s
.87821 .61428 m
.87821 .61803 L
s
.90872 .61428 m
.90872 .61803 L
s
.96975 .61428 m
.96975 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .03872 m
1 .03872 L
s
.99375 .15164 m
1 .15164 L
s
.99375 .26456 m
1 .26456 L
s
.99375 .37748 m
1 .37748 L
s
.99375 .4904 m
1 .4904 L
s
.99375 .60332 m
1 .60332 L
s
.125 Mabswid
.99625 .0613 m
1 .0613 L
s
.99625 .08389 m
1 .08389 L
s
.99625 .10647 m
1 .10647 L
s
.99625 .12905 m
1 .12905 L
s
.99625 .17422 m
1 .17422 L
s
.99625 .19681 m
1 .19681 L
s
.99625 .21939 m
1 .21939 L
s
.99625 .24197 m
1 .24197 L
s
.99625 .28714 m
1 .28714 L
s
.99625 .30973 m
1 .30973 L
s
.99625 .33231 m
1 .33231 L
s
.99625 .35489 m
1 .35489 L
s
.99625 .40006 m
1 .40006 L
s
.99625 .42265 m
1 .42265 L
s
.99625 .44523 m
1 .44523 L
s
.99625 .46781 m
1 .46781 L
s
.99625 .51298 m
1 .51298 L
s
.99625 .53557 m
1 .53557 L
s
.99625 .55815 m
1 .55815 L
s
.99625 .58073 m
1 .58073 L
s
.99625 .01613 m
1 .01613 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
1 0 0 r
.015 w
.02381 .02951 Mdot
.12956 .20237 Mdot
.19143 .29647 Mdot
.23532 .35477 Mdot
.26936 .39773 Mdot
.29718 .4233 Mdot
.3207 .44683 Mdot
.34107 .46012 Mdot
.35904 .47137 Mdot
.37512 .47649 Mdot
.38966 .48467 Mdot
.40293 .4949 Mdot
.41515 .5041 Mdot
.42645 .50922 Mdot
.43698 .51433 Mdot
.44683 .51536 Mdot
.45608 .52354 Mdot
.4648 .52558 Mdot
.47304 .5307 Mdot
.48087 .53684 Mdot
.48831 .53888 Mdot
.49541 .54399 Mdot
.50869 .54502 Mdot
.51492 .54706 Mdot
.5209 .54809 Mdot
.52666 .5532 Mdot
.53221 .55627 Mdot
.53756 .55729 Mdot
.54273 .56036 Mdot
.54774 .56343 Mdot
.55258 .56445 Mdot
.55727 .56547 Mdot
.56625 .5665 Mdot
.57055 .56752 Mdot
.57473 .56854 Mdot
.5788 .56957 Mdot
.58662 .57161 Mdot
.59039 .57263 Mdot
.59407 .57366 Mdot
.59766 .57468 Mdot
.62067 .5757 Mdot
.62956 .57673 Mdot
.64331 .57775 Mdot
.64849 .57877 Mdot
.66532 .57979 Mdot
.66981 .58082 Mdot
.67201 .58184 Mdot
.68048 .58389 Mdot
.68253 .58491 Mdot
.73531 .58593 Mdot
.73817 .58695 Mdot
.75038 .58798 Mdot
.76993 .589 Mdot
.77556 .59002 Mdot
.77885 .59104 Mdot
.7852 .59207 Mdot
.79031 .59309 Mdot
.79131 .59411 Mdot
.82674 .59514 Mdot
.87162 .59616 Mdot
.87569 .59718 Mdot
.90436 .5982 Mdot
.9667 .59923 Mdot
.97619 .60332 Mdot
0 1 0 r
.006 w
.02381 .01472 m
.12956 .21381 L
.19143 .31684 L
.23532 .37897 L
.26936 .42009 L
.29718 .44911 L
.3207 .47056 L
.34107 .48701 L
.35904 .49998 L
.37512 .51045 L
.38966 .51906 L
.40293 .52626 L
.41515 .53237 L
.42645 .5376 L
.43698 .54214 L
.44683 .5461 L
.45608 .5496 L
.4648 .5527 L
.47304 .55547 L
.48087 .55797 L
.48831 .56021 L
.49541 .56226 L
.50219 .56412 L
.50869 .56582 L
.51492 .56738 L
.5209 .56882 L
.52666 .57015 L
.53221 .57138 L
.53756 .57253 L
.54273 .5736 L
.54774 .57459 L
.55258 .57553 L
.55727 .5764 L
.56183 .57723 L
.56625 .578 L
.57055 .57873 L
.57473 .57942 L
.5788 .58007 L
.58276 .58069 L
.58662 .58128 L
.59039 .58184 L
.59407 .58237 L
.59766 .58287 L
.60117 .58335 L
.6046 .58381 L
.60795 .58425 L
.61123 .58467 L
.61444 .58507 L
.61759 .58546 L
.62067 .58583 L
Mistroke
.62369 .58618 L
.62665 .58652 L
.62956 .58685 L
.63241 .58716 L
.63521 .58747 L
.63796 .58776 L
.64066 .58804 L
.64331 .58831 L
.64592 .58858 L
.64849 .58883 L
.65101 .58907 L
.65349 .58931 L
.65593 .58954 L
.65833 .58976 L
.6607 .58998 L
.66303 .59018 L
.66532 .59039 L
.66758 .59058 L
.66981 .59077 L
.67201 .59096 L
.67417 .59113 L
.6763 .59131 L
.67841 .59148 L
.68048 .59164 L
.68253 .5918 L
.68455 .59196 L
.68655 .59211 L
.68852 .59226 L
.69046 .5924 L
.69238 .59254 L
.69427 .59268 L
.69615 .59281 L
.698 .59294 L
.69982 .59307 L
.70163 .59319 L
.70341 .59331 L
.70518 .59343 L
.70692 .59354 L
.70864 .59366 L
.71035 .59377 L
.71203 .59387 L
.7137 .59398 L
.71535 .59408 L
.71698 .59418 L
.7186 .59428 L
.7202 .59438 L
.72178 .59447 L
.72334 .59456 L
.72489 .59465 L
.72642 .59474 L
Mistroke
.72794 .59483 L
.72945 .59492 L
.73093 .595 L
.73241 .59508 L
.73387 .59516 L
.73531 .59524 L
.73675 .59532 L
.73817 .59539 L
.73957 .59547 L
.74097 .59554 L
.74235 .59561 L
.74371 .59568 L
.74507 .59575 L
.74642 .59582 L
.74775 .59588 L
.74907 .59595 L
.75038 .59601 L
.75168 .59608 L
.75296 .59614 L
.75424 .5962 L
.75551 .59626 L
.75676 .59632 L
.75801 .59638 L
.75924 .59643 L
.76047 .59649 L
.76168 .59655 L
.76289 .5966 L
.76409 .59665 L
.76527 .59671 L
.76645 .59676 L
.76762 .59681 L
.76878 .59686 L
.76993 .59691 L
.77108 .59696 L
.77221 .59701 L
.77334 .59705 L
.77445 .5971 L
.77556 .59715 L
.77667 .59719 L
.77776 .59724 L
.77885 .59728 L
.77992 .59732 L
.78099 .59737 L
.78206 .59741 L
.78311 .59745 L
.78416 .59749 L
.7852 .59753 L
.78624 .59757 L
.78727 .59761 L
.78829 .59765 L
Mistroke
.7893 .59769 L
.79031 .59772 L
.79131 .59776 L
.7923 .5978 L
.79329 .59783 L
.79427 .59787 L
.79524 .59791 L
.79621 .59794 L
.79718 .59797 L
.79813 .59801 L
.79908 .59804 L
.80003 .59807 L
.80097 .59811 L
.8019 .59814 L
.80283 .59817 L
.80375 .5982 L
.80467 .59823 L
.80558 .59827 L
.80648 .5983 L
.80738 .59833 L
.80828 .59836 L
.80917 .59838 L
.81005 .59841 L
.81093 .59844 L
.81181 .59847 L
.81267 .5985 L
.81354 .59853 L
.8144 .59855 L
.81525 .59858 L
.8161 .59861 L
.81695 .59863 L
.81779 .59866 L
.81862 .59869 L
.81946 .59871 L
.82028 .59874 L
.82111 .59876 L
.82192 .59879 L
.82274 .59881 L
.82355 .59883 L
.82435 .59886 L
.82515 .59888 L
.82595 .59891 L
.82674 .59893 L
.82753 .59895 L
.82832 .59897 L
.8291 .599 L
.82987 .59902 L
.83064 .59904 L
.83141 .59906 L
.83218 .59908 L
Mistroke
.83294 .59911 L
.8337 .59913 L
.83445 .59915 L
.8352 .59917 L
.83595 .59919 L
.83669 .59921 L
.83743 .59923 L
.83816 .59925 L
.83889 .59927 L
.83962 .59929 L
.84035 .59931 L
.84107 .59933 L
.84179 .59935 L
.8425 .59937 L
.84321 .59938 L
.84392 .5994 L
.84462 .59942 L
.84533 .59944 L
.84602 .59946 L
.84672 .59947 L
.84741 .59949 L
.8481 .59951 L
.84879 .59953 L
.84947 .59954 L
.85015 .59956 L
.85082 .59958 L
.8515 .59959 L
.85217 .59961 L
.85284 .59963 L
.8535 .59964 L
.85416 .59966 L
.85482 .59968 L
.85548 .59969 L
.85613 .59971 L
.85678 .59972 L
.85743 .59974 L
.85808 .59975 L
.85872 .59977 L
.85936 .59978 L
.85999 .5998 L
.86063 .59981 L
.86126 .59983 L
.86189 .59984 L
.86252 .59986 L
.86314 .59987 L
.86376 .59989 L
.86438 .5999 L
.865 .59991 L
.86561 .59993 L
.86622 .59994 L
Mistroke
.86683 .59996 L
.86744 .59997 L
.86804 .59998 L
.86865 .6 L
.86924 .60001 L
.86984 .60002 L
.87044 .60003 L
.87103 .60005 L
.87162 .60006 L
.87221 .60007 L
.87279 .60009 L
.87338 .6001 L
.87396 .60011 L
.87454 .60012 L
.87511 .60013 L
.87569 .60015 L
.87626 .60016 L
.87683 .60017 L
.8774 .60018 L
.87797 .60019 L
.87853 .60021 L
.87909 .60022 L
.87965 .60023 L
.88021 .60024 L
.88076 .60025 L
.88132 .60026 L
.88187 .60027 L
.88242 .60029 L
.88297 .6003 L
.88351 .60031 L
.88406 .60032 L
.8846 .60033 L
.88514 .60034 L
.88568 .60035 L
.88621 .60036 L
.88675 .60037 L
.88728 .60038 L
.88781 .60039 L
.88834 .6004 L
.88887 .60041 L
.88939 .60042 L
.88992 .60043 L
.89044 .60044 L
.89096 .60045 L
.89148 .60046 L
.89199 .60047 L
.89251 .60048 L
.89302 .60049 L
.89353 .6005 L
.89404 .60051 L
Mistroke
.89455 .60052 L
.89505 .60053 L
.89556 .60054 L
.89606 .60055 L
.89656 .60056 L
.89706 .60057 L
.89756 .60057 L
.89806 .60058 L
.89855 .60059 L
.89904 .6006 L
.89953 .60061 L
.90002 .60062 L
.90051 .60063 L
.901 .60064 L
.90148 .60065 L
.90197 .60065 L
.90245 .60066 L
.90293 .60067 L
.90341 .60068 L
.90389 .60069 L
.90436 .6007 L
.90484 .6007 L
.90531 .60071 L
.90578 .60072 L
.90625 .60073 L
.90672 .60074 L
.90719 .60074 L
.90765 .60075 L
.90812 .60076 L
.90858 .60077 L
.90904 .60078 L
.9095 .60078 L
.90996 .60079 L
.91042 .6008 L
.91088 .60081 L
.91133 .60081 L
.91178 .60082 L
.91224 .60083 L
.91269 .60084 L
.91314 .60084 L
.91358 .60085 L
.91403 .60086 L
.91448 .60087 L
.91492 .60087 L
.91536 .60088 L
.91581 .60089 L
.91625 .60089 L
.91668 .6009 L
.91712 .60091 L
.91756 .60092 L
Mistroke
.91799 .60092 L
.91843 .60093 L
.91886 .60094 L
.91929 .60094 L
.91972 .60095 L
.92015 .60096 L
.92058 .60096 L
.92101 .60097 L
.92143 .60098 L
.92186 .60098 L
.92228 .60099 L
.9227 .601 L
.92312 .601 L
.92354 .60101 L
.92396 .60101 L
.92438 .60102 L
.9248 .60103 L
.92521 .60103 L
.92562 .60104 L
.92604 .60105 L
.92645 .60105 L
.92686 .60106 L
.92727 .60106 L
.92768 .60107 L
.92809 .60108 L
.92849 .60108 L
.9289 .60109 L
.9293 .60109 L
.9297 .6011 L
.93011 .60111 L
.93051 .60111 L
.93091 .60112 L
.93131 .60112 L
.9317 .60113 L
.9321 .60114 L
.9325 .60114 L
.93289 .60115 L
.93328 .60115 L
.93368 .60116 L
.93407 .60116 L
.93446 .60117 L
.93485 .60117 L
.93524 .60118 L
.93563 .60119 L
.93601 .60119 L
.9364 .6012 L
.93678 .6012 L
.93717 .60121 L
.93755 .60121 L
.93793 .60122 L
Mistroke
.93831 .60122 L
.93869 .60123 L
.93907 .60123 L
.93945 .60124 L
.93983 .60124 L
.9402 .60125 L
.94058 .60125 L
.94095 .60126 L
.94133 .60126 L
.9417 .60127 L
.94207 .60127 L
.94244 .60128 L
.94281 .60128 L
.94318 .60129 L
.94355 .60129 L
.94392 .6013 L
.94428 .6013 L
.94465 .60131 L
.94501 .60131 L
.94538 .60132 L
.94574 .60132 L
.9461 .60133 L
.94646 .60133 L
.94682 .60134 L
.94718 .60134 L
.94754 .60135 L
.9479 .60135 L
.94825 .60136 L
.94861 .60136 L
.94897 .60137 L
.94932 .60137 L
.94967 .60137 L
.95003 .60138 L
.95038 .60138 L
.95073 .60139 L
.95108 .60139 L
.95143 .6014 L
.95178 .6014 L
.95213 .60141 L
.95247 .60141 L
.95282 .60141 L
.95317 .60142 L
.95351 .60142 L
.95385 .60143 L
.9542 .60143 L
.95454 .60144 L
.95488 .60144 L
.95522 .60144 L
.95556 .60145 L
.9559 .60145 L
Mistroke
.95624 .60146 L
.95658 .60146 L
.95692 .60147 L
.95725 .60147 L
.95759 .60147 L
.95792 .60148 L
.95826 .60148 L
.95859 .60149 L
.95892 .60149 L
.95926 .60149 L
.95959 .6015 L
.95992 .6015 L
.96025 .60151 L
.96058 .60151 L
.96091 .60151 L
.96123 .60152 L
.96156 .60152 L
.96189 .60153 L
.96221 .60153 L
.96254 .60153 L
.96286 .60154 L
.96318 .60154 L
.96351 .60154 L
.96383 .60155 L
.96415 .60155 L
.96447 .60156 L
.96479 .60156 L
.96511 .60156 L
.96543 .60157 L
.96575 .60157 L
.96607 .60157 L
.96638 .60158 L
.9667 .60158 L
.96702 .60159 L
.96733 .60159 L
.96764 .60159 L
.96796 .6016 L
.96827 .6016 L
.96858 .6016 L
.96889 .60161 L
.96921 .60161 L
.96952 .60161 L
.96983 .60162 L
.97014 .60162 L
.97044 .60162 L
.97075 .60163 L
.97106 .60163 L
.97137 .60163 L
.97167 .60164 L
.97198 .60164 L
Mistroke
.97228 .60164 L
.97259 .60165 L
.97289 .60165 L
.97319 .60165 L
.9735 .60166 L
.9738 .60166 L
.9741 .60166 L
.9744 .60167 L
.9747 .60167 L
.975 .60167 L
.9753 .60168 L
.9756 .60168 L
.97589 .60168 L
.97619 .60169 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{62, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`006P3oool2000002H0oooo1@00000S0?ooo`@000009P3oool2000002L0oooo0`00000U0?oo
o`8000009@3oool300000140oooo000I0?ooo`040000003oool0oooo000002L0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`0V0?ooo`040000003oool0oooo000002L0oooo00<00000
0?ooo`3oool08`3oool010000000oooo0?ooo`00000T0?ooo`040000003oool0oooo00000100oooo
000I0?ooo`040000003oool0oooo000002L0oooo00<000000?ooo`3oool0903oool00`000000oooo
0?ooo`0W0?ooo`030000003oool0oooo02<0oooo1@00000W0?ooo`030000003oool0oooo0280oooo
00@000000?ooo`3oool00000403oool001T0oooo00@000000?ooo`3oool000009`3oool00`000000
oooo0?ooo`0U0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool08P3oool010000000
oooo0?ooo`00000V0?ooo`8000009@3oool300000140oooo000I0?ooo`040000003oool0oooo0000
02L0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`00000U0?ooo`040000003oool0
oooo000002D0oooo00<000000?ooo`0000009P3oool00`000000oooo0?ooo`0U0?ooo`030000003o
ool0oooo0100oooo000J0?ooo`8000009`3oool2000002H0oooo0P00000W0?ooo`8000009`3oool2
000002H0oooo0`00000U0?ooo`<00000403oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo
8@3oool00?l0oooo8@3oool00?l0oooo8@3oool001D0ooooo`00000;00000040oooo000E0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`40
00000@3oool001D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0V0?ooo`030000
003oool0oooo02D0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0U0?ooo`030000
003oool0oooo02H0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0=0?ooo`400000
0@3oool001D0oooo00<000000?ooo`3oool0o`3oool70?ooo`4000000@3oool001D0oooo0P000004
0?ooo`03003o003oool0oooo0?l0oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`040?oo
o`03003o003oool0oooo0?l0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`040?ooo`03
003o003oool0oooo0?l0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`030?ooo`030?l0
003oool00?l00?l0oooo0@3oool100000040oooo00020?ooo`800000103oool2000000@0oooo0P00
00050?ooo`030000003oool0oooo0080oooo0`3o00000`00o`00oooo0?ooo`3n0?ooo`4000000@3o
ool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo00<0oooo00@0o`000?ooo`3oool00?l0o`3oool100000040oooo00001@3oool00000
0?ooo`3oool0000000/0oooo00<000000?ooo`3oool00P3oool3000000H0oooo00<00?l00?ooo`3o
ool0o03oool200000040oooo00001@3oool000000?ooo`3oool0000000T0oooo0P0000050?ooo`03
0000003oool0oooo00L0oooo00<00?l00?ooo`3oool0o03oool100000040oooo00001@3oool00000
0?ooo`3oool0000000T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`03
003o003oool0oooo0?`0oooo0@0000010?ooo`000P3oool2000000X0oooo0`0000040?ooo`030000
003oool0oooo00P0oooo00<00?l00?ooo`3oool0n`3oool100000040oooo000E0?ooo`030000003o
ool0oooo00P0oooo00<00?l00?ooo`3oool0n`3oool100000040oooo000E0?ooo`030000003oool0
oooo00T0oooo00<00?l00?ooo`3oool0nP3oool100000040oooo000E0?ooo`8000002P3oool00`00
o`00oooo0?ooo`3i0?ooo`8000000@3oool001D0oooo00<000000?ooo`3oool02P3oool00`00o`00
oooo0?ooo`3i0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool02P3oool00`00o`00oooo
0?ooo`3i0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool02`3oool00`00o`00oooo0?oo
o`3h0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool02`3oool00`00o`00oooo0?ooo`3h
0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0303oool00`00o`00oooo0?ooo`3g0?oo
o`4000000@3oool001D0oooo0P00000>0?ooo`03003o003oool0oooo0?D0oooo0P0000010?ooo`00
5@3oool00`000000oooo0?ooo`0=0?ooo`03003o003oool0oooo0?H0oooo0@0000010?ooo`005@3o
ool00`000000oooo0?ooo`0>0?ooo`03003o003oool0oooo0?D0oooo0@0000010?ooo`005@3oool0
0`000000oooo0?ooo`0>0?ooo`03003o003oool0oooo0?D0oooo0@0000010?ooo`005@3oool00`00
0000oooo0?ooo`0?0?ooo`03003o003oool0oooo0?@0oooo0@0000010?ooo`005@3oool00`000000
oooo0?ooo`0?0?ooo`03003o003oool0oooo0?@0oooo0@0000010?ooo`005@3oool200000140oooo
00<00?l00?ooo`3oool0lP3oool200000040oooo000E0?ooo`030000003oool0oooo0100oooo00<0
0?l00?ooo`3oool0l`3oool100000040oooo000E0?ooo`030000003oool0oooo0140oooo00<00?l0
0?ooo`3oool0lP3oool100000040oooo000E0?ooo`030000003oool0oooo0140oooo00<00?l00?oo
o`3oool0lP3oool100000040oooo000E0?ooo`030000003oool0oooo0180oooo00<00?l00?ooo`3o
ool0l@3oool100000040oooo000E0?ooo`030000003oool0oooo0180oooo00<00?l00?ooo`3oool0
l@3oool100000040oooo000E0?ooo`800000503oool00`00o`00oooo0?ooo`3_0?ooo`8000000@3o
ool001D0oooo00<000000?ooo`3oool04`3oool00`00o`00oooo0?ooo`3`0?ooo`4000000@3oool0
01D0oooo00<000000?ooo`3oool0503oool00`00o`00oooo0?ooo`3_0?ooo`4000000@3oool001D0
oooo00<000000?ooo`3oool0503oool00`00o`00oooo0?ooo`3_0?ooo`4000000@3oool00080oooo
0P0000040?ooo`8000000`3oool3000000D0oooo00<000000?ooo`3oool05@3oool00`00o`00oooo
0?ooo`3^0?ooo`4000000@3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo01D0oooo00<00?l00?ooo`3oool0kP3oool100000040
oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000103oool30000
01H0oooo00<00?l00?ooo`3oool0k03oool200000040oooo00001@3oool000000?ooo`3oool00000
00P0oooo0`0000050?ooo`030000003oool0oooo01L0oooo00<00?l00?ooo`3oool0k03oool10000
0040oooo00001@3oool000000?ooo`3oool0000000T0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0G0?ooo`03003o003oool0oooo0>`0oooo0@0000010?ooo`000P3oool2000000X0
oooo0`0000040?ooo`030000003oool0oooo01P0oooo00<00?l00?ooo`3oool0j`3oool100000040
oooo000E0?ooo`030000003oool0oooo01P0oooo00<00?l00?ooo`3oool0j`3oool100000040oooo
000E0?ooo`030000003oool0oooo01T0oooo00<00?l00?ooo`3oool0jP3oool100000040oooo000E
0?ooo`8000006P3oool00`00o`00oooo0?ooo`3Y0?ooo`8000000@3oool001D0oooo00<000000?oo
o`3oool06P3oool00`00o`00oooo0?ooo`3Y0?ooo`4000000@3oool001D0oooo00<000000?ooo`3o
ool06P3oool00`00o`00oooo0?ooo`3Y0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0
6`3oool00`00o`00oooo0?ooo`3X0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool06`3o
ool00`00o`00oooo0?ooo`3X0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0703oool0
0`00o`00oooo0?ooo`3W0?ooo`4000000@3oool001D0oooo0P00000M0?ooo`03003o003oool0oooo
0>H0oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`0M0?ooo`03003o003oool0o`000>H0
oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0M0?ooo`04003o003o0000o`000?l00>D0
oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0N0?ooo`03003o003o0000oooo0>D0oooo
0@0000010?ooo`005@3oool00`000000oooo0?ooo`0N0?ooo`03003o003oool0oooo0>D0oooo0@00
00010?ooo`005@3oool00`000000oooo0?ooo`0O0?ooo`03003o003oool0oooo0>@0oooo0@000001
0?ooo`005@3oool200000240oooo00<00?l00?ooo`3oool0hP3oool200000040oooo000E0?ooo`03
0000003oool0oooo0200oooo00<00?l00?ooo`3oool0h`3oool100000040oooo000E0?ooo`030000
003oool0oooo0240oooo00<00?l00?ooo`3oool0hP3oool100000040oooo000E0?ooo`030000003o
ool0oooo0280oooo00<00?l00?ooo`3oool0h@3oool100000040oooo000E0?ooo`030000003oool0
oooo0280oooo00<00?l00?ooo`3oool0h@3oool100000040oooo000E0?ooo`030000003oool0oooo
02<0oooo00<00?l00?ooo`3oool0h03oool100000040oooo000E0?ooo`800000903oool00`00o`00
oooo0?ooo`3O0?ooo`8000000@3oool001D0oooo00<000000?ooo`3oool0903oool00`00o`00oooo
0?ooo`3O0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool09@3oool00`00o`00oooo0?oo
o`3N0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool09@3oool00`00o`00oooo0?ooo`3N
0?ooo`4000000@3oool00080oooo0P0000040?ooo`8000001@3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo02H0oooo00<00?l00?ooo`3oool0g@3oool100000040oooo00001@3oool0
00000?ooo`3oool0000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0W0?oo
o`03003o003oool0oooo0=`0oooo0@0000010?ooo`0000D0oooo0000003oool0oooo0000000:0?oo
o`030000003oool0oooo00<0oooo0`00000W0?ooo`03003o003oool0oooo0=/0oooo0P0000010?oo
o`0000D0oooo0000003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0:03oool00`00o`00oooo0?ooo`3K0?ooo`4000000@3oool000050?ooo`000000oooo0?oo
o`000000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo02P0oooo00<00?l0
0?ooo`3oool0f`3oool100000040oooo00020?ooo`8000002@3oool4000000@0oooo00<000000?oo
o`3oool0:@3oool00`00o`00oooo0?ooo`3J0?ooo`4000000@3oool001D0oooo00<000000?ooo`3o
ool0:P3oool00`00o`00oooo0?ooo`3I0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0
:P3oool00`00o`00oooo0?ooo`3I0?ooo`4000000@3oool001D0oooo0P00000/0?ooo`03003o003o
ool0oooo0=L0oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`0/0?ooo`03003o003oool0
oooo0=L0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0/0?ooo`04003o003oool0oooo
0?l00=H0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0]0?ooo`04003o003o0000o`00
0?l00=D0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0]0?ooo`03003o003oool0o`00
0=H0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0^0?ooo`03003o003oool0oooo0=D0
oooo0@0000010?ooo`005@3oool200000300oooo00<00?l00?ooo`3oool0d`3oool200000040oooo
000E0?ooo`030000003oool0oooo02l0oooo00<00?l00?ooo`3oool0e03oool100000040oooo000E
0?ooo`030000003oool0oooo0300oooo00<00?l00?ooo`3oool0d`3oool100000040oooo000E0?oo
o`030000003oool0oooo0340oooo00<00?l00?ooo`3oool0dP3oool100000040oooo000E0?ooo`03
0000003oool0oooo0340oooo00<00?l00?ooo`3oool0dP3oool100000040oooo000E0?ooo`030000
003oool0oooo0380oooo00<00?l00?ooo`3oool0d@3oool100000040oooo000E0?ooo`800000=03o
ool00`00o`00oooo0?ooo`3?0?ooo`8000000@3oool001D0oooo00<000000?ooo`3oool0<`3oool0
0`00o`00oooo0?ooo`3@0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0=03oool00`00
o`00oooo0?ooo`3?0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0=@3oool00`00o`00
oooo0?ooo`3>0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0=@3oool00`00o`00oooo
0?ooo`3>0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0=P3oool00`00o`00oooo0?oo
o`020?ooo`030?l0003oool0oooo0<P0oooo0@0000010?ooo`005@3oool2000003P0oooo00<00?l0
0?ooo`3oool00`3o00380?ooo`8000000@3oool001D0oooo00<000000?ooo`3oool0>03oool01000
o`00oooo0?ooo`3o003:0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0>03oool00`00
o`00oooo0?ooo`3;0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0>@3oool00`00o`00
oooo0?ooo`3:0?ooo`4000000@3oool00080oooo0P0000040?ooo`800000103oool2000000D0oooo
00<000000?ooo`3oool0>P3oool00`00o`00oooo0?ooo`390?ooo`4000000@3oool000050?ooo`00
0000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo03X0
oooo00<00?l00?ooo`3oool0b@3oool100000040oooo00001@3oool000000?ooo`3oool0000000P0
oooo00@000000?ooo`3oool00000103oool3000003/0oooo00<00?l00?ooo`3oool0a`3oool20000
0040oooo00001@3oool000000?ooo`3oool0000000T0oooo0P0000050?ooo`030000003oool0oooo
03`0oooo00<00?l00?ooo`3oool0a`3oool100000040oooo00001@3oool000000?ooo`3oool00000
00P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0m0?ooo`03003o003oool0
oooo0<H0oooo0@0000010?ooo`000P3oool2000000X0oooo0P0000050?ooo`030000003oool0oooo
03d0oooo00<00?l00?ooo`3oool0aP3oool100000040oooo000E0?ooo`030000003oool0oooo03h0
oooo00<00?l00?ooo`3oool0a@3oool100000040oooo000E0?ooo`030000003oool0oooo03l0oooo
00<00?l00?ooo`3oool00P3oool00`3o0000oooo0?ooo`2o0?ooo`4000000@3oool001D0oooo0P00
00110?ooo`03003o003oool0oooo00<0o`00_`3oool200000040oooo000E0?ooo`030000003oool0
oooo0440oooo00@00?l00?ooo`3oool0o`00`@3oool100000040oooo000E0?ooo`030000003oool0
oooo0480oooo00<00?l00?ooo`3oool0`@3oool100000040oooo000E0?ooo`030000003oool0oooo
0480oooo00<00?l00?ooo`3oool0`@3oool100000040oooo000E0?ooo`030000003oool0oooo04<0
oooo00<00?l00?ooo`3oool0`03oool100000040oooo000E0?ooo`030000003oool0oooo04@0oooo
00<00?l00?ooo`3oool0_`3oool100000040oooo000E0?ooo`800000AP3oool00`00o`00oooo0?oo
o`030?ooo`030?l0003oool0oooo0;L0oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`16
0?ooo`04003o003oool0oooo0?ooo`<0o`00^@3oool100000040oooo000E0?ooo`030000003oool0
oooo04L0oooo00D00?l00?ooo`3oool0oooo0?l0002j0?ooo`4000000@3oool001D0oooo00<00000
0?ooo`3oool0B03oool00`00o`00oooo0?ooo`2k0?ooo`4000000@3oool001D0oooo00<000000?oo
o`3oool0B@3oool00`00o`00oooo0?ooo`2j0?ooo`4000000@3oool001D0oooo00<000000?ooo`3o
ool0BP3oool00`00o`00oooo0?ooo`2i0?ooo`4000000@3oool001D0oooo0P00001<0?ooo`03003o
003oool0oooo00@0oooo00<0o`000?ooo`3oool0/03oool200000040oooo000E0?ooo`030000003o
ool0oooo04`0oooo00<00?l00?ooo`3oool00P3oool30?l00;80oooo0@0000010?ooo`005@3oool0
0`000000oooo0?ooo`1=0?ooo`03003o003oool0oooo0080oooo00<0o`000?ooo`3oool0/@3oool1
00000040oooo000E0?ooo`030000003oool0oooo04h0oooo00<00?l00?ooo`3oool01P3oool00`3o
0000oooo0?ooo`2/0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0C`3oool00`00o`00
oooo0?ooo`040?ooo`<0o`00[@3oool100000040oooo000E0?ooo`030000003oool0oooo0500oooo
00<00?l00?ooo`3oool0103oool00`3o0000oooo0?ooo`2/0?ooo`4000000@3oool001D0oooo0P00
001B0?ooo`03003o003oool0oooo00P0oooo00<0o`000?ooo`3oool0YP3oool200000040oooo000E
0?ooo`030000003oool0oooo0580oooo00<00?l00?ooo`3oool01P3oool30?l00:P0oooo0@000001
0?ooo`005@3oool00`000000oooo0?ooo`1C0?ooo`03003o003oool0oooo00H0oooo00D0o`000?oo
o`3oool0oooo0?l0002U0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0E03oool2003o
00T0oooo0`3o002T0?ooo`4000000@3oool00080oooo0P0000040?ooo`8000000`3oool3000000D0
oooo00<000000?ooo`3oool0EP3oool00`00o`00oooo0?ooo`070?ooo`050?l0003oool0oooo0?oo
o`3o0000X@3oool100000040oooo00001@3oool000000?ooo`3oool0000000X0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`1G0?ooo`03003o003oool0oooo00T0oooo0`3o002P0?oo
o`4000000@3oool000050?ooo`000000oooo0?ooo`0000002@3oool3000000@0oooo0`00001H0?oo
o`03003o003oool0oooo00T0oooo00<0o`000?ooo`3oool0WP3oool200000040oooo00001@3oool0
00000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`1I
0?ooo`800?l0303oool00`3o0000oooo0?ooo`2L0?ooo`4000000@3oool000050?ooo`000000oooo
0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo05/0oooo00<0
0?l00?ooo`3oool0203oool30?l009d0oooo0@0000010?ooo`000P3oool2000000X0oooo0`000004
0?ooo`030000003oool0oooo05`0oooo00<00?l00?ooo`3oool0203oool01@3o0000oooo0?ooo`3o
ool0o`0009X0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`1M0?ooo`800?l02`3oool3
0?l000030?ooo`3o0000oooo09H0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`1O0?oo
o`800?l02P3oool00`3o0000oooo0?l000020?l009H0oooo0@0000010?ooo`005@3oool200000680
oooo00<00?l00?ooo`3oool02P3oool00`3o0000oooo0?l000020?ooo`030?l0003oool0oooo08l0
oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`1R0?ooo`800?l02`3oool60?l00940oooo
0@0000010?ooo`005@3oool00`000000oooo0?ooo`1T0?ooo`03003o003oool0oooo00T0oooo00@0
o`000?ooo`3oool0o`000P3oool00`3o0000oooo0?ooo`2=0?ooo`4000000@3oool001D0oooo00<0
00000?ooo`3oool0I@3oool2003o00h0oooo103o002=0?ooo`4000000@3oool001D0oooo00<00000
0?ooo`3oool0I`3oool3003o00`0oooo1@3o002;0?ooo`4000000@3oool001D0oooo00<000000?oo
o`3oool0JP3oool2003o00`0oooo103o002:0?ooo`4000000@3oool001D0oooo0P00001]0?ooo`80
0?l0303oool01@3o0000oooo0?l0003oool0o`0008H0oooo0P0000010?ooo`005@3oool00`000000
oooo0?ooo`1^0?ooo`800?l02`3oool50?l008H0oooo0@0000010?ooo`005@3oool00`000000oooo
0?ooo`1`0?ooo`800?l02P3oool01@3o0000oooo0?l0003oool0o`000080oooo00<0o`000?ooo`3o
ool0P03oool100000040oooo000E0?ooo`030000003oool0oooo0780oooo0`00o`0:0?ooo`L0o`00
00<0oooo0?l0003oool0O@3oool100000040oooo000E0?ooo`030000003oool0oooo07D0oooo0P00
o`090?ooo`030?l0003oool0oooo00H0o`00O@3oool100000040oooo000E0?ooo`030000003oool0
oooo07L0oooo1@00o`090?ooo`030?l0003oool0o`0000@0o`00NP3oool100000040oooo000E0?oo
o`800000O@3oool2003o00X0oooo1P3o001g0?ooo`8000000@3oool001D0oooo00<000000?ooo`3o
ool0OP3oool3003o00T0oooo1`3o001e0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0
P@3oool6003o00H0oooo1@3o00000`3oool0o`000?l000020?l006l0oooo0@0000010?ooo`005@3o
ool00`000000oooo0?ooo`270?ooo`<00?l0103oool30?l000030?ooo`3o0000o`0000H0o`00K03o
ool100000040oooo000E0?ooo`030000003oool0oooo08X0oooo1@00o`040?ooo`T0o`001@3oool0
103o0000oooo0?ooo`3o001Q0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0S`3oool6
003o00<0oooo1@3o00030?ooo`H0o`0000@0oooo0?l0003oool0o`000`3oool20?l005L0oooo0@00
00010?ooo`005@3oool2000009H0oooo1`00o`050?ooo`050?l0003oool0oooo0?l0003oool01@3o
00000`3oool0o`000?l000020?l000030?ooo`3o0000oooo0580oooo0P0000010?ooo`005@3oool0
0`000000oooo0?ooo`2L0?ooo`L00?l0103oool00`3o0000oooo0?l000030?ooo`H0o`003@3oool0
1@3o0000oooo0?ooo`3oool0o`000440oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`2S
0?ooo`X00?l00P3oool00`3o0000oooo0?l000020?l000/0oooo0`3o0000103oool0o`000?l0003o
00030?ooo`<0o`0000<0oooo0?l0003oool0=`3oool100000040oooo000E0?ooo`030000003oool0
oooo0:d0oooo3P00o`050?ooo`050?l0003oool0oooo0?ooo`3o00000`3oool70?l000T0oooo00<0
o`000?ooo`3oool0:`3oool100000040oooo000=0?ooo`D000000`3oool00`000000oooo0?ooo`2j
0?oooaD00?l000<0o`000?ooo`3oool01@3oool30?l000X0oooo0P3o00060?ooo`030?l0003oool0
oooo00h0oooo00<0o`000?ooo`3oool01P3oool100000040oooo000?0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0cP3oool]003o00060?ooo`3o0000o`000?l0003oool0o`001@3o
ool100000040oooo000?0?ooo`030000003oool0oooo00<0oooo0`00003T0?ooo`80o`001P3oool0
0`3o0000oooo0?ooo`0<0?ooo`H00?l000@0o`000?ooo`3oool0oooo0P0000010?ooo`003`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo0@3oool00`3o0000oooo0?ooo`03
0?ooo`4000000@3oool000l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo02H0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0V0?oo
o`030000003oool0oooo02D0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0U0?oo
o`030000003oool0oooo00d0oooo0@0000010?ooo`003P3oool2000000D0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo0@0000010?ooo`00
5@3ooooo000000/000000@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.684521, 0.424608, \
0.0246964, 0.00333684}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
If we use the differential growth model, we can improve the fitting \
significantly.\
\>", "Text"],

Cell[CellGroupData[{

Cell["newtonM[boe]", "Input"],

Cell[BoxData[
    \({0.7139394740998706`, 0.8379963656154868`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["plotFit[boe,DifferentialGrowth->True,logPlot->True]", "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.152571 -0.550153 1.15347 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.17638 -0.0125 -3 -9 ]
[.17638 -0.0125 3 0 ]
[.32895 -0.0125 -3 -9 ]
[.32895 -0.0125 3 0 ]
[.48152 -0.0125 -3 -9 ]
[.48152 -0.0125 3 0 ]
[.63409 -0.0125 -3 -9 ]
[.63409 -0.0125 3 0 ]
[.78666 -0.0125 -3 -9 ]
[.78666 -0.0125 3 0 ]
[.93923 -0.0125 -3 -9 ]
[.93923 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .02658 -18 -4.5 ]
[-0.0125 .02658 0 4.5 ]
[-0.0125 .14193 -18 -4.5 ]
[-0.0125 .14193 0 4.5 ]
[-0.0125 .25728 -18 -4.5 ]
[-0.0125 .25728 0 4.5 ]
[-0.0125 .37262 -18 -4.5 ]
[-0.0125 .37262 0 4.5 ]
[-0.0125 .48797 -18 -4.5 ]
[-0.0125 .48797 0 4.5 ]
[-0.0125 .60332 -6 -4.5 ]
[-0.0125 .60332 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.17638 0 m
.17638 .00625 L
s
[(1)] .17638 -0.0125 0 1 Mshowa
.32895 0 m
.32895 .00625 L
s
[(2)] .32895 -0.0125 0 1 Mshowa
.48152 0 m
.48152 .00625 L
s
[(3)] .48152 -0.0125 0 1 Mshowa
.63409 0 m
.63409 .00625 L
s
[(4)] .63409 -0.0125 0 1 Mshowa
.78666 0 m
.78666 .00625 L
s
[(5)] .78666 -0.0125 0 1 Mshowa
.93923 0 m
.93923 .00625 L
s
[(6)] .93923 -0.0125 0 1 Mshowa
.125 Mabswid
.05432 0 m
.05432 .00375 L
s
.08484 0 m
.08484 .00375 L
s
.11535 0 m
.11535 .00375 L
s
.14587 0 m
.14587 .00375 L
s
.20689 0 m
.20689 .00375 L
s
.23741 0 m
.23741 .00375 L
s
.26792 0 m
.26792 .00375 L
s
.29844 0 m
.29844 .00375 L
s
.35947 0 m
.35947 .00375 L
s
.38998 0 m
.38998 .00375 L
s
.42049 0 m
.42049 .00375 L
s
.45101 0 m
.45101 .00375 L
s
.51204 0 m
.51204 .00375 L
s
.54255 0 m
.54255 .00375 L
s
.57306 0 m
.57306 .00375 L
s
.60358 0 m
.60358 .00375 L
s
.66461 0 m
.66461 .00375 L
s
.69512 0 m
.69512 .00375 L
s
.72564 0 m
.72564 .00375 L
s
.75615 0 m
.75615 .00375 L
s
.81718 0 m
.81718 .00375 L
s
.84769 0 m
.84769 .00375 L
s
.87821 0 m
.87821 .00375 L
s
.90872 0 m
.90872 .00375 L
s
.96975 0 m
.96975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .02658 m
.00625 .02658 L
s
[(0.5)] -0.0125 .02658 1 0 Mshowa
0 .14193 m
.00625 .14193 L
s
[(0.6)] -0.0125 .14193 1 0 Mshowa
0 .25728 m
.00625 .25728 L
s
[(0.7)] -0.0125 .25728 1 0 Mshowa
0 .37262 m
.00625 .37262 L
s
[(0.8)] -0.0125 .37262 1 0 Mshowa
0 .48797 m
.00625 .48797 L
s
[(0.9)] -0.0125 .48797 1 0 Mshowa
0 .60332 m
.00625 .60332 L
s
[(1)] -0.0125 .60332 1 0 Mshowa
.125 Mabswid
0 .04965 m
.00375 .04965 L
s
0 .07272 m
.00375 .07272 L
s
0 .09579 m
.00375 .09579 L
s
0 .11886 m
.00375 .11886 L
s
0 .165 m
.00375 .165 L
s
0 .18807 m
.00375 .18807 L
s
0 .21114 m
.00375 .21114 L
s
0 .23421 m
.00375 .23421 L
s
0 .28035 m
.00375 .28035 L
s
0 .30342 m
.00375 .30342 L
s
0 .32649 m
.00375 .32649 L
s
0 .34956 m
.00375 .34956 L
s
0 .39569 m
.00375 .39569 L
s
0 .41876 m
.00375 .41876 L
s
0 .44183 m
.00375 .44183 L
s
0 .4649 m
.00375 .4649 L
s
0 .51104 m
.00375 .51104 L
s
0 .53411 m
.00375 .53411 L
s
0 .55718 m
.00375 .55718 L
s
0 .58025 m
.00375 .58025 L
s
0 .00351 m
.00375 .00351 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.17638 .61178 m
.17638 .61803 L
s
.32895 .61178 m
.32895 .61803 L
s
.48152 .61178 m
.48152 .61803 L
s
.63409 .61178 m
.63409 .61803 L
s
.78666 .61178 m
.78666 .61803 L
s
.93923 .61178 m
.93923 .61803 L
s
.125 Mabswid
.05432 .61428 m
.05432 .61803 L
s
.08484 .61428 m
.08484 .61803 L
s
.11535 .61428 m
.11535 .61803 L
s
.14587 .61428 m
.14587 .61803 L
s
.20689 .61428 m
.20689 .61803 L
s
.23741 .61428 m
.23741 .61803 L
s
.26792 .61428 m
.26792 .61803 L
s
.29844 .61428 m
.29844 .61803 L
s
.35947 .61428 m
.35947 .61803 L
s
.38998 .61428 m
.38998 .61803 L
s
.42049 .61428 m
.42049 .61803 L
s
.45101 .61428 m
.45101 .61803 L
s
.51204 .61428 m
.51204 .61803 L
s
.54255 .61428 m
.54255 .61803 L
s
.57306 .61428 m
.57306 .61803 L
s
.60358 .61428 m
.60358 .61803 L
s
.66461 .61428 m
.66461 .61803 L
s
.69512 .61428 m
.69512 .61803 L
s
.72564 .61428 m
.72564 .61803 L
s
.75615 .61428 m
.75615 .61803 L
s
.81718 .61428 m
.81718 .61803 L
s
.84769 .61428 m
.84769 .61803 L
s
.87821 .61428 m
.87821 .61803 L
s
.90872 .61428 m
.90872 .61803 L
s
.96975 .61428 m
.96975 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .02658 m
1 .02658 L
s
.99375 .14193 m
1 .14193 L
s
.99375 .25728 m
1 .25728 L
s
.99375 .37262 m
1 .37262 L
s
.99375 .48797 m
1 .48797 L
s
.99375 .60332 m
1 .60332 L
s
.125 Mabswid
.99625 .04965 m
1 .04965 L
s
.99625 .07272 m
1 .07272 L
s
.99625 .09579 m
1 .09579 L
s
.99625 .11886 m
1 .11886 L
s
.99625 .165 m
1 .165 L
s
.99625 .18807 m
1 .18807 L
s
.99625 .21114 m
1 .21114 L
s
.99625 .23421 m
1 .23421 L
s
.99625 .28035 m
1 .28035 L
s
.99625 .30342 m
1 .30342 L
s
.99625 .32649 m
1 .32649 L
s
.99625 .34956 m
1 .34956 L
s
.99625 .39569 m
1 .39569 L
s
.99625 .41876 m
1 .41876 L
s
.99625 .44183 m
1 .44183 L
s
.99625 .4649 m
1 .4649 L
s
.99625 .51104 m
1 .51104 L
s
.99625 .53411 m
1 .53411 L
s
.99625 .55718 m
1 .55718 L
s
.99625 .58025 m
1 .58025 L
s
.99625 .00351 m
1 .00351 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
1 0 0 r
.015 w
.02381 .01718 Mdot
.12956 .19375 Mdot
.19143 .28988 Mdot
.23532 .34943 Mdot
.26936 .39331 Mdot
.29718 .41943 Mdot
.3207 .44346 Mdot
.34107 .45705 Mdot
.35904 .46854 Mdot
.37512 .47376 Mdot
.38966 .48212 Mdot
.40293 .49257 Mdot
.41515 .50197 Mdot
.42645 .5072 Mdot
.43698 .51242 Mdot
.44683 .51347 Mdot
.45608 .52182 Mdot
.4648 .52391 Mdot
.47304 .52914 Mdot
.48087 .53541 Mdot
.48831 .5375 Mdot
.49541 .54272 Mdot
.50869 .54376 Mdot
.51492 .54585 Mdot
.5209 .5469 Mdot
.52666 .55212 Mdot
.53221 .55526 Mdot
.53756 .5563 Mdot
.54273 .55944 Mdot
.54774 .56257 Mdot
.55258 .56362 Mdot
.55727 .56466 Mdot
.56625 .56571 Mdot
.57055 .56675 Mdot
.57473 .5678 Mdot
.5788 .56884 Mdot
.58662 .57093 Mdot
.59039 .57197 Mdot
.59407 .57302 Mdot
.59766 .57406 Mdot
.62067 .57511 Mdot
.62956 .57615 Mdot
.64331 .5772 Mdot
.64849 .57824 Mdot
.66532 .57929 Mdot
.66981 .58033 Mdot
.67201 .58138 Mdot
.68048 .58347 Mdot
.68253 .58451 Mdot
.73531 .58556 Mdot
.73817 .5866 Mdot
.75038 .58765 Mdot
.76993 .58869 Mdot
.77556 .58974 Mdot
.77885 .59078 Mdot
.7852 .59183 Mdot
.79031 .59287 Mdot
.79131 .59392 Mdot
.82674 .59496 Mdot
.87162 .59601 Mdot
.87569 .59705 Mdot
.90436 .59809 Mdot
.9667 .59914 Mdot
.97619 .60332 Mdot
0 1 0 r
.006 w
.02381 .01472 m
.12956 .19858 L
.19143 .2933 L
.23532 .35139 L
.26936 .39073 L
.29718 .41915 L
.3207 .44065 L
.34107 .45749 L
.35904 .47105 L
.37512 .48219 L
.38966 .49153 L
.40293 .49946 L
.41515 .50629 L
.42645 .51222 L
.43698 .51744 L
.44683 .52205 L
.45608 .52617 L
.4648 .52987 L
.47304 .5332 L
.48087 .53623 L
.48831 .53899 L
.49541 .54152 L
.50219 .54384 L
.50869 .54598 L
.51492 .54797 L
.5209 .54981 L
.52666 .55152 L
.53221 .55312 L
.53756 .55462 L
.54273 .55602 L
.54774 .55734 L
.55258 .55859 L
.55727 .55976 L
.56183 .56087 L
.56625 .56191 L
.57055 .56291 L
.57473 .56385 L
.5788 .56475 L
.58276 .5656 L
.58662 .56642 L
.59039 .5672 L
.59407 .56794 L
.59766 .56865 L
.60117 .56933 L
.6046 .56998 L
.60795 .57061 L
.61123 .57121 L
.61444 .57178 L
.61759 .57234 L
.62067 .57287 L
Mistroke
.62369 .57339 L
.62665 .57388 L
.62956 .57436 L
.63241 .57482 L
.63521 .57527 L
.63796 .5757 L
.64066 .57612 L
.64331 .57652 L
.64592 .57691 L
.64849 .57729 L
.65101 .57766 L
.65349 .57801 L
.65593 .57836 L
.65833 .5787 L
.6607 .57902 L
.66303 .57934 L
.66532 .57964 L
.66758 .57994 L
.66981 .58024 L
.67201 .58052 L
.67417 .58079 L
.6763 .58106 L
.67841 .58132 L
.68048 .58158 L
.68253 .58183 L
.68455 .58207 L
.68655 .58231 L
.68852 .58254 L
.69046 .58276 L
.69238 .58299 L
.69427 .5832 L
.69615 .58341 L
.698 .58362 L
.69982 .58382 L
.70163 .58401 L
.70341 .58421 L
.70518 .58439 L
.70692 .58458 L
.70864 .58476 L
.71035 .58494 L
.71203 .58511 L
.7137 .58528 L
.71535 .58544 L
.71698 .58561 L
.7186 .58577 L
.7202 .58592 L
.72178 .58608 L
.72334 .58623 L
.72489 .58638 L
.72642 .58652 L
Mistroke
.72794 .58666 L
.72945 .5868 L
.73093 .58694 L
.73241 .58707 L
.73387 .58721 L
.73531 .58734 L
.73675 .58746 L
.73817 .58759 L
.73957 .58771 L
.74097 .58783 L
.74235 .58795 L
.74371 .58807 L
.74507 .58819 L
.74642 .5883 L
.74775 .58841 L
.74907 .58852 L
.75038 .58863 L
.75168 .58874 L
.75296 .58884 L
.75424 .58894 L
.75551 .58904 L
.75676 .58914 L
.75801 .58924 L
.75924 .58934 L
.76047 .58944 L
.76168 .58953 L
.76289 .58962 L
.76409 .58971 L
.76527 .5898 L
.76645 .58989 L
.76762 .58998 L
.76878 .59007 L
.76993 .59015 L
.77108 .59023 L
.77221 .59032 L
.77334 .5904 L
.77445 .59048 L
.77556 .59056 L
.77667 .59064 L
.77776 .59071 L
.77885 .59079 L
.77992 .59087 L
.78099 .59094 L
.78206 .59101 L
.78311 .59109 L
.78416 .59116 L
.7852 .59123 L
.78624 .5913 L
.78727 .59137 L
.78829 .59143 L
Mistroke
.7893 .5915 L
.79031 .59157 L
.79131 .59163 L
.7923 .5917 L
.79329 .59176 L
.79427 .59182 L
.79524 .59189 L
.79621 .59195 L
.79718 .59201 L
.79813 .59207 L
.79908 .59213 L
.80003 .59219 L
.80097 .59225 L
.8019 .5923 L
.80283 .59236 L
.80375 .59242 L
.80467 .59247 L
.80558 .59253 L
.80648 .59258 L
.80738 .59263 L
.80828 .59269 L
.80917 .59274 L
.81005 .59279 L
.81093 .59284 L
.81181 .59289 L
.81267 .59295 L
.81354 .59299 L
.8144 .59304 L
.81525 .59309 L
.8161 .59314 L
.81695 .59319 L
.81779 .59324 L
.81862 .59328 L
.81946 .59333 L
.82028 .59338 L
.82111 .59342 L
.82192 .59347 L
.82274 .59351 L
.82355 .59355 L
.82435 .5936 L
.82515 .59364 L
.82595 .59368 L
.82674 .59373 L
.82753 .59377 L
.82832 .59381 L
.8291 .59385 L
.82987 .59389 L
.83064 .59393 L
.83141 .59397 L
.83218 .59401 L
Mistroke
.83294 .59405 L
.8337 .59409 L
.83445 .59413 L
.8352 .59417 L
.83595 .59421 L
.83669 .59424 L
.83743 .59428 L
.83816 .59432 L
.83889 .59435 L
.83962 .59439 L
.84035 .59443 L
.84107 .59446 L
.84179 .5945 L
.8425 .59453 L
.84321 .59457 L
.84392 .5946 L
.84462 .59464 L
.84533 .59467 L
.84602 .5947 L
.84672 .59474 L
.84741 .59477 L
.8481 .5948 L
.84879 .59483 L
.84947 .59487 L
.85015 .5949 L
.85082 .59493 L
.8515 .59496 L
.85217 .59499 L
.85284 .59502 L
.8535 .59505 L
.85416 .59508 L
.85482 .59511 L
.85548 .59514 L
.85613 .59517 L
.85678 .5952 L
.85743 .59523 L
.85808 .59526 L
.85872 .59529 L
.85936 .59532 L
.85999 .59535 L
.86063 .59538 L
.86126 .5954 L
.86189 .59543 L
.86252 .59546 L
.86314 .59549 L
.86376 .59551 L
.86438 .59554 L
.865 .59557 L
.86561 .59559 L
.86622 .59562 L
Mistroke
.86683 .59564 L
.86744 .59567 L
.86804 .5957 L
.86865 .59572 L
.86924 .59575 L
.86984 .59577 L
.87044 .5958 L
.87103 .59582 L
.87162 .59585 L
.87221 .59587 L
.87279 .59589 L
.87338 .59592 L
.87396 .59594 L
.87454 .59597 L
.87511 .59599 L
.87569 .59601 L
.87626 .59604 L
.87683 .59606 L
.8774 .59608 L
.87797 .59611 L
.87853 .59613 L
.87909 .59615 L
.87965 .59617 L
.88021 .59619 L
.88076 .59622 L
.88132 .59624 L
.88187 .59626 L
.88242 .59628 L
.88297 .5963 L
.88351 .59632 L
.88406 .59635 L
.8846 .59637 L
.88514 .59639 L
.88568 .59641 L
.88621 .59643 L
.88675 .59645 L
.88728 .59647 L
.88781 .59649 L
.88834 .59651 L
.88887 .59653 L
.88939 .59655 L
.88992 .59657 L
.89044 .59659 L
.89096 .59661 L
.89148 .59663 L
.89199 .59665 L
.89251 .59666 L
.89302 .59668 L
.89353 .5967 L
.89404 .59672 L
Mistroke
.89455 .59674 L
.89505 .59676 L
.89556 .59678 L
.89606 .59679 L
.89656 .59681 L
.89706 .59683 L
.89756 .59685 L
.89806 .59687 L
.89855 .59688 L
.89904 .5969 L
.89953 .59692 L
.90002 .59694 L
.90051 .59695 L
.901 .59697 L
.90148 .59699 L
.90197 .59701 L
.90245 .59702 L
.90293 .59704 L
.90341 .59706 L
.90389 .59707 L
.90436 .59709 L
.90484 .5971 L
.90531 .59712 L
.90578 .59714 L
.90625 .59715 L
.90672 .59717 L
.90719 .59719 L
.90765 .5972 L
.90812 .59722 L
.90858 .59723 L
.90904 .59725 L
.9095 .59726 L
.90996 .59728 L
.91042 .59729 L
.91088 .59731 L
.91133 .59732 L
.91178 .59734 L
.91224 .59735 L
.91269 .59737 L
.91314 .59738 L
.91358 .5974 L
.91403 .59741 L
.91448 .59743 L
.91492 .59744 L
.91536 .59746 L
.91581 .59747 L
.91625 .59749 L
.91668 .5975 L
.91712 .59751 L
.91756 .59753 L
Mistroke
.91799 .59754 L
.91843 .59756 L
.91886 .59757 L
.91929 .59758 L
.91972 .5976 L
.92015 .59761 L
.92058 .59762 L
.92101 .59764 L
.92143 .59765 L
.92186 .59766 L
.92228 .59768 L
.9227 .59769 L
.92312 .5977 L
.92354 .59772 L
.92396 .59773 L
.92438 .59774 L
.9248 .59776 L
.92521 .59777 L
.92562 .59778 L
.92604 .59779 L
.92645 .59781 L
.92686 .59782 L
.92727 .59783 L
.92768 .59784 L
.92809 .59786 L
.92849 .59787 L
.9289 .59788 L
.9293 .59789 L
.9297 .59791 L
.93011 .59792 L
.93051 .59793 L
.93091 .59794 L
.93131 .59795 L
.9317 .59796 L
.9321 .59798 L
.9325 .59799 L
.93289 .598 L
.93328 .59801 L
.93368 .59802 L
.93407 .59803 L
.93446 .59805 L
.93485 .59806 L
.93524 .59807 L
.93563 .59808 L
.93601 .59809 L
.9364 .5981 L
.93678 .59811 L
.93717 .59812 L
.93755 .59814 L
.93793 .59815 L
Mistroke
.93831 .59816 L
.93869 .59817 L
.93907 .59818 L
.93945 .59819 L
.93983 .5982 L
.9402 .59821 L
.94058 .59822 L
.94095 .59823 L
.94133 .59824 L
.9417 .59825 L
.94207 .59826 L
.94244 .59827 L
.94281 .59828 L
.94318 .59829 L
.94355 .59831 L
.94392 .59832 L
.94428 .59833 L
.94465 .59834 L
.94501 .59835 L
.94538 .59836 L
.94574 .59837 L
.9461 .59838 L
.94646 .59839 L
.94682 .5984 L
.94718 .5984 L
.94754 .59841 L
.9479 .59842 L
.94825 .59843 L
.94861 .59844 L
.94897 .59845 L
.94932 .59846 L
.94967 .59847 L
.95003 .59848 L
.95038 .59849 L
.95073 .5985 L
.95108 .59851 L
.95143 .59852 L
.95178 .59853 L
.95213 .59854 L
.95247 .59855 L
.95282 .59856 L
.95317 .59857 L
.95351 .59857 L
.95385 .59858 L
.9542 .59859 L
.95454 .5986 L
.95488 .59861 L
.95522 .59862 L
.95556 .59863 L
.9559 .59864 L
Mistroke
.95624 .59865 L
.95658 .59865 L
.95692 .59866 L
.95725 .59867 L
.95759 .59868 L
.95792 .59869 L
.95826 .5987 L
.95859 .59871 L
.95892 .59871 L
.95926 .59872 L
.95959 .59873 L
.95992 .59874 L
.96025 .59875 L
.96058 .59876 L
.96091 .59876 L
.96123 .59877 L
.96156 .59878 L
.96189 .59879 L
.96221 .5988 L
.96254 .59881 L
.96286 .59881 L
.96318 .59882 L
.96351 .59883 L
.96383 .59884 L
.96415 .59885 L
.96447 .59885 L
.96479 .59886 L
.96511 .59887 L
.96543 .59888 L
.96575 .59889 L
.96607 .59889 L
.96638 .5989 L
.9667 .59891 L
.96702 .59892 L
.96733 .59892 L
.96764 .59893 L
.96796 .59894 L
.96827 .59895 L
.96858 .59895 L
.96889 .59896 L
.96921 .59897 L
.96952 .59898 L
.96983 .59898 L
.97014 .59899 L
.97044 .599 L
.97075 .59901 L
.97106 .59901 L
.97137 .59902 L
.97167 .59903 L
.97198 .59904 L
Mistroke
.97228 .59904 L
.97259 .59905 L
.97289 .59906 L
.97319 .59906 L
.9735 .59907 L
.9738 .59908 L
.9741 .59909 L
.9744 .59909 L
.9747 .5991 L
.975 .59911 L
.9753 .59911 L
.9756 .59912 L
.97589 .59913 L
.97619 .59913 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{62, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`006P3oool2000002H0oooo1@00000S0?ooo`@000009P3oool2000002L0oooo0`00000U0?oo
o`8000009@3oool300000140oooo000I0?ooo`040000003oool0oooo000002L0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`0V0?ooo`040000003oool0oooo000002L0oooo00<00000
0?ooo`3oool08`3oool010000000oooo0?ooo`00000T0?ooo`040000003oool0oooo00000100oooo
000I0?ooo`040000003oool0oooo000002L0oooo00<000000?ooo`3oool0903oool00`000000oooo
0?ooo`0W0?ooo`030000003oool0oooo02<0oooo1@00000W0?ooo`030000003oool0oooo0280oooo
00@000000?ooo`3oool00000403oool001T0oooo00@000000?ooo`3oool000009`3oool00`000000
oooo0?ooo`0U0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool08P3oool010000000
oooo0?ooo`00000V0?ooo`8000009@3oool300000140oooo000I0?ooo`040000003oool0oooo0000
02L0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`00000U0?ooo`040000003oool0
oooo000002D0oooo00<000000?ooo`0000009P3oool00`000000oooo0?ooo`0U0?ooo`030000003o
ool0oooo0100oooo000J0?ooo`8000009`3oool2000002H0oooo0P00000W0?ooo`8000009`3oool2
000002H0oooo0`00000U0?ooo`<00000403oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo
8@3oool00?l0oooo8@3oool00?l0oooo8@3oool001D0ooooo`00000;00000040oooo000E0?ooo`80
0000103oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo0P0000010?ooo`00
5@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0
9@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0
9P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo00d0oooo0@0000010?ooo`005@3o
ool00`000000oooo0?ooo`3o0?ooo`L0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`03
0?ooo`03003o003oool0oooo0?l0oooo0@3oool100000040oooo00020?ooo`800000103oool20000
00@0oooo0P0000050?ooo`030000003oool0oooo0080oooo0P3o00000`00o`00oooo0?ooo`3o0?oo
o`4000000@3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo00<0oooo00<0o`00003o003oool0o`3oool10?ooo`4000000@3oool0
00050?ooo`000000oooo0?ooo`0000002`3oool00`000000oooo0?ooo`020?ooo`<000001@3oool0
0`00o`00oooo0?ooo`3m0?ooo`8000000@3oool000050?ooo`000000oooo0?ooo`0000002@3oool2
000000D0oooo00<000000?ooo`3oool01@3oool00`00o`00oooo0?ooo`3n0?ooo`4000000@3oool0
00050?ooo`000000oooo0?ooo`0000002@3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00H0oooo00<00?l00?ooo`3oool0o@3oool100000040oooo00020?ooo`8000002P3oool30000
00@0oooo00<000000?ooo`3oool01P3oool00`00o`00oooo0?ooo`3m0?ooo`4000000@3oool001D0
oooo00<000000?ooo`3oool01`3oool00`00o`00oooo0?ooo`3l0?ooo`4000000@3oool001D0oooo
00<000000?ooo`3oool0203oool00`00o`00oooo0?ooo`3k0?ooo`4000000@3oool001D0oooo0P00
00090?ooo`03003o003oool0oooo0?X0oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`09
0?ooo`03003o003oool0oooo0?X0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`090?oo
o`03003o003oool0oooo0?X0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0:0?ooo`03
003o003oool0oooo0?T0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0:0?ooo`03003o
003oool0oooo0?T0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0;0?ooo`03003o003o
ool0oooo0?P0oooo0@0000010?ooo`005@3oool2000000d0oooo00<00?l00?ooo`3oool0mP3oool2
00000040oooo000E0?ooo`030000003oool0oooo00`0oooo00<00?l00?ooo`3oool0m`3oool10000
0040oooo000E0?ooo`030000003oool0oooo00d0oooo00<00?l00?ooo`3oool0mP3oool100000040
oooo000E0?ooo`030000003oool0oooo00d0oooo00<00?l00?ooo`3oool0mP3oool100000040oooo
000E0?ooo`030000003oool0oooo00h0oooo00<00?l00?ooo`3oool0m@3oool100000040oooo000E
0?ooo`030000003oool0oooo00h0oooo00<00?l00?ooo`3oool0m@3oool100000040oooo000E0?oo
o`800000403oool00`00o`00oooo0?ooo`3c0?ooo`8000000@3oool001D0oooo00<000000?ooo`3o
ool0403oool00`00o`00oooo0?ooo`3c0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0
403oool00`00o`00oooo0?ooo`3c0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool04@3o
ool00`00o`00oooo0?ooo`3b0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool04@3oool0
0`00o`00oooo0?ooo`3b0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool04P3oool00`00
o`00oooo0?ooo`3a0?ooo`4000000@3oool001D0oooo0P00000C0?ooo`03003o003oool0oooo0?00
oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`0C0?ooo`03003o003oool0oooo0?00oooo
0@0000010?ooo`005@3oool00`000000oooo0?ooo`0D0?ooo`03003o003oool0oooo0>l0oooo0@00
00010?ooo`005@3oool00`000000oooo0?ooo`0D0?ooo`03003o003oool0oooo0>l0oooo0@000001
0?ooo`005@3oool00`000000oooo0?ooo`0E0?ooo`03003o003oool0oooo0>h0oooo0@0000010?oo
o`000P3oool2000000@0oooo0P0000030?ooo`<000001@3oool00`000000oooo0?ooo`0E0?ooo`03
003o003oool0oooo0>h0oooo0@0000010?ooo`0000D0oooo0000003oool0oooo000000080?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool05P3oool00`00o`00oooo0?ooo`3]0?oo
o`4000000@3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`000004
0?ooo`<000005P3oool00`00o`00oooo0?ooo`3/0?ooo`8000000@3oool000050?ooo`000000oooo
0?ooo`000000203oool3000000D0oooo00<000000?ooo`3oool05`3oool00`00o`00oooo0?ooo`3/
0?ooo`4000000@3oool000050?ooo`000000oooo0?ooo`0000002@3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo01P0oooo00<00?l00?ooo`3oool0j`3oool100000040oooo00020?oo
o`8000002P3oool3000000@0oooo00<000000?ooo`3oool0603oool00`00o`00oooo0?ooo`3[0?oo
o`4000000@3oool001D0oooo00<000000?ooo`3oool06@3oool00`00o`00oooo0?ooo`3Z0?ooo`40
00000@3oool001D0oooo00<000000?ooo`3oool06@3oool00`00o`00oooo0?ooo`3Z0?ooo`400000
0@3oool001D0oooo0P00000K0?ooo`03003o003oool0oooo0>P0oooo0P0000010?ooo`005@3oool0
0`000000oooo0?ooo`0J0?ooo`03003o003oool0oooo0>T0oooo0@0000010?ooo`005@3oool00`00
0000oooo0?ooo`0K0?ooo`03003o003oool0oooo0>P0oooo0@0000010?ooo`005@3oool00`000000
oooo0?ooo`0L0?ooo`03003o003oool0oooo0>L0oooo0@0000010?ooo`005@3oool00`000000oooo
0?ooo`0L0?ooo`03003o003oool0oooo0>L0oooo0@0000010?ooo`005@3oool00`000000oooo0?oo
o`0M0?ooo`03003o003oool0oooo0>H0oooo0@0000010?ooo`005@3oool2000001h0oooo00<00?l0
0?ooo`3oool0i@3oool200000040oooo000E0?ooo`030000003oool0oooo01h0oooo00<00?l00?l0
003oool0i@3oool100000040oooo000E0?ooo`030000003oool0oooo01h0oooo00<00?l00?l0003o
0000i@3oool100000040oooo000E0?ooo`030000003oool0oooo01l0oooo00<00?l00?ooo`3oool0
i03oool100000040oooo000E0?ooo`030000003oool0oooo0200oooo00<00?l00?ooo`3oool0h`3o
ool100000040oooo000E0?ooo`030000003oool0oooo0200oooo00<00?l00?ooo`3oool0h`3oool1
00000040oooo000E0?ooo`8000008P3oool00`00o`00oooo0?ooo`3Q0?ooo`8000000@3oool001D0
oooo00<000000?ooo`3oool08P3oool00`00o`00oooo0?ooo`3Q0?ooo`4000000@3oool001D0oooo
00<000000?ooo`3oool08P3oool00`00o`00oooo0?ooo`3Q0?ooo`4000000@3oool001D0oooo00<0
00000?ooo`3oool08`3oool00`00o`00oooo0?ooo`3P0?ooo`4000000@3oool001D0oooo00<00000
0?ooo`3oool0903oool00`00o`00oooo0?ooo`3O0?ooo`4000000@3oool001D0oooo00<000000?oo
o`3oool0903oool00`00o`00oooo0?ooo`3O0?ooo`4000000@3oool001D0oooo0P00000V0?ooo`03
003o003oool0oooo0=d0oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`0V0?ooo`03003o
003oool0oooo0=d0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0V0?ooo`03003o003o
ool0oooo0=d0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0W0?ooo`03003o003oool0
oooo0=`0oooo0@0000010?ooo`000P3oool2000000@0oooo0P0000050?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0:03oool00`00o`00oooo0?ooo`3K0?ooo`4000000@3oool00005
0?ooo`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
02T0oooo00<00?l00?ooo`3oool0fP3oool100000040oooo00001@3oool000000?ooo`3oool00000
00X0oooo00<000000?ooo`3oool00`3oool3000002T0oooo00<00?l00?ooo`3oool0f@3oool20000
0040oooo00001@3oool000000?ooo`3oool0000000/0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0Z0?ooo`03003o003oool0oooo0=T0oooo0@0000010?ooo`0000D0oooo0000003o
ool0oooo000000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0:`3oool0
0`00o`00oooo0?ooo`3H0?ooo`4000000@3oool00080oooo0P0000090?ooo`@00000103oool00`00
0000oooo0?ooo`0[0?ooo`03003o003oool0oooo0=P0oooo0@0000010?ooo`005@3oool00`000000
oooo0?ooo`0/0?ooo`03003o003oool0oooo0=L0oooo0@0000010?ooo`005@3oool00`000000oooo
0?ooo`0]0?ooo`03003o003oool0oooo0=H0oooo0@0000010?ooo`005@3oool2000002h0oooo00<0
0?l00?ooo`3oool0e@3oool200000040oooo000E0?ooo`030000003oool0oooo02h0oooo00<00?l0
0?ooo`3oool0e@3oool100000040oooo000E0?ooo`030000003oool0oooo02l0oooo00<00?l00?oo
o`3oool0e03oool100000040oooo000E0?ooo`030000003oool0oooo02h0oooo00<0o`00003o003o
0000e@3oool100000040oooo000E0?ooo`030000003oool0oooo02l0oooo00<0o`00003o003oool0
e03oool100000040oooo000E0?ooo`030000003oool0oooo0340oooo00<00?l00?ooo`3oool0dP3o
ool100000040oooo000E0?ooo`800000<P3oool00`00o`00oooo0?ooo`3A0?ooo`8000000@3oool0
01D0oooo00<000000?ooo`3oool0<P3oool00`00o`00oooo0?ooo`3A0?ooo`4000000@3oool001D0
oooo00<000000?ooo`3oool0<`3oool00`00o`00oooo0?ooo`3@0?ooo`4000000@3oool001D0oooo
00<000000?ooo`3oool0=03oool00`00o`00oooo0?ooo`3?0?ooo`4000000@3oool001D0oooo00<0
00000?ooo`3oool0=03oool00`00o`00oooo0?ooo`3?0?ooo`4000000@3oool001D0oooo00<00000
0?ooo`3oool0=@3oool00`00o`00oooo0?ooo`3>0?ooo`4000000@3oool001D0oooo00<000000?oo
o`3oool0=P3oool00`00o`00oooo0?ooo`3=0?ooo`4000000@3oool001D0oooo0P00000h0?ooo`03
003o003oool0oooo0</0oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`0g0?ooo`03003o
003oool0oooo0<`0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0h0?ooo`03003o003o
ool0oooo0</0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0i0?ooo`03003o003oool0
oooo0<X0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0j0?ooo`03003o003oool0oooo
0<T0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`0j0?ooo`03003o003o0000oooo0<T0
oooo0@0000010?ooo`005@3oool2000003/0oooo00<0o`00003o003o0000b03oool200000040oooo
000E0?ooo`030000003oool0oooo03/0oooo00<0o`00003o003oool0b03oool100000040oooo000E
0?ooo`030000003oool0oooo03d0oooo00<00?l00?ooo`3oool0aP3oool100000040oooo000E0?oo
o`030000003oool0oooo03d0oooo00<00?l00?ooo`3oool0aP3oool100000040oooo00020?ooo`80
0000103oool2000000@0oooo0P0000050?ooo`030000003oool0oooo03h0oooo00<00?l00?ooo`3o
ool0a@3oool100000040oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3o
ool00000103oool00`000000oooo0?ooo`0o0?ooo`03003o003oool0oooo0<@0oooo0@0000010?oo
o`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo000000@0oooo0`000010
0?ooo`03003o003oool0oooo0<80oooo0P0000010?ooo`0000D0oooo0000003oool0oooo00000009
0?ooo`8000001@3oool00`000000oooo0?ooo`110?ooo`03003o003oool0oooo0<80oooo0@000001
0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0@P3oool00`00o`00oooo0?ooo`310?ooo`4000000@3oool00080oooo0P00000:
0?ooo`8000001@3oool00`000000oooo0?ooo`120?ooo`03003o003oool0oooo0<40oooo0@000001
0?ooo`005@3oool00`000000oooo0?ooo`130?ooo`03003o003oool0oooo0<00oooo0@0000010?oo
o`005@3oool00`000000oooo0?ooo`140?ooo`03003o003oool0oooo0;l0oooo0@0000010?ooo`00
5@3oool2000004@0oooo0P3o00000`00o`00oooo0?ooo`2m0?ooo`8000000@3oool001D0oooo00<0
00000?ooo`3oool0A03oool00`3o0000oooo003o002o0?ooo`4000000@3oool001D0oooo00<00000
0?ooo`3oool0A`3oool00`00o`00oooo0?ooo`2l0?ooo`4000000@3oool001D0oooo00<000000?oo
o`3oool0B03oool2003o0;`0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`1:0?ooo`03
003o003oool0oooo0;T0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`1;0?ooo`03003o
003oool0oooo0;P0oooo0@0000010?ooo`005@3oool2000004`0oooo00<0o`00003o003oool0]`3o
ool200000040oooo000E0?ooo`030000003oool0oooo04X0oooo0`3o00000`00o`00oooo0?ooo`2f
0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0B`3oool0103o0000oooo0?ooo`00o`2g
0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0C`3oool00`00o`00oooo0?ooo`2d0?oo
o`4000000@3oool001D0oooo00<000000?ooo`3oool0D03oool00`00o`00oooo0?ooo`2c0?ooo`40
00000@3oool001D0oooo00<000000?ooo`3oool0D@3oool00`00o`00oooo0?ooo`2b0?ooo`400000
0@3oool001D0oooo0P00001C0?ooo`03003o003oool0oooo0;00oooo0P0000010?ooo`005@3oool0
0`000000oooo0?ooo`1A0?ooo`80o`0000<00?l00?ooo`3oool0/03oool100000040oooo000E0?oo
o`030000003oool0oooo0580oooo00@0o`000?ooo`00o`000?l0/03oool100000040oooo000E0?oo
o`030000003oool0oooo05H0oooo00<00?l00?ooo`3oool0[@3oool100000040oooo000E0?ooo`03
0000003oool0oooo05L0oooo00<00?l00?ooo`3oool0[03oool100000040oooo000E0?ooo`030000
003oool0oooo05H0oooo0P3o00000`00o`00oooo0?ooo`2[0?ooo`4000000@3oool001D0oooo0P00
001H0?ooo`040?l0003oool00?l0003o0:X0oooo0P0000010?ooo`005@3oool00`000000oooo0?oo
o`1K0?ooo`03003o003o0000oooo0:P0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`1K
0?ooo`030?l00000o`00o`000080oooo00<0o`000?ooo`3oool0X`3oool100000040oooo000E0?oo
o`030000003oool0oooo05`0oooo00<0o`00003o003oool00`3o002T0?ooo`4000000@3oool00080
oooo0P0000040?ooo`8000000`3oool3000000D0oooo00<000000?ooo`3oool0GP3oool2003o0005
0?l0003oool0oooo0?ooo`3o0000X@3oool100000040oooo00001@3oool000000?ooo`3oool00000
00X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1P0?ooo`03003o003oool0oooo
00<0o`00X03oool100000040oooo00001@3oool000000?ooo`3oool0000000T0oooo0`0000040?oo
o`<00000H@3oool2003o00030?ooo`3o0000oooo09l0oooo0P0000010?ooo`0000D0oooo0000003o
ool0oooo000000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0H`3oool2
003o0080oooo00<0o`000?ooo`3oool0W03oool100000040oooo00001@3oool000000?ooo`3oool0
000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`1U0?ooo`800?l00P3o
002M0?ooo`4000000@3oool00080oooo0P00000:0?ooo`<00000103oool00`000000oooo0?ooo`1W
0?ooo`800?l0W@3oool100000040oooo000E0?ooo`030000003oool0oooo06T0oooo0P00o`000`3o
0000oooo0?ooo`2H0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0JP3oool01@3o0000
0?l00?l0003oool0o`0009L0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`1[0?ooo`03
0?l00000o`000?l000<0o`00U@3oool100000040oooo000E0?ooo`800000K`3oool2003o0080o`00
00<0oooo0?l0003oool0T03oool200000040oooo000E0?ooo`030000003oool0oooo0700oooo0P00
o`030?l00940oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`1b0?ooo`800?l00P3oool0
0`3o0000oooo0?l0002=0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0M03oool2003o
00@0o`00S03oool100000040oooo000E0?ooo`030000003oool0oooo07H0oooo00D00?l00?ooo`3o
0000oooo0?l0002;0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0M`3oool5003o0003
0?l0003oool0oooo08L0oooo0@0000010?ooo`005@3oool2000007/0oooo0P3o0002003o00030?l0
003oool0oooo08@0oooo0P0000010?ooo`005@3oool00`000000oooo0?ooo`1l0?ooo`80o`0000<0
0?l00?l0003o0000Q@3oool100000040oooo000E0?ooo`030000003oool0oooo07h0oooo00<0o`00
003o0000o`000P00o`001@3o0000oooo0?l0003oool0o`0007h0oooo0@0000010?ooo`005@3oool0
0`000000oooo0?ooo`200?ooo`030?l0003oool0o`0000@00?l00P3o001m0?ooo`4000000@3oool0
01D0oooo00<000000?ooo`3oool0P`3oool0103o0000oooo0?l0003oool3003o07`0oooo0@000001
0?ooo`005@3oool00`000000oooo0?ooo`270?ooo`<0o`001000o`1h0?ooo`4000000@3oool001D0
oooo0P0000290?ooo`H0o`001000o`1c0?ooo`8000000@3oool001D0oooo00<000000?ooo`3oool0
RP3oool70?l000040?ooo`00o`000?l0003o0740oooo0@0000010?ooo`005@3oool00`000000oooo
0?ooo`2=0?ooo`P0o`001P00o`1[0?ooo`4000000@3oool001D0oooo00<000000?ooo`3oool0SP3o
ool30?l00080oooo203o0005003o06H0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`2D
0?ooo`P0o`00103oool6003o0600oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`2H0?oo
o`D0o`000`3oool50?l000P00?l00P3o001G0?ooo`4000000@3oool001D0oooo0P00002L0?ooo`03
0?l0003oool0oooo00<0oooo00D0o`000?ooo`3oool0o`000?ooo`050?l000030?ooo`00o`000?l0
00L00?l0D03oool200000040oooo000E0?ooo`030000003oool0oooo0:L0oooo00<0o`000?ooo`3o
00000`3oool70?l000X00?l00P3oool01@3o0000oooo0?ooo`3oool0o`000440oooo0@0000010?oo
o`005@3oool00`000000oooo0?ooo`2_0?ooo`040?l0003oool0o`000?l000X0oooo3@00o`020?l0
00030?ooo`3o0000oooo03L0oooo0@0000010?ooo`005@3oool00`000000oooo0?ooo`300?ooo`05
0?l0003oool0oooo0?ooo`3o00000`3oool00`3o00000?l0003o000?003o00X0oooo00<0o`000?oo
o`3oool07`3oool100000040oooo000=0?ooo`D000000`3oool00`000000oooo0?ooo`390?ooo`<0
o`0000@0oooo0?l0003o0000o`001`3oool20?l001X00?l02P3oool00`3o0000oooo0?ooo`060?oo
o`4000000@3oool000l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3>0?ooo`03
0?l0003oool0oooo00L0oooo00<0o`000?ooo`3oool02@3oool30?l000@0oooo0`3o00040?ooo`l0
0?l01@3oool100000040oooo000?0?ooo`030000003oool0oooo00<0oooo0`00003U0?ooo`030?l0
003oool0oooo00@0oooo00<0o`000?ooo`3oool03P3oool00`3o0000oooo0?l000020?l000<0oooo
0P0000010?ooo`003`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo0@3o
ool00`3o0000oooo0?ooo`030?ooo`4000000@3oool000l0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool09@3oool0
0`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool09P3oool0
0`000000oooo0?ooo`0U0?ooo`030000003oool0oooo00d0oooo0@0000010?ooo`003P3oool20000
00D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo0@0000010?ooo`005@3ooooo000000/000000@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.684521, 0.436716, \
0.0246964, 0.00326663}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
By carefully setting options, one can use plotFit to draw publication quality \
graphics. \
\>", "Text"],

Cell[CellGroupData[{

Cell["Options[plotFit]", "Input"],

Cell[BoxData[
    \({logPlot \[Rule] False, DifferentialGrowth \[Rule] False, 
      dataStyle \[Rule] {RGBColor[1, 0, 0], PointSize[0.015`]}, 
      fitStyle \[Rule] {RGBColor[0, 1, 0], Thickness[0.006`]}, 
      InitialM \[Rule] \(-1\), InitialR \[Rule] \(-1\)}\)], "Output"]
}, Open  ]],

Cell["\<\
Note that when you set logPlot to True, the horizontal axis is log(x+1) in \
stead of log(x),  to avoid log(0). The meaning of the two options InitalM and \
Inital R is the same as in newtonM and newtonLD. As a illustrative example, \
we now use graphics to compare the two models fitted to the Boe el al. (1994) \
data.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
graphic1=plotFit[boe,fitStyle->{RGBColor[0,0,0],Dashing[{0.015}]},dataStyle->{\
RGBColor[0,0,0],PointSize[0.012]},logPlot->True]\
\>", "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.152571 -0.525882 1.1292 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.17638 -0.0125 -3 -9 ]
[.17638 -0.0125 3 0 ]
[.32895 -0.0125 -3 -9 ]
[.32895 -0.0125 3 0 ]
[.48152 -0.0125 -3 -9 ]
[.48152 -0.0125 3 0 ]
[.63409 -0.0125 -3 -9 ]
[.63409 -0.0125 3 0 ]
[.78666 -0.0125 -3 -9 ]
[.78666 -0.0125 3 0 ]
[.93923 -0.0125 -3 -9 ]
[.93923 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .03872 -18 -4.5 ]
[-0.0125 .03872 0 4.5 ]
[-0.0125 .15164 -18 -4.5 ]
[-0.0125 .15164 0 4.5 ]
[-0.0125 .26456 -18 -4.5 ]
[-0.0125 .26456 0 4.5 ]
[-0.0125 .37748 -18 -4.5 ]
[-0.0125 .37748 0 4.5 ]
[-0.0125 .4904 -18 -4.5 ]
[-0.0125 .4904 0 4.5 ]
[-0.0125 .60332 -6 -4.5 ]
[-0.0125 .60332 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.17638 0 m
.17638 .00625 L
s
[(1)] .17638 -0.0125 0 1 Mshowa
.32895 0 m
.32895 .00625 L
s
[(2)] .32895 -0.0125 0 1 Mshowa
.48152 0 m
.48152 .00625 L
s
[(3)] .48152 -0.0125 0 1 Mshowa
.63409 0 m
.63409 .00625 L
s
[(4)] .63409 -0.0125 0 1 Mshowa
.78666 0 m
.78666 .00625 L
s
[(5)] .78666 -0.0125 0 1 Mshowa
.93923 0 m
.93923 .00625 L
s
[(6)] .93923 -0.0125 0 1 Mshowa
.125 Mabswid
.05432 0 m
.05432 .00375 L
s
.08484 0 m
.08484 .00375 L
s
.11535 0 m
.11535 .00375 L
s
.14587 0 m
.14587 .00375 L
s
.20689 0 m
.20689 .00375 L
s
.23741 0 m
.23741 .00375 L
s
.26792 0 m
.26792 .00375 L
s
.29844 0 m
.29844 .00375 L
s
.35947 0 m
.35947 .00375 L
s
.38998 0 m
.38998 .00375 L
s
.42049 0 m
.42049 .00375 L
s
.45101 0 m
.45101 .00375 L
s
.51204 0 m
.51204 .00375 L
s
.54255 0 m
.54255 .00375 L
s
.57306 0 m
.57306 .00375 L
s
.60358 0 m
.60358 .00375 L
s
.66461 0 m
.66461 .00375 L
s
.69512 0 m
.69512 .00375 L
s
.72564 0 m
.72564 .00375 L
s
.75615 0 m
.75615 .00375 L
s
.81718 0 m
.81718 .00375 L
s
.84769 0 m
.84769 .00375 L
s
.87821 0 m
.87821 .00375 L
s
.90872 0 m
.90872 .00375 L
s
.96975 0 m
.96975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .03872 m
.00625 .03872 L
s
[(0.5)] -0.0125 .03872 1 0 Mshowa
0 .15164 m
.00625 .15164 L
s
[(0.6)] -0.0125 .15164 1 0 Mshowa
0 .26456 m
.00625 .26456 L
s
[(0.7)] -0.0125 .26456 1 0 Mshowa
0 .37748 m
.00625 .37748 L
s
[(0.8)] -0.0125 .37748 1 0 Mshowa
0 .4904 m
.00625 .4904 L
s
[(0.9)] -0.0125 .4904 1 0 Mshowa
0 .60332 m
.00625 .60332 L
s
[(1)] -0.0125 .60332 1 0 Mshowa
.125 Mabswid
0 .0613 m
.00375 .0613 L
s
0 .08389 m
.00375 .08389 L
s
0 .10647 m
.00375 .10647 L
s
0 .12905 m
.00375 .12905 L
s
0 .17422 m
.00375 .17422 L
s
0 .19681 m
.00375 .19681 L
s
0 .21939 m
.00375 .21939 L
s
0 .24197 m
.00375 .24197 L
s
0 .28714 m
.00375 .28714 L
s
0 .30973 m
.00375 .30973 L
s
0 .33231 m
.00375 .33231 L
s
0 .35489 m
.00375 .35489 L
s
0 .40006 m
.00375 .40006 L
s
0 .42265 m
.00375 .42265 L
s
0 .44523 m
.00375 .44523 L
s
0 .46781 m
.00375 .46781 L
s
0 .51298 m
.00375 .51298 L
s
0 .53557 m
.00375 .53557 L
s
0 .55815 m
.00375 .55815 L
s
0 .58073 m
.00375 .58073 L
s
0 .01613 m
.00375 .01613 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.17638 .61178 m
.17638 .61803 L
s
.32895 .61178 m
.32895 .61803 L
s
.48152 .61178 m
.48152 .61803 L
s
.63409 .61178 m
.63409 .61803 L
s
.78666 .61178 m
.78666 .61803 L
s
.93923 .61178 m
.93923 .61803 L
s
.125 Mabswid
.05432 .61428 m
.05432 .61803 L
s
.08484 .61428 m
.08484 .61803 L
s
.11535 .61428 m
.11535 .61803 L
s
.14587 .61428 m
.14587 .61803 L
s
.20689 .61428 m
.20689 .61803 L
s
.23741 .61428 m
.23741 .61803 L
s
.26792 .61428 m
.26792 .61803 L
s
.29844 .61428 m
.29844 .61803 L
s
.35947 .61428 m
.35947 .61803 L
s
.38998 .61428 m
.38998 .61803 L
s
.42049 .61428 m
.42049 .61803 L
s
.45101 .61428 m
.45101 .61803 L
s
.51204 .61428 m
.51204 .61803 L
s
.54255 .61428 m
.54255 .61803 L
s
.57306 .61428 m
.57306 .61803 L
s
.60358 .61428 m
.60358 .61803 L
s
.66461 .61428 m
.66461 .61803 L
s
.69512 .61428 m
.69512 .61803 L
s
.72564 .61428 m
.72564 .61803 L
s
.75615 .61428 m
.75615 .61803 L
s
.81718 .61428 m
.81718 .61803 L
s
.84769 .61428 m
.84769 .61803 L
s
.87821 .61428 m
.87821 .61803 L
s
.90872 .61428 m
.90872 .61803 L
s
.96975 .61428 m
.96975 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .03872 m
1 .03872 L
s
.99375 .15164 m
1 .15164 L
s
.99375 .26456 m
1 .26456 L
s
.99375 .37748 m
1 .37748 L
s
.99375 .4904 m
1 .4904 L
s
.99375 .60332 m
1 .60332 L
s
.125 Mabswid
.99625 .0613 m
1 .0613 L
s
.99625 .08389 m
1 .08389 L
s
.99625 .10647 m
1 .10647 L
s
.99625 .12905 m
1 .12905 L
s
.99625 .17422 m
1 .17422 L
s
.99625 .19681 m
1 .19681 L
s
.99625 .21939 m
1 .21939 L
s
.99625 .24197 m
1 .24197 L
s
.99625 .28714 m
1 .28714 L
s
.99625 .30973 m
1 .30973 L
s
.99625 .33231 m
1 .33231 L
s
.99625 .35489 m
1 .35489 L
s
.99625 .40006 m
1 .40006 L
s
.99625 .42265 m
1 .42265 L
s
.99625 .44523 m
1 .44523 L
s
.99625 .46781 m
1 .46781 L
s
.99625 .51298 m
1 .51298 L
s
.99625 .53557 m
1 .53557 L
s
.99625 .55815 m
1 .55815 L
s
.99625 .58073 m
1 .58073 L
s
.99625 .01613 m
1 .01613 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 0 r
.012 w
.02381 .02951 Mdot
.12956 .20237 Mdot
.19143 .29647 Mdot
.23532 .35477 Mdot
.26936 .39773 Mdot
.29718 .4233 Mdot
.3207 .44683 Mdot
.34107 .46012 Mdot
.35904 .47137 Mdot
.37512 .47649 Mdot
.38966 .48467 Mdot
.40293 .4949 Mdot
.41515 .5041 Mdot
.42645 .50922 Mdot
.43698 .51433 Mdot
.44683 .51536 Mdot
.45608 .52354 Mdot
.4648 .52558 Mdot
.47304 .5307 Mdot
.48087 .53684 Mdot
.48831 .53888 Mdot
.49541 .54399 Mdot
.50869 .54502 Mdot
.51492 .54706 Mdot
.5209 .54809 Mdot
.52666 .5532 Mdot
.53221 .55627 Mdot
.53756 .55729 Mdot
.54273 .56036 Mdot
.54774 .56343 Mdot
.55258 .56445 Mdot
.55727 .56547 Mdot
.56625 .5665 Mdot
.57055 .56752 Mdot
.57473 .56854 Mdot
.5788 .56957 Mdot
.58662 .57161 Mdot
.59039 .57263 Mdot
.59407 .57366 Mdot
.59766 .57468 Mdot
.62067 .5757 Mdot
.62956 .57673 Mdot
.64331 .57775 Mdot
.64849 .57877 Mdot
.66532 .57979 Mdot
.66981 .58082 Mdot
.67201 .58184 Mdot
.68048 .58389 Mdot
.68253 .58491 Mdot
.73531 .58593 Mdot
.73817 .58695 Mdot
.75038 .58798 Mdot
.76993 .589 Mdot
.77556 .59002 Mdot
.77885 .59104 Mdot
.7852 .59207 Mdot
.79031 .59309 Mdot
.79131 .59411 Mdot
.82674 .59514 Mdot
.87162 .59616 Mdot
.87569 .59718 Mdot
.90436 .5982 Mdot
.9667 .59923 Mdot
.97619 .60332 Mdot
.5 Mabswid
[ .015 ] 0 setdash
.02381 .01472 m
.12956 .21381 L
.19143 .31684 L
.23532 .37897 L
.26936 .42009 L
.29718 .44911 L
.3207 .47056 L
.34107 .48701 L
.35904 .49998 L
.37512 .51045 L
.38966 .51906 L
.40293 .52626 L
.41515 .53237 L
.42645 .5376 L
.43698 .54214 L
.44683 .5461 L
.45608 .5496 L
.4648 .5527 L
.47304 .55547 L
.48087 .55797 L
.48831 .56021 L
.49541 .56226 L
.50219 .56412 L
.50869 .56582 L
.51492 .56738 L
.5209 .56882 L
.52666 .57015 L
.53221 .57138 L
.53756 .57253 L
.54273 .5736 L
.54774 .57459 L
.55258 .57553 L
.55727 .5764 L
.56183 .57723 L
.56625 .578 L
.57055 .57873 L
.57473 .57942 L
.5788 .58007 L
.58276 .58069 L
.58662 .58128 L
.59039 .58184 L
.59407 .58237 L
.59766 .58287 L
.60117 .58335 L
.6046 .58381 L
.60795 .58425 L
.61123 .58467 L
.61444 .58507 L
.61759 .58546 L
.62067 .58583 L
Mistroke
.62369 .58618 L
.62665 .58652 L
.62956 .58685 L
.63241 .58716 L
.63521 .58747 L
.63796 .58776 L
.64066 .58804 L
.64331 .58831 L
.64592 .58858 L
.64849 .58883 L
.65101 .58907 L
.65349 .58931 L
.65593 .58954 L
.65833 .58976 L
.6607 .58998 L
.66303 .59018 L
.66532 .59039 L
.66758 .59058 L
.66981 .59077 L
.67201 .59096 L
.67417 .59113 L
.6763 .59131 L
.67841 .59148 L
.68048 .59164 L
.68253 .5918 L
.68455 .59196 L
.68655 .59211 L
.68852 .59226 L
.69046 .5924 L
.69238 .59254 L
.69427 .59268 L
.69615 .59281 L
.698 .59294 L
.69982 .59307 L
.70163 .59319 L
.70341 .59331 L
.70518 .59343 L
.70692 .59354 L
.70864 .59366 L
.71035 .59377 L
.71203 .59387 L
.7137 .59398 L
.71535 .59408 L
.71698 .59418 L
.7186 .59428 L
.7202 .59438 L
.72178 .59447 L
.72334 .59456 L
.72489 .59465 L
.72642 .59474 L
Mistroke
.72794 .59483 L
.72945 .59492 L
.73093 .595 L
.73241 .59508 L
.73387 .59516 L
.73531 .59524 L
.73675 .59532 L
.73817 .59539 L
.73957 .59547 L
.74097 .59554 L
.74235 .59561 L
.74371 .59568 L
.74507 .59575 L
.74642 .59582 L
.74775 .59588 L
.74907 .59595 L
.75038 .59601 L
.75168 .59608 L
.75296 .59614 L
.75424 .5962 L
.75551 .59626 L
.75676 .59632 L
.75801 .59638 L
.75924 .59643 L
.76047 .59649 L
.76168 .59655 L
.76289 .5966 L
.76409 .59665 L
.76527 .59671 L
.76645 .59676 L
.76762 .59681 L
.76878 .59686 L
.76993 .59691 L
.77108 .59696 L
.77221 .59701 L
.77334 .59705 L
.77445 .5971 L
.77556 .59715 L
.77667 .59719 L
.77776 .59724 L
.77885 .59728 L
.77992 .59732 L
.78099 .59737 L
.78206 .59741 L
.78311 .59745 L
.78416 .59749 L
.7852 .59753 L
.78624 .59757 L
.78727 .59761 L
.78829 .59765 L
Mistroke
.7893 .59769 L
.79031 .59772 L
.79131 .59776 L
.7923 .5978 L
.79329 .59783 L
.79427 .59787 L
.79524 .59791 L
.79621 .59794 L
.79718 .59797 L
.79813 .59801 L
.79908 .59804 L
.80003 .59807 L
.80097 .59811 L
.8019 .59814 L
.80283 .59817 L
.80375 .5982 L
.80467 .59823 L
.80558 .59827 L
.80648 .5983 L
.80738 .59833 L
.80828 .59836 L
.80917 .59838 L
.81005 .59841 L
.81093 .59844 L
.81181 .59847 L
.81267 .5985 L
.81354 .59853 L
.8144 .59855 L
.81525 .59858 L
.8161 .59861 L
.81695 .59863 L
.81779 .59866 L
.81862 .59869 L
.81946 .59871 L
.82028 .59874 L
.82111 .59876 L
.82192 .59879 L
.82274 .59881 L
.82355 .59883 L
.82435 .59886 L
.82515 .59888 L
.82595 .59891 L
.82674 .59893 L
.82753 .59895 L
.82832 .59897 L
.8291 .599 L
.82987 .59902 L
.83064 .59904 L
.83141 .59906 L
.83218 .59908 L
Mistroke
.83294 .59911 L
.8337 .59913 L
.83445 .59915 L
.8352 .59917 L
.83595 .59919 L
.83669 .59921 L
.83743 .59923 L
.83816 .59925 L
.83889 .59927 L
.83962 .59929 L
.84035 .59931 L
.84107 .59933 L
.84179 .59935 L
.8425 .59937 L
.84321 .59938 L
.84392 .5994 L
.84462 .59942 L
.84533 .59944 L
.84602 .59946 L
.84672 .59947 L
.84741 .59949 L
.8481 .59951 L
.84879 .59953 L
.84947 .59954 L
.85015 .59956 L
.85082 .59958 L
.8515 .59959 L
.85217 .59961 L
.85284 .59963 L
.8535 .59964 L
.85416 .59966 L
.85482 .59968 L
.85548 .59969 L
.85613 .59971 L
.85678 .59972 L
.85743 .59974 L
.85808 .59975 L
.85872 .59977 L
.85936 .59978 L
.85999 .5998 L
.86063 .59981 L
.86126 .59983 L
.86189 .59984 L
.86252 .59986 L
.86314 .59987 L
.86376 .59989 L
.86438 .5999 L
.865 .59991 L
.86561 .59993 L
.86622 .59994 L
Mistroke
.86683 .59996 L
.86744 .59997 L
.86804 .59998 L
.86865 .6 L
.86924 .60001 L
.86984 .60002 L
.87044 .60003 L
.87103 .60005 L
.87162 .60006 L
.87221 .60007 L
.87279 .60009 L
.87338 .6001 L
.87396 .60011 L
.87454 .60012 L
.87511 .60013 L
.87569 .60015 L
.87626 .60016 L
.87683 .60017 L
.8774 .60018 L
.87797 .60019 L
.87853 .60021 L
.87909 .60022 L
.87965 .60023 L
.88021 .60024 L
.88076 .60025 L
.88132 .60026 L
.88187 .60027 L
.88242 .60029 L
.88297 .6003 L
.88351 .60031 L
.88406 .60032 L
.8846 .60033 L
.88514 .60034 L
.88568 .60035 L
.88621 .60036 L
.88675 .60037 L
.88728 .60038 L
.88781 .60039 L
.88834 .6004 L
.88887 .60041 L
.88939 .60042 L
.88992 .60043 L
.89044 .60044 L
.89096 .60045 L
.89148 .60046 L
.89199 .60047 L
.89251 .60048 L
.89302 .60049 L
.89353 .6005 L
.89404 .60051 L
Mistroke
.89455 .60052 L
.89505 .60053 L
.89556 .60054 L
.89606 .60055 L
.89656 .60056 L
.89706 .60057 L
.89756 .60057 L
.89806 .60058 L
.89855 .60059 L
.89904 .6006 L
.89953 .60061 L
.90002 .60062 L
.90051 .60063 L
.901 .60064 L
.90148 .60065 L
.90197 .60065 L
.90245 .60066 L
.90293 .60067 L
.90341 .60068 L
.90389 .60069 L
.90436 .6007 L
.90484 .6007 L
.90531 .60071 L
.90578 .60072 L
.90625 .60073 L
.90672 .60074 L
.90719 .60074 L
.90765 .60075 L
.90812 .60076 L
.90858 .60077 L
.90904 .60078 L
.9095 .60078 L
.90996 .60079 L
.91042 .6008 L
.91088 .60081 L
.91133 .60081 L
.91178 .60082 L
.91224 .60083 L
.91269 .60084 L
.91314 .60084 L
.91358 .60085 L
.91403 .60086 L
.91448 .60087 L
.91492 .60087 L
.91536 .60088 L
.91581 .60089 L
.91625 .60089 L
.91668 .6009 L
.91712 .60091 L
.91756 .60092 L
Mistroke
.91799 .60092 L
.91843 .60093 L
.91886 .60094 L
.91929 .60094 L
.91972 .60095 L
.92015 .60096 L
.92058 .60096 L
.92101 .60097 L
.92143 .60098 L
.92186 .60098 L
.92228 .60099 L
.9227 .601 L
.92312 .601 L
.92354 .60101 L
.92396 .60101 L
.92438 .60102 L
.9248 .60103 L
.92521 .60103 L
.92562 .60104 L
.92604 .60105 L
.92645 .60105 L
.92686 .60106 L
.92727 .60106 L
.92768 .60107 L
.92809 .60108 L
.92849 .60108 L
.9289 .60109 L
.9293 .60109 L
.9297 .6011 L
.93011 .60111 L
.93051 .60111 L
.93091 .60112 L
.93131 .60112 L
.9317 .60113 L
.9321 .60114 L
.9325 .60114 L
.93289 .60115 L
.93328 .60115 L
.93368 .60116 L
.93407 .60116 L
.93446 .60117 L
.93485 .60117 L
.93524 .60118 L
.93563 .60119 L
.93601 .60119 L
.9364 .6012 L
.93678 .6012 L
.93717 .60121 L
.93755 .60121 L
.93793 .60122 L
Mistroke
.93831 .60122 L
.93869 .60123 L
.93907 .60123 L
.93945 .60124 L
.93983 .60124 L
.9402 .60125 L
.94058 .60125 L
.94095 .60126 L
.94133 .60126 L
.9417 .60127 L
.94207 .60127 L
.94244 .60128 L
.94281 .60128 L
.94318 .60129 L
.94355 .60129 L
.94392 .6013 L
.94428 .6013 L
.94465 .60131 L
.94501 .60131 L
.94538 .60132 L
.94574 .60132 L
.9461 .60133 L
.94646 .60133 L
.94682 .60134 L
.94718 .60134 L
.94754 .60135 L
.9479 .60135 L
.94825 .60136 L
.94861 .60136 L
.94897 .60137 L
.94932 .60137 L
.94967 .60137 L
.95003 .60138 L
.95038 .60138 L
.95073 .60139 L
.95108 .60139 L
.95143 .6014 L
.95178 .6014 L
.95213 .60141 L
.95247 .60141 L
.95282 .60141 L
.95317 .60142 L
.95351 .60142 L
.95385 .60143 L
.9542 .60143 L
.95454 .60144 L
.95488 .60144 L
.95522 .60144 L
.95556 .60145 L
.9559 .60145 L
Mistroke
.95624 .60146 L
.95658 .60146 L
.95692 .60147 L
.95725 .60147 L
.95759 .60147 L
.95792 .60148 L
.95826 .60148 L
.95859 .60149 L
.95892 .60149 L
.95926 .60149 L
.95959 .6015 L
.95992 .6015 L
.96025 .60151 L
.96058 .60151 L
.96091 .60151 L
.96123 .60152 L
.96156 .60152 L
.96189 .60153 L
.96221 .60153 L
.96254 .60153 L
.96286 .60154 L
.96318 .60154 L
.96351 .60154 L
.96383 .60155 L
.96415 .60155 L
.96447 .60156 L
.96479 .60156 L
.96511 .60156 L
.96543 .60157 L
.96575 .60157 L
.96607 .60157 L
.96638 .60158 L
.9667 .60158 L
.96702 .60159 L
.96733 .60159 L
.96764 .60159 L
.96796 .6016 L
.96827 .6016 L
.96858 .6016 L
.96889 .60161 L
.96921 .60161 L
.96952 .60161 L
.96983 .60162 L
.97014 .60162 L
.97044 .60162 L
.97075 .60163 L
.97106 .60163 L
.97137 .60163 L
.97167 .60164 L
.97198 .60164 L
Mistroke
.97228 .60164 L
.97259 .60165 L
.97289 .60165 L
.97319 .60165 L
.9735 .60166 L
.9738 .60166 L
.9741 .60166 L
.9744 .60167 L
.9747 .60167 L
.975 .60167 L
.9753 .60168 L
.9756 .60168 L
.97589 .60168 L
.97619 .60169 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{340.375, 210.375},
  ImageMargins->{{62, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005D0000dR000`400?l00000o`00003oo`3ooomE
0?ooo`00703oool2000002d0oooo1@00000/0?ooo`@00000;@3oool200000300oooo0`00000/0?oo
o`800000;@3oool3000001@0oooo000K0?ooo`040000003oool0oooo000002h0oooo00<000000?oo
o`3oool0;03oool00`000000oooo0?ooo`0]0?ooo`040000003oool0oooo00000300oooo00<00000
0?ooo`3oool0:P3oool010000000oooo0?ooo`00000/0?ooo`040000003oool0oooo000001<0oooo
000K0?ooo`040000003oool0oooo000002h0oooo00<000000?ooo`3oool0;@3oool00`000000oooo
0?ooo`0^0?ooo`030000003oool0oooo02`0oooo1@00000^0?ooo`030000003oool0oooo02X0oooo
00@000000?ooo`3oool000004`3oool001/0oooo00@000000?ooo`3oool00000;P3oool00`000000
oooo0?ooo`0^0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0:`3oool010000000
oooo0?ooo`00000]0?ooo`800000;@3oool3000001@0oooo000K0?ooo`040000003oool0oooo0000
02h0oooo00<000000?ooo`3oool0;03oool010000000oooo0?ooo`00000/0?ooo`040000003oool0
oooo000002h0oooo00<000000?ooo`000000;@3oool00`000000oooo0?ooo`0]0?ooo`030000003o
ool0oooo01<0oooo000L0?ooo`800000;P3oool2000002l0oooo0P00000^0?ooo`800000<03oool2
000002d0oooo0`00000]0?ooo`<000004`3oool00?l0ooooE@3oool00?l0ooooE@3oool00?l0oooo
E@3oool00?l0ooooE@3oool00?l0ooooE@3oool00?l0ooooE@3oool001D0ooooo`00001000000000
5@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool1000000005@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo02d0oooo00<0
00000?ooo`3oool0;P3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo02h0oooo00<0
00000?ooo`3oool0;@3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0140oooo0@00
000001D0oooo00<000000?ooo`3oool0o`3oooll0?ooo`400000000E0?ooo`030000003oool0oooo
0?l0oooo?03oool1000000005@3oool3000000D0oooo00<000000?ooo`3oool0o`3ooolc0?ooo`80
0000000E0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o`3ooolc0?ooo`400000
000E0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o`3ooolc0?ooo`400000000E
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0o`3ooolb0?ooo`400000000E0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0o`3ooold0?ooo`400000000E0?ooo`03
0000003oool0oooo00@0oooo0`00003o0?ooocD0oooo0@0000000080oooo0P0000040?ooo`800000
103oool2000000D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?oooc@0oooo
0@00000000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool`0?ooo`40000000001@3oool00000
0?ooo`3oool0000000/0oooo00<000000?ooo`3oool00P3oool3000000T0oooo00<000000?ooo`3o
ool0o`3oool^0?ooo`<0000000001@3oool000000?ooo`3oool0000000T0oooo0P0000050?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0o`3oool_0?ooo`40000000001@3oool00000
0?ooo`3oool0000000T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo0?l0oooo;`3oool1000000000P3oool2000000X0oooo0`0000040?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool0o`3oool^0?ooo`400000000E0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool0o`3oool^0?ooo`400000000E0?ooo`030000003oool0
oooo0?l0oooo?03oool1000000005@3oool00`000000oooo0?ooo`3o0?oooc`0oooo0@00000001D0
oooo0`00003o0?oooc/0oooo0P00000001D0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`3o0?ooob`0oooo0@00000001D0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`3o0?ooob/0oooo0@00000001D0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3o
0?ooob/0oooo0@00000001D0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3o0?oo
obX0oooo0@00000001D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3o0?ooobT0
oooo0@00000001D0oooo00<000000?ooo`3oool0o`3oooll0?ooo`400000000E0?ooo`<00000o`3o
oolk0?ooo`800000000E0?ooo`030000003oool0oooo0?l0oooo?03oool1000000005@3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo0?l0oooo9`3oool1000000005@3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo0?l0oooo9`3oool1000000005@3oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo0?l0oooo9P3oool1000000005@3oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo0?l0oooo9P3oool1000000005@3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo0?l0oooo9@3oool1000000005@3oool300000?l0oooo>`3oool20000
00005@3oool00`000000oooo0?ooo`3o0?oooc`0oooo0@00000001D0oooo00<000000?ooo`3oool0
o`3oooll0?ooo`400000000E0?ooo`030000003oool0oooo0?l0oooo?03oool1000000005@3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0?l0oooo8`3oool1000000005@3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo0?l0oooo8P3oool1000000005@3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo0?l0oooo8P3oool1000000005@3oool3000001P0oooo
00<000000?ooo`3oool0o`3ooolP0?ooo`800000000E0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool0o`3ooolP0?ooo`400000000E0?ooo`030000003oool0oooo0?l0oooo?03oool1
000000005@3oool00`000000oooo0?ooo`3o0?oooc`0oooo0@00000001D0oooo00<000000?ooo`3o
ool0o`3oooll0?ooo`40000000020?ooo`800000103oool2000000<0oooo0`0000050?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool0o`3ooolN0?ooo`40000000001@3oool000000?oo
o`3oool0000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0?l0oooo7P3oool10000000000D0oooo0000003oool0oooo000000080?ooo`04
0000003oool0oooo000000@0oooo0`00000L0?ooo`030000003oool0oooo0?l0oooo6`3oool30000
000000D0oooo0000003oool0oooo000000080?ooo`<000001@3oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo0?l0oooo7@3oool10000000000D0oooo0000003oool0oooo000000090?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`3o0?oo
oa`0oooo0@0000000080oooo0P00000:0?ooo`<00000103oool00`000000oooo0?ooo`3o0?oooc`0
oooo0@00000001D0oooo00<000000?ooo`3oool0o`3oooll0?ooo`400000000E0?ooo`030000003o
ool0oooo0?l0oooo?03oool1000000005@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0
oooo0?l0oooo6P3oool1000000005@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo
0?l0oooo6P3oool1000000005@3oool300000200oooo00<000000?ooo`3oool0o`3ooolH0?ooo`80
0000000E0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0o`3ooolI0?ooo`400000
000E0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0o`3ooolH0?ooo`400000000E
0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0o`3ooolH0?ooo`400000000E0?oo
o`030000003oool0oooo0?l0oooo?03oool1000000005@3oool00`000000oooo0?ooo`3o0?oooc`0
oooo0@00000001D0oooo00<000000?ooo`3oool0o`3oooll0?ooo`400000000E0?ooo`<00000903o
ool00`000000oooo0?ooo`3o0?oooa@0oooo0P00000001D0oooo00<000000?ooo`3oool0903oool0
0`000000oooo0000003o0?oooaD0oooo0@00000001D0oooo00<000000?ooo`3oool09@3oool30000
0?l0oooo503oool1000000005@3oool00`000000oooo0?ooo`0U0?ooo`800000o`3ooolE0?ooo`40
0000000E0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0o`3ooolC0?ooo`400000
000E0?ooo`030000003oool0oooo0?l0oooo?03oool1000000005@3oool00`000000oooo0?ooo`3o
0?oooc`0oooo0@00000001D0oooo0`00003o0?oooc/0oooo0P00000001D0oooo00<000000?ooo`3o
ool0:03oool00`000000oooo0?ooo`3o0?oooa40oooo0@00000001D0oooo00<000000?ooo`3oool0
:@3oool00`000000oooo0?ooo`3o0?oooa00oooo0@00000001D0oooo00<000000?ooo`3oool0:@3o
ool00`000000oooo0?ooo`3o0?oooa00oooo0@00000001D0oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`3o0?ooo`l0oooo0@00000001D0oooo00<000000?ooo`3oool0:`3oool00`00
0000oooo0?ooo`3o0?ooo`h0oooo0@00000001D0oooo00<000000?ooo`3oool0o`3oooll0?ooo`40
0000000E0?ooo`<00000o`3ooolk0?ooo`800000000E0?ooo`030000003oool0oooo0?l0oooo?03o
ool1000000005@3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0?l0oooo303oool1
000000005@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0?l0oooo2`3oool10000
00005@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0?l0oooo2`3oool100000000
0P3oool2000000@0oooo0P0000050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
;`3oool00`000000oooo0?ooo`3o0?ooo`X0oooo0@00000000050?ooo`000000oooo0?ooo`000000
2P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0
o`3oool90?ooo`40000000001@3oool000000?ooo`3oool0000000X0oooo00<000000?ooo`3oool0
0`3oool300000?l0oooo>P3oool30000000000D0oooo0000003oool0oooo0000000;0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0o`3oooll0?ooo`40000000001@3oool000000?oo
o`3oool0000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`3o0?oooc`0
oooo0@0000000080oooo0P0000090?ooo`@00000103oool00`000000oooo0?ooo`0b0?ooo`030000
003oool0oooo0?l0oooo1`3oool1000000005@3oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo0?l0oooo1P3oool1000000005@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0
oooo0?l0oooo1P3oool1000000005@3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo
0?l0oooo1@3oool1000000005@3oool3000003D0oooo00<000000?ooo`3oool0o`3oool30?ooo`80
0000000E0?ooo`030000003oool0oooo0?l0oooo?03oool1000000005@3oool00`000000oooo0?oo
o`3o0?oooc`0oooo0@00000001D0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`3n
0?ooo`400000000E0?ooo`030000003oool0oooo03T0oooo0`00003o0?ooo`400000000E0?ooo`03
0000003oool0oooo03L0oooo00@000000?ooo`3oool00000o`3oool10?ooo`400000000E0?ooo`03
0000003oool0oooo03L0oooo00<000000?ooo`3oool0o`3oool20?ooo`400000000E0?ooo`030000
003oool0oooo03P0oooo00<000000?ooo`3oool0o`3oool10?ooo`400000000E0?ooo`<00000>03o
ool00`000000oooo0?ooo`3o0?ooo`800000000E0?ooo`030000003oool0oooo03T0oooo00<00000
0?ooo`3oool0o`3oool1000000005@3oool00`000000oooo0?ooo`3o0?oooc`0oooo0@00000001D0
oooo00<000000?ooo`3oool0o`3oooll0?ooo`400000000E0?ooo`030000003oool0oooo0?l0oooo
?03oool1000000005@3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo0?`0oooo0@00
000001D0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`3k0?ooo`400000000E0?oo
o`<00000?P3oool00`000000oooo0?ooo`3i0?ooo`800000000E0?ooo`030000003oool0oooo03l0
oooo00<000000?ooo`3oool0n@3oool1000000005@3oool00`000000oooo0?ooo`3o0?oooc`0oooo
0@00000001D0oooo00<000000?ooo`3oool0o`3oooll0?ooo`400000000E0?ooo`030000003oool0
oooo0?l0oooo?03oool1000000005@3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo
0?H0oooo0@00000001D0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo0?00oooo0@00000001D0oooo0`0000130?ooo`040000003oool0oooo0?ooo`<0
0000l03oool2000000005@3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0l03oool1000000005@3oool00`000000oooo0?ooo`140?ooo`030000003o
ool0oooo0?@0oooo0@00000001D0oooo00<000000?ooo`3oool0o`3oooll0?ooo`400000000E0?oo
o`030000003oool0oooo0?l0oooo?03oool1000000000P3oool2000000@0oooo0P0000040?ooo`80
00001@3oool00`000000oooo0?ooo`3o0?oooc`0oooo0@00000000050?ooo`000000oooo0?ooo`00
0000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo04L0oooo00<000000?oo
o`3oool0l@3oool10000000000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo
000000@0oooo0`0000180?ooo`030000003oool0oooo0>h0oooo0`00000000050?ooo`000000oooo
0?ooo`0000002@3oool2000000D0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`3_
0?ooo`40000000001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`190?ooo`030000003oool0oooo0>l0oooo0@0000000080oooo0P00000:
0?ooo`8000001@3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo0>h0oooo0@000000
01D0oooo00<000000?ooo`3oool0o`3oooll0?ooo`400000000E0?ooo`030000003oool0oooo0?l0
oooo?03oool1000000005@3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo0>D0oooo
0@00000001D0oooo0`00001=0?ooo`030000003oool0oooo0080oooo0`00003U0?ooo`800000000E
0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3U
0?ooo`400000000E0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0j@3oool10000
00005@3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo0>P0oooo0@00000001D0oooo
00<000000?ooo`3oool0o`3oooll0?ooo`400000000E0?ooo`030000003oool0oooo0?l0oooo?03o
ool1000000005@3oool00`000000oooo0?ooo`3o0?oooc`0oooo0@00000001D0oooo0`00001C0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0g03oool2000000005@3oool00`000000
oooo0?ooo`1D0?ooo`030000003oool0oooo00<0oooo0`00003N0?ooo`400000000E0?ooo`030000
003oool0oooo05D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3M0?ooo`400000
000E0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0hP3oool1000000005@3oool0
0`000000oooo0?ooo`3o0?oooc`0oooo0@00000001D0oooo00<000000?ooo`3oool0o`3oooll0?oo
o`400000000E0?ooo`030000003oool0oooo0?l0oooo?03oool1000000005@3oool00`000000oooo
0?ooo`1J0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0e@3oool1000000005@3o
ool3000005/0oooo00<000000?ooo`3oool0103oool300000=D0oooo0P00000001D0oooo00<00000
0?ooo`3oool0G03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0=D0oooo0@000000
01D0oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`3K0?ooo`400000000E0?ooo`03
0000003oool0oooo0?l0oooo?03oool1000000005@3oool00`000000oooo0?ooo`1Y0?ooo`030000
003oool0oooo0<l0oooo0@00000001D0oooo00<000000?ooo`3oool0H@3oool00`000000oooo0?oo
o`040?ooo`<00000d03oool1000000005@3oool00`000000oooo0?ooo`1R0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0c`3oool1000000005@3oool300000680oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`380?ooo`800000000E0?ooo`030000003oool0oooo06<0
oooo0P0000090?ooo`<00000bP3oool1000000005@3oool00`000000oooo0?ooo`1_0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0a03oool1000000005@3oool00`000000oooo0?oo
o`1c0?ooo`<00000a@3oool1000000005@3oool00`000000oooo0?ooo`1X0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2o0?ooo`40000000020?oo
o`800000103oool2000000<0oooo0`0000050?ooo`030000003oool0oooo06T0oooo00<000000?oo
o`3oool0303oool300000<00oooo0@00000000050?ooo`000000oooo0?ooo`0000002P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`2o0?ooo`40000000001@3oool000000?ooo`3oool0000000T0oooo0`0000040?oo
o`<00000J`3oool200000<`0oooo0`00000000050?ooo`000000oooo0?ooo`000000203oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0oooo07d0oooo00<000000?ooo`3oool0^`3oool1
0000000000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0O03oool300000;`0oooo0@0000000080oooo0P00000:0?ooo`<00000103oool0
0`000000oooo0?ooo`1`0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0^`3oool1
000000005@3oool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool0]`3oool1000000005@3oool00`000000oooo0?ooo`1b0?ooo`800000303oool300000003
0?ooo`000000oooo0;D0oooo0@00000001D0oooo00<000000?ooo`3oool0M03oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0000000800000]@3oool1000000005@3oool3000008@0oooo00D0
00000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo0:`0oooo0P00000001D0oooo00<0
00000?ooo`3oool0N03oool00`000000oooo0?ooo`0<0?ooo`H00000[P3oool1000000005@3oool0
0`000000oooo0?ooo`1i0?ooo`8000003@3oool010000000oooo0?ooo`00002_0?ooo`400000000E
0?ooo`030000003oool0oooo07/0oooo0P00000A0?ooo`040000003oool0oooo00000:T0oooo0@00
000001D0oooo00<000000?ooo`3oool0S@3oool600000:P0oooo0@00000001D0oooo00<000000?oo
o`3oool0SP3oool01P000000oooo0?ooo`000000oooo00000:L0oooo0@00000001D0oooo00<00000
0?ooo`3oool0P03oool200000100oooo0`00002V0?ooo`400000000E0?ooo`<00000PP3oool20000
00l0oooo00@000000?ooo`3oool00000X`3oool2000000005@3oool00`000000oooo0?ooo`240?oo
o`030000003oool0oooo00h0oooo1000002R0?ooo`400000000E0?ooo`030000003oool0oooo09H0
oooo1@00002P0?ooo`400000000E0?ooo`030000003oool0oooo08T0oooo00<000000?ooo`3oool0
303oool400000080oooo00<000000?ooo`000000VP3oool1000000005@3oool00`000000oooo0?oo
o`2:0?ooo`@00000303oool00`000000oooo0?ooo`06000009P0oooo0@00000001D0oooo00<00000
0?ooo`3oool0WP3oool00`000000oooo00000004000009H0oooo0@00000001D0oooo00<000000?oo
o`3oool0TP3oool3000000d0oooo1@0000000`3oool000000?ooo`2A0?ooo`400000000E0?ooo`<0
0000U@3oool2000000d0oooo1P00002@0?ooo`800000000E0?ooo`030000003oool0oooo0:H0oooo
00<000000?ooo`0000000`00002?0?ooo`400000000E0?ooo`030000003oool0oooo09/0oooo0`00
000;0?ooo`@0000000D0oooo0000003oool0oooo000000290?ooo`400000000E0?ooo`030000003o
ool0oooo09h0oooo0P00000;0?ooo`T0000000<0oooo0000003oool0Q03oool1000000005@3oool0
0`000000oooo0?ooo`2T0?ooo`8000001P3oool00`000000oooo000000020?ooo`P00000PP3oool1
000000005@3oool00`000000oooo0?ooo`2V0?ooo`@00000203oool2000000030?ooo`0000000000
00D000001P3oool00`000000oooo0?ooo`1f0?ooo`400000000E0?ooo`030000003oool0oooo0:d0
oooo100000060?ooo`H00000103oool3000000030?ooo`000000oooo0080oooo00<000000?ooo`00
0000K`3oool1000000005@3oool00`000000oooo0?ooo`2a0?ooo`8000001`3oool2000000H0oooo
00<000000?ooo`0000000P0000000`3oool0000000000003000000<0oooo00<000000?ooo`000000
J03oool1000000005@3oool300000;L0oooo1@0000090?ooo`070000003oool0oooo0?ooo`000000
oooo000000030?ooo`D0000000<0oooo000000000000H`3oool2000000005@3oool00`000000oooo
0?ooo`300?ooo`<000003@3oool010000000oooo0000003oool4000000h0oooo0P0000030?ooo`03
0000003oool0oooo04d0oooo0@00000001D0oooo00<000000?ooo`3oool0aP3oool5000000@0oooo
00<000000?ooo`3oool00`3oool2000000h0oooo10000000103oool000000000000000040?ooo`06
0000003oool000000000003oool00000A03oool1000000005@3oool00`000000oooo0?ooo`3@0?oo
o`D000000`3oool6000000P0oooo0P0000030?ooo`030000003oool0oooo0080oooo2@00000:0?oo
o`030000003oool0oooo03D0oooo0@00000001D0oooo00<000000?ooo`3oool0g@3oool00`000000
oooo0?ooo`020?ooo`<0000000<0oooo000000000000100000050?ooo`050000003oool000000000
003oool0100000080?ooo`<000003@3oool2000000P0oooo00<000000?ooo`3oool0703oool10000
00003@3oool5000000<0oooo00<000000?ooo`3oool0l03oool600000080oooo0`0000040?ooo`H0
000000<0oooo000000000000103oool00`000000oooo0?ooo`020?ooo`@000001P3oool300000180
oooo00<000000?ooo`3oool0203oool1000000003`3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo0?l0oooo3@3oool8000000<0oooo1`0000050?ooo`L00000103oool6000000L0oooo
0@00000000l0oooo00<000000?ooo`3oool00`3oool300000?l0oooo<@3oool00`000000oooo0000
0002000000@0oooo0`00000000l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o
0?oooc@0oooo00<000000?ooo`3oool01@3oool1000000003`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0^0?oo
o`030000003oool0oooo02d0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0]0?oo
o`030000003oool0oooo02d0oooo00<000000?ooo`3oool04@3oool1000000003P3oool2000000D0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo0@00000001D0ooooo`00001000000000\
\>"],
  ImageRangeCache->{{{0, 339.375}, {209.375, 0}} -> {-0.609858, 0.429075, \
0.0206578, 0.00279116}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
graphic2=plotFit[boe, \
DifferentialGrowth->True,fitStyle->RGBColor[0,0,0],dataStyle->{RGBColor[1,1,1]\
},logPlot->True]\
\>", "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.152571 -0.550153 1.15347 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.17638 -0.0125 -3 -9 ]
[.17638 -0.0125 3 0 ]
[.32895 -0.0125 -3 -9 ]
[.32895 -0.0125 3 0 ]
[.48152 -0.0125 -3 -9 ]
[.48152 -0.0125 3 0 ]
[.63409 -0.0125 -3 -9 ]
[.63409 -0.0125 3 0 ]
[.78666 -0.0125 -3 -9 ]
[.78666 -0.0125 3 0 ]
[.93923 -0.0125 -3 -9 ]
[.93923 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .02658 -18 -4.5 ]
[-0.0125 .02658 0 4.5 ]
[-0.0125 .14193 -18 -4.5 ]
[-0.0125 .14193 0 4.5 ]
[-0.0125 .25728 -18 -4.5 ]
[-0.0125 .25728 0 4.5 ]
[-0.0125 .37262 -18 -4.5 ]
[-0.0125 .37262 0 4.5 ]
[-0.0125 .48797 -18 -4.5 ]
[-0.0125 .48797 0 4.5 ]
[-0.0125 .60332 -6 -4.5 ]
[-0.0125 .60332 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.17638 0 m
.17638 .00625 L
s
[(1)] .17638 -0.0125 0 1 Mshowa
.32895 0 m
.32895 .00625 L
s
[(2)] .32895 -0.0125 0 1 Mshowa
.48152 0 m
.48152 .00625 L
s
[(3)] .48152 -0.0125 0 1 Mshowa
.63409 0 m
.63409 .00625 L
s
[(4)] .63409 -0.0125 0 1 Mshowa
.78666 0 m
.78666 .00625 L
s
[(5)] .78666 -0.0125 0 1 Mshowa
.93923 0 m
.93923 .00625 L
s
[(6)] .93923 -0.0125 0 1 Mshowa
.125 Mabswid
.05432 0 m
.05432 .00375 L
s
.08484 0 m
.08484 .00375 L
s
.11535 0 m
.11535 .00375 L
s
.14587 0 m
.14587 .00375 L
s
.20689 0 m
.20689 .00375 L
s
.23741 0 m
.23741 .00375 L
s
.26792 0 m
.26792 .00375 L
s
.29844 0 m
.29844 .00375 L
s
.35947 0 m
.35947 .00375 L
s
.38998 0 m
.38998 .00375 L
s
.42049 0 m
.42049 .00375 L
s
.45101 0 m
.45101 .00375 L
s
.51204 0 m
.51204 .00375 L
s
.54255 0 m
.54255 .00375 L
s
.57306 0 m
.57306 .00375 L
s
.60358 0 m
.60358 .00375 L
s
.66461 0 m
.66461 .00375 L
s
.69512 0 m
.69512 .00375 L
s
.72564 0 m
.72564 .00375 L
s
.75615 0 m
.75615 .00375 L
s
.81718 0 m
.81718 .00375 L
s
.84769 0 m
.84769 .00375 L
s
.87821 0 m
.87821 .00375 L
s
.90872 0 m
.90872 .00375 L
s
.96975 0 m
.96975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .02658 m
.00625 .02658 L
s
[(0.5)] -0.0125 .02658 1 0 Mshowa
0 .14193 m
.00625 .14193 L
s
[(0.6)] -0.0125 .14193 1 0 Mshowa
0 .25728 m
.00625 .25728 L
s
[(0.7)] -0.0125 .25728 1 0 Mshowa
0 .37262 m
.00625 .37262 L
s
[(0.8)] -0.0125 .37262 1 0 Mshowa
0 .48797 m
.00625 .48797 L
s
[(0.9)] -0.0125 .48797 1 0 Mshowa
0 .60332 m
.00625 .60332 L
s
[(1)] -0.0125 .60332 1 0 Mshowa
.125 Mabswid
0 .04965 m
.00375 .04965 L
s
0 .07272 m
.00375 .07272 L
s
0 .09579 m
.00375 .09579 L
s
0 .11886 m
.00375 .11886 L
s
0 .165 m
.00375 .165 L
s
0 .18807 m
.00375 .18807 L
s
0 .21114 m
.00375 .21114 L
s
0 .23421 m
.00375 .23421 L
s
0 .28035 m
.00375 .28035 L
s
0 .30342 m
.00375 .30342 L
s
0 .32649 m
.00375 .32649 L
s
0 .34956 m
.00375 .34956 L
s
0 .39569 m
.00375 .39569 L
s
0 .41876 m
.00375 .41876 L
s
0 .44183 m
.00375 .44183 L
s
0 .4649 m
.00375 .4649 L
s
0 .51104 m
.00375 .51104 L
s
0 .53411 m
.00375 .53411 L
s
0 .55718 m
.00375 .55718 L
s
0 .58025 m
.00375 .58025 L
s
0 .00351 m
.00375 .00351 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.17638 .61178 m
.17638 .61803 L
s
.32895 .61178 m
.32895 .61803 L
s
.48152 .61178 m
.48152 .61803 L
s
.63409 .61178 m
.63409 .61803 L
s
.78666 .61178 m
.78666 .61803 L
s
.93923 .61178 m
.93923 .61803 L
s
.125 Mabswid
.05432 .61428 m
.05432 .61803 L
s
.08484 .61428 m
.08484 .61803 L
s
.11535 .61428 m
.11535 .61803 L
s
.14587 .61428 m
.14587 .61803 L
s
.20689 .61428 m
.20689 .61803 L
s
.23741 .61428 m
.23741 .61803 L
s
.26792 .61428 m
.26792 .61803 L
s
.29844 .61428 m
.29844 .61803 L
s
.35947 .61428 m
.35947 .61803 L
s
.38998 .61428 m
.38998 .61803 L
s
.42049 .61428 m
.42049 .61803 L
s
.45101 .61428 m
.45101 .61803 L
s
.51204 .61428 m
.51204 .61803 L
s
.54255 .61428 m
.54255 .61803 L
s
.57306 .61428 m
.57306 .61803 L
s
.60358 .61428 m
.60358 .61803 L
s
.66461 .61428 m
.66461 .61803 L
s
.69512 .61428 m
.69512 .61803 L
s
.72564 .61428 m
.72564 .61803 L
s
.75615 .61428 m
.75615 .61803 L
s
.81718 .61428 m
.81718 .61803 L
s
.84769 .61428 m
.84769 .61803 L
s
.87821 .61428 m
.87821 .61803 L
s
.90872 .61428 m
.90872 .61803 L
s
.96975 .61428 m
.96975 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .02658 m
1 .02658 L
s
.99375 .14193 m
1 .14193 L
s
.99375 .25728 m
1 .25728 L
s
.99375 .37262 m
1 .37262 L
s
.99375 .48797 m
1 .48797 L
s
.99375 .60332 m
1 .60332 L
s
.125 Mabswid
.99625 .04965 m
1 .04965 L
s
.99625 .07272 m
1 .07272 L
s
.99625 .09579 m
1 .09579 L
s
.99625 .11886 m
1 .11886 L
s
.99625 .165 m
1 .165 L
s
.99625 .18807 m
1 .18807 L
s
.99625 .21114 m
1 .21114 L
s
.99625 .23421 m
1 .23421 L
s
.99625 .28035 m
1 .28035 L
s
.99625 .30342 m
1 .30342 L
s
.99625 .32649 m
1 .32649 L
s
.99625 .34956 m
1 .34956 L
s
.99625 .39569 m
1 .39569 L
s
.99625 .41876 m
1 .41876 L
s
.99625 .44183 m
1 .44183 L
s
.99625 .4649 m
1 .4649 L
s
.99625 .51104 m
1 .51104 L
s
.99625 .53411 m
1 .53411 L
s
.99625 .55718 m
1 .55718 L
s
.99625 .58025 m
1 .58025 L
s
.99625 .00351 m
1 .00351 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
1 1 1 r
.008 w
.02381 .01718 Mdot
.12956 .19375 Mdot
.19143 .28988 Mdot
.23532 .34943 Mdot
.26936 .39331 Mdot
.29718 .41943 Mdot
.3207 .44346 Mdot
.34107 .45705 Mdot
.35904 .46854 Mdot
.37512 .47376 Mdot
.38966 .48212 Mdot
.40293 .49257 Mdot
.41515 .50197 Mdot
.42645 .5072 Mdot
.43698 .51242 Mdot
.44683 .51347 Mdot
.45608 .52182 Mdot
.4648 .52391 Mdot
.47304 .52914 Mdot
.48087 .53541 Mdot
.48831 .5375 Mdot
.49541 .54272 Mdot
.50869 .54376 Mdot
.51492 .54585 Mdot
.5209 .5469 Mdot
.52666 .55212 Mdot
.53221 .55526 Mdot
.53756 .5563 Mdot
.54273 .55944 Mdot
.54774 .56257 Mdot
.55258 .56362 Mdot
.55727 .56466 Mdot
.56625 .56571 Mdot
.57055 .56675 Mdot
.57473 .5678 Mdot
.5788 .56884 Mdot
.58662 .57093 Mdot
.59039 .57197 Mdot
.59407 .57302 Mdot
.59766 .57406 Mdot
.62067 .57511 Mdot
.62956 .57615 Mdot
.64331 .5772 Mdot
.64849 .57824 Mdot
.66532 .57929 Mdot
.66981 .58033 Mdot
.67201 .58138 Mdot
.68048 .58347 Mdot
.68253 .58451 Mdot
.73531 .58556 Mdot
.73817 .5866 Mdot
.75038 .58765 Mdot
.76993 .58869 Mdot
.77556 .58974 Mdot
.77885 .59078 Mdot
.7852 .59183 Mdot
.79031 .59287 Mdot
.79131 .59392 Mdot
.82674 .59496 Mdot
.87162 .59601 Mdot
.87569 .59705 Mdot
.90436 .59809 Mdot
.9667 .59914 Mdot
.97619 .60332 Mdot
0 0 0 r
.5 Mabswid
.02381 .01472 m
.12956 .19858 L
.19143 .2933 L
.23532 .35139 L
.26936 .39073 L
.29718 .41915 L
.3207 .44065 L
.34107 .45749 L
.35904 .47105 L
.37512 .48219 L
.38966 .49153 L
.40293 .49946 L
.41515 .50629 L
.42645 .51222 L
.43698 .51744 L
.44683 .52205 L
.45608 .52617 L
.4648 .52987 L
.47304 .5332 L
.48087 .53623 L
.48831 .53899 L
.49541 .54152 L
.50219 .54384 L
.50869 .54598 L
.51492 .54797 L
.5209 .54981 L
.52666 .55152 L
.53221 .55312 L
.53756 .55462 L
.54273 .55602 L
.54774 .55734 L
.55258 .55859 L
.55727 .55976 L
.56183 .56087 L
.56625 .56191 L
.57055 .56291 L
.57473 .56385 L
.5788 .56475 L
.58276 .5656 L
.58662 .56642 L
.59039 .5672 L
.59407 .56794 L
.59766 .56865 L
.60117 .56933 L
.6046 .56998 L
.60795 .57061 L
.61123 .57121 L
.61444 .57178 L
.61759 .57234 L
.62067 .57287 L
Mistroke
.62369 .57339 L
.62665 .57388 L
.62956 .57436 L
.63241 .57482 L
.63521 .57527 L
.63796 .5757 L
.64066 .57612 L
.64331 .57652 L
.64592 .57691 L
.64849 .57729 L
.65101 .57766 L
.65349 .57801 L
.65593 .57836 L
.65833 .5787 L
.6607 .57902 L
.66303 .57934 L
.66532 .57964 L
.66758 .57994 L
.66981 .58024 L
.67201 .58052 L
.67417 .58079 L
.6763 .58106 L
.67841 .58132 L
.68048 .58158 L
.68253 .58183 L
.68455 .58207 L
.68655 .58231 L
.68852 .58254 L
.69046 .58276 L
.69238 .58299 L
.69427 .5832 L
.69615 .58341 L
.698 .58362 L
.69982 .58382 L
.70163 .58401 L
.70341 .58421 L
.70518 .58439 L
.70692 .58458 L
.70864 .58476 L
.71035 .58494 L
.71203 .58511 L
.7137 .58528 L
.71535 .58544 L
.71698 .58561 L
.7186 .58577 L
.7202 .58592 L
.72178 .58608 L
.72334 .58623 L
.72489 .58638 L
.72642 .58652 L
Mistroke
.72794 .58666 L
.72945 .5868 L
.73093 .58694 L
.73241 .58707 L
.73387 .58721 L
.73531 .58734 L
.73675 .58746 L
.73817 .58759 L
.73957 .58771 L
.74097 .58783 L
.74235 .58795 L
.74371 .58807 L
.74507 .58819 L
.74642 .5883 L
.74775 .58841 L
.74907 .58852 L
.75038 .58863 L
.75168 .58874 L
.75296 .58884 L
.75424 .58894 L
.75551 .58904 L
.75676 .58914 L
.75801 .58924 L
.75924 .58934 L
.76047 .58944 L
.76168 .58953 L
.76289 .58962 L
.76409 .58971 L
.76527 .5898 L
.76645 .58989 L
.76762 .58998 L
.76878 .59007 L
.76993 .59015 L
.77108 .59023 L
.77221 .59032 L
.77334 .5904 L
.77445 .59048 L
.77556 .59056 L
.77667 .59064 L
.77776 .59071 L
.77885 .59079 L
.77992 .59087 L
.78099 .59094 L
.78206 .59101 L
.78311 .59109 L
.78416 .59116 L
.7852 .59123 L
.78624 .5913 L
.78727 .59137 L
.78829 .59143 L
Mistroke
.7893 .5915 L
.79031 .59157 L
.79131 .59163 L
.7923 .5917 L
.79329 .59176 L
.79427 .59182 L
.79524 .59189 L
.79621 .59195 L
.79718 .59201 L
.79813 .59207 L
.79908 .59213 L
.80003 .59219 L
.80097 .59225 L
.8019 .5923 L
.80283 .59236 L
.80375 .59242 L
.80467 .59247 L
.80558 .59253 L
.80648 .59258 L
.80738 .59263 L
.80828 .59269 L
.80917 .59274 L
.81005 .59279 L
.81093 .59284 L
.81181 .59289 L
.81267 .59295 L
.81354 .59299 L
.8144 .59304 L
.81525 .59309 L
.8161 .59314 L
.81695 .59319 L
.81779 .59324 L
.81862 .59328 L
.81946 .59333 L
.82028 .59338 L
.82111 .59342 L
.82192 .59347 L
.82274 .59351 L
.82355 .59355 L
.82435 .5936 L
.82515 .59364 L
.82595 .59368 L
.82674 .59373 L
.82753 .59377 L
.82832 .59381 L
.8291 .59385 L
.82987 .59389 L
.83064 .59393 L
.83141 .59397 L
.83218 .59401 L
Mistroke
.83294 .59405 L
.8337 .59409 L
.83445 .59413 L
.8352 .59417 L
.83595 .59421 L
.83669 .59424 L
.83743 .59428 L
.83816 .59432 L
.83889 .59435 L
.83962 .59439 L
.84035 .59443 L
.84107 .59446 L
.84179 .5945 L
.8425 .59453 L
.84321 .59457 L
.84392 .5946 L
.84462 .59464 L
.84533 .59467 L
.84602 .5947 L
.84672 .59474 L
.84741 .59477 L
.8481 .5948 L
.84879 .59483 L
.84947 .59487 L
.85015 .5949 L
.85082 .59493 L
.8515 .59496 L
.85217 .59499 L
.85284 .59502 L
.8535 .59505 L
.85416 .59508 L
.85482 .59511 L
.85548 .59514 L
.85613 .59517 L
.85678 .5952 L
.85743 .59523 L
.85808 .59526 L
.85872 .59529 L
.85936 .59532 L
.85999 .59535 L
.86063 .59538 L
.86126 .5954 L
.86189 .59543 L
.86252 .59546 L
.86314 .59549 L
.86376 .59551 L
.86438 .59554 L
.865 .59557 L
.86561 .59559 L
.86622 .59562 L
Mistroke
.86683 .59564 L
.86744 .59567 L
.86804 .5957 L
.86865 .59572 L
.86924 .59575 L
.86984 .59577 L
.87044 .5958 L
.87103 .59582 L
.87162 .59585 L
.87221 .59587 L
.87279 .59589 L
.87338 .59592 L
.87396 .59594 L
.87454 .59597 L
.87511 .59599 L
.87569 .59601 L
.87626 .59604 L
.87683 .59606 L
.8774 .59608 L
.87797 .59611 L
.87853 .59613 L
.87909 .59615 L
.87965 .59617 L
.88021 .59619 L
.88076 .59622 L
.88132 .59624 L
.88187 .59626 L
.88242 .59628 L
.88297 .5963 L
.88351 .59632 L
.88406 .59635 L
.8846 .59637 L
.88514 .59639 L
.88568 .59641 L
.88621 .59643 L
.88675 .59645 L
.88728 .59647 L
.88781 .59649 L
.88834 .59651 L
.88887 .59653 L
.88939 .59655 L
.88992 .59657 L
.89044 .59659 L
.89096 .59661 L
.89148 .59663 L
.89199 .59665 L
.89251 .59666 L
.89302 .59668 L
.89353 .5967 L
.89404 .59672 L
Mistroke
.89455 .59674 L
.89505 .59676 L
.89556 .59678 L
.89606 .59679 L
.89656 .59681 L
.89706 .59683 L
.89756 .59685 L
.89806 .59687 L
.89855 .59688 L
.89904 .5969 L
.89953 .59692 L
.90002 .59694 L
.90051 .59695 L
.901 .59697 L
.90148 .59699 L
.90197 .59701 L
.90245 .59702 L
.90293 .59704 L
.90341 .59706 L
.90389 .59707 L
.90436 .59709 L
.90484 .5971 L
.90531 .59712 L
.90578 .59714 L
.90625 .59715 L
.90672 .59717 L
.90719 .59719 L
.90765 .5972 L
.90812 .59722 L
.90858 .59723 L
.90904 .59725 L
.9095 .59726 L
.90996 .59728 L
.91042 .59729 L
.91088 .59731 L
.91133 .59732 L
.91178 .59734 L
.91224 .59735 L
.91269 .59737 L
.91314 .59738 L
.91358 .5974 L
.91403 .59741 L
.91448 .59743 L
.91492 .59744 L
.91536 .59746 L
.91581 .59747 L
.91625 .59749 L
.91668 .5975 L
.91712 .59751 L
.91756 .59753 L
Mistroke
.91799 .59754 L
.91843 .59756 L
.91886 .59757 L
.91929 .59758 L
.91972 .5976 L
.92015 .59761 L
.92058 .59762 L
.92101 .59764 L
.92143 .59765 L
.92186 .59766 L
.92228 .59768 L
.9227 .59769 L
.92312 .5977 L
.92354 .59772 L
.92396 .59773 L
.92438 .59774 L
.9248 .59776 L
.92521 .59777 L
.92562 .59778 L
.92604 .59779 L
.92645 .59781 L
.92686 .59782 L
.92727 .59783 L
.92768 .59784 L
.92809 .59786 L
.92849 .59787 L
.9289 .59788 L
.9293 .59789 L
.9297 .59791 L
.93011 .59792 L
.93051 .59793 L
.93091 .59794 L
.93131 .59795 L
.9317 .59796 L
.9321 .59798 L
.9325 .59799 L
.93289 .598 L
.93328 .59801 L
.93368 .59802 L
.93407 .59803 L
.93446 .59805 L
.93485 .59806 L
.93524 .59807 L
.93563 .59808 L
.93601 .59809 L
.9364 .5981 L
.93678 .59811 L
.93717 .59812 L
.93755 .59814 L
.93793 .59815 L
Mistroke
.93831 .59816 L
.93869 .59817 L
.93907 .59818 L
.93945 .59819 L
.93983 .5982 L
.9402 .59821 L
.94058 .59822 L
.94095 .59823 L
.94133 .59824 L
.9417 .59825 L
.94207 .59826 L
.94244 .59827 L
.94281 .59828 L
.94318 .59829 L
.94355 .59831 L
.94392 .59832 L
.94428 .59833 L
.94465 .59834 L
.94501 .59835 L
.94538 .59836 L
.94574 .59837 L
.9461 .59838 L
.94646 .59839 L
.94682 .5984 L
.94718 .5984 L
.94754 .59841 L
.9479 .59842 L
.94825 .59843 L
.94861 .59844 L
.94897 .59845 L
.94932 .59846 L
.94967 .59847 L
.95003 .59848 L
.95038 .59849 L
.95073 .5985 L
.95108 .59851 L
.95143 .59852 L
.95178 .59853 L
.95213 .59854 L
.95247 .59855 L
.95282 .59856 L
.95317 .59857 L
.95351 .59857 L
.95385 .59858 L
.9542 .59859 L
.95454 .5986 L
.95488 .59861 L
.95522 .59862 L
.95556 .59863 L
.9559 .59864 L
Mistroke
.95624 .59865 L
.95658 .59865 L
.95692 .59866 L
.95725 .59867 L
.95759 .59868 L
.95792 .59869 L
.95826 .5987 L
.95859 .59871 L
.95892 .59871 L
.95926 .59872 L
.95959 .59873 L
.95992 .59874 L
.96025 .59875 L
.96058 .59876 L
.96091 .59876 L
.96123 .59877 L
.96156 .59878 L
.96189 .59879 L
.96221 .5988 L
.96254 .59881 L
.96286 .59881 L
.96318 .59882 L
.96351 .59883 L
.96383 .59884 L
.96415 .59885 L
.96447 .59885 L
.96479 .59886 L
.96511 .59887 L
.96543 .59888 L
.96575 .59889 L
.96607 .59889 L
.96638 .5989 L
.9667 .59891 L
.96702 .59892 L
.96733 .59892 L
.96764 .59893 L
.96796 .59894 L
.96827 .59895 L
.96858 .59895 L
.96889 .59896 L
.96921 .59897 L
.96952 .59898 L
.96983 .59898 L
.97014 .59899 L
.97044 .599 L
.97075 .59901 L
.97106 .59901 L
.97137 .59902 L
.97167 .59903 L
.97198 .59904 L
Mistroke
.97228 .59904 L
.97259 .59905 L
.97289 .59906 L
.97319 .59906 L
.9735 .59907 L
.9738 .59908 L
.9741 .59909 L
.9744 .59909 L
.9747 .5991 L
.975 .59911 L
.9753 .59911 L
.9756 .59912 L
.97589 .59913 L
.97619 .59913 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{62, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCacheValid->False],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Show[graphic2,graphic1,FrameLabel->{\"log of k+1\", \"prob. of no more than k \
mutants\"}]\
\>", "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.152571 -0.525882 1.1292 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.17638 -0.0125 -3 -9 ]
[.17638 -0.0125 3 0 ]
[.32895 -0.0125 -3 -9 ]
[.32895 -0.0125 3 0 ]
[.48152 -0.0125 -3 -9 ]
[.48152 -0.0125 3 0 ]
[.63409 -0.0125 -3 -9 ]
[.63409 -0.0125 3 0 ]
[.78666 -0.0125 -3 -9 ]
[.78666 -0.0125 3 0 ]
[.93923 -0.0125 -3 -9 ]
[.93923 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -32 -21.5625 ]
[.5 -0.0125 32 -9 ]
[-0.0125 .03872 -18 -4.5 ]
[-0.0125 .03872 0 4.5 ]
[-0.0125 .15164 -18 -4.5 ]
[-0.0125 .15164 0 4.5 ]
[-0.0125 .26456 -18 -4.5 ]
[-0.0125 .26456 0 4.5 ]
[-0.0125 .37748 -18 -4.5 ]
[-0.0125 .37748 0 4.5 ]
[-0.0125 .4904 -18 -4.5 ]
[-0.0125 .4904 0 4.5 ]
[-0.0125 .60332 -6 -4.5 ]
[-0.0125 .60332 0 4.5 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -30.5625 -95 ]
[-0.01875 .30902 -18 95 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.17638 0 m
.17638 .00625 L
s
[(1)] .17638 -0.0125 0 1 Mshowa
.32895 0 m
.32895 .00625 L
s
[(2)] .32895 -0.0125 0 1 Mshowa
.48152 0 m
.48152 .00625 L
s
[(3)] .48152 -0.0125 0 1 Mshowa
.63409 0 m
.63409 .00625 L
s
[(4)] .63409 -0.0125 0 1 Mshowa
.78666 0 m
.78666 .00625 L
s
[(5)] .78666 -0.0125 0 1 Mshowa
.93923 0 m
.93923 .00625 L
s
[(6)] .93923 -0.0125 0 1 Mshowa
.125 Mabswid
.05432 0 m
.05432 .00375 L
s
.08484 0 m
.08484 .00375 L
s
.11535 0 m
.11535 .00375 L
s
.14587 0 m
.14587 .00375 L
s
.20689 0 m
.20689 .00375 L
s
.23741 0 m
.23741 .00375 L
s
.26792 0 m
.26792 .00375 L
s
.29844 0 m
.29844 .00375 L
s
.35947 0 m
.35947 .00375 L
s
.38998 0 m
.38998 .00375 L
s
.42049 0 m
.42049 .00375 L
s
.45101 0 m
.45101 .00375 L
s
.51204 0 m
.51204 .00375 L
s
.54255 0 m
.54255 .00375 L
s
.57306 0 m
.57306 .00375 L
s
.60358 0 m
.60358 .00375 L
s
.66461 0 m
.66461 .00375 L
s
.69512 0 m
.69512 .00375 L
s
.72564 0 m
.72564 .00375 L
s
.75615 0 m
.75615 .00375 L
s
.81718 0 m
.81718 .00375 L
s
.84769 0 m
.84769 .00375 L
s
.87821 0 m
.87821 .00375 L
s
.90872 0 m
.90872 .00375 L
s
.96975 0 m
.96975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -93 -25.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(log) show
87.000 12.813 moveto
(of) show
105.000 12.813 moveto
(k) show
111.000 12.813 moveto
%%IncludeResource: font Mathematica1Mono
%%IncludeFont: Mathematica1Mono
/Mathematica1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(+) show
117.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
123.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .03872 m
.00625 .03872 L
s
[(0.5)] -0.0125 .03872 1 0 Mshowa
0 .15164 m
.00625 .15164 L
s
[(0.6)] -0.0125 .15164 1 0 Mshowa
0 .26456 m
.00625 .26456 L
s
[(0.7)] -0.0125 .26456 1 0 Mshowa
0 .37748 m
.00625 .37748 L
s
[(0.8)] -0.0125 .37748 1 0 Mshowa
0 .4904 m
.00625 .4904 L
s
[(0.9)] -0.0125 .4904 1 0 Mshowa
0 .60332 m
.00625 .60332 L
s
[(1)] -0.0125 .60332 1 0 Mshowa
.125 Mabswid
0 .0613 m
.00375 .0613 L
s
0 .08389 m
.00375 .08389 L
s
0 .10647 m
.00375 .10647 L
s
0 .12905 m
.00375 .12905 L
s
0 .17422 m
.00375 .17422 L
s
0 .19681 m
.00375 .19681 L
s
0 .21939 m
.00375 .21939 L
s
0 .24197 m
.00375 .24197 L
s
0 .28714 m
.00375 .28714 L
s
0 .30973 m
.00375 .30973 L
s
0 .33231 m
.00375 .33231 L
s
0 .35489 m
.00375 .35489 L
s
0 .40006 m
.00375 .40006 L
s
0 .42265 m
.00375 .42265 L
s
0 .44523 m
.00375 .44523 L
s
0 .46781 m
.00375 .46781 L
s
0 .51298 m
.00375 .51298 L
s
0 .53557 m
.00375 .53557 L
s
0 .55815 m
.00375 .55815 L
s
0 .58073 m
.00375 .58073 L
s
0 .01613 m
.00375 .01613 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -180.281 -10.2813 Mabsadd m
1 1 Mabs scale
currentpoint translate
156 10.2813 translate
90 rotate
-156 -10.2813 translate
0 20.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(prob) show
87.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(.) show
99.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(of) show
117.000 12.813 moveto
(no) show
135.000 12.813 moveto
(more) show
165.000 12.813 moveto
(than) show
195.000 12.813 moveto
(k) show
207.000 12.813 moveto
(mutants) show
249.000 12.813 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .61178 m
.02381 .61803 L
s
.17638 .61178 m
.17638 .61803 L
s
.32895 .61178 m
.32895 .61803 L
s
.48152 .61178 m
.48152 .61803 L
s
.63409 .61178 m
.63409 .61803 L
s
.78666 .61178 m
.78666 .61803 L
s
.93923 .61178 m
.93923 .61803 L
s
.125 Mabswid
.05432 .61428 m
.05432 .61803 L
s
.08484 .61428 m
.08484 .61803 L
s
.11535 .61428 m
.11535 .61803 L
s
.14587 .61428 m
.14587 .61803 L
s
.20689 .61428 m
.20689 .61803 L
s
.23741 .61428 m
.23741 .61803 L
s
.26792 .61428 m
.26792 .61803 L
s
.29844 .61428 m
.29844 .61803 L
s
.35947 .61428 m
.35947 .61803 L
s
.38998 .61428 m
.38998 .61803 L
s
.42049 .61428 m
.42049 .61803 L
s
.45101 .61428 m
.45101 .61803 L
s
.51204 .61428 m
.51204 .61803 L
s
.54255 .61428 m
.54255 .61803 L
s
.57306 .61428 m
.57306 .61803 L
s
.60358 .61428 m
.60358 .61803 L
s
.66461 .61428 m
.66461 .61803 L
s
.69512 .61428 m
.69512 .61803 L
s
.72564 .61428 m
.72564 .61803 L
s
.75615 .61428 m
.75615 .61803 L
s
.81718 .61428 m
.81718 .61803 L
s
.84769 .61428 m
.84769 .61803 L
s
.87821 .61428 m
.87821 .61803 L
s
.90872 .61428 m
.90872 .61803 L
s
.96975 .61428 m
.96975 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .03872 m
1 .03872 L
s
.99375 .15164 m
1 .15164 L
s
.99375 .26456 m
1 .26456 L
s
.99375 .37748 m
1 .37748 L
s
.99375 .4904 m
1 .4904 L
s
.99375 .60332 m
1 .60332 L
s
.125 Mabswid
.99625 .0613 m
1 .0613 L
s
.99625 .08389 m
1 .08389 L
s
.99625 .10647 m
1 .10647 L
s
.99625 .12905 m
1 .12905 L
s
.99625 .17422 m
1 .17422 L
s
.99625 .19681 m
1 .19681 L
s
.99625 .21939 m
1 .21939 L
s
.99625 .24197 m
1 .24197 L
s
.99625 .28714 m
1 .28714 L
s
.99625 .30973 m
1 .30973 L
s
.99625 .33231 m
1 .33231 L
s
.99625 .35489 m
1 .35489 L
s
.99625 .40006 m
1 .40006 L
s
.99625 .42265 m
1 .42265 L
s
.99625 .44523 m
1 .44523 L
s
.99625 .46781 m
1 .46781 L
s
.99625 .51298 m
1 .51298 L
s
.99625 .53557 m
1 .53557 L
s
.99625 .55815 m
1 .55815 L
s
.99625 .58073 m
1 .58073 L
s
.99625 .01613 m
1 .01613 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
1 1 1 r
.008 w
.02381 .02951 Mdot
.12956 .20237 Mdot
.19143 .29647 Mdot
.23532 .35477 Mdot
.26936 .39773 Mdot
.29718 .4233 Mdot
.3207 .44683 Mdot
.34107 .46012 Mdot
.35904 .47137 Mdot
.37512 .47649 Mdot
.38966 .48467 Mdot
.40293 .4949 Mdot
.41515 .5041 Mdot
.42645 .50922 Mdot
.43698 .51433 Mdot
.44683 .51536 Mdot
.45608 .52354 Mdot
.4648 .52558 Mdot
.47304 .5307 Mdot
.48087 .53684 Mdot
.48831 .53888 Mdot
.49541 .54399 Mdot
.50869 .54502 Mdot
.51492 .54706 Mdot
.5209 .54809 Mdot
.52666 .5532 Mdot
.53221 .55627 Mdot
.53756 .55729 Mdot
.54273 .56036 Mdot
.54774 .56343 Mdot
.55258 .56445 Mdot
.55727 .56547 Mdot
.56625 .5665 Mdot
.57055 .56752 Mdot
.57473 .56854 Mdot
.5788 .56957 Mdot
.58662 .57161 Mdot
.59039 .57263 Mdot
.59407 .57366 Mdot
.59766 .57468 Mdot
.62067 .5757 Mdot
.62956 .57673 Mdot
.64331 .57775 Mdot
.64849 .57877 Mdot
.66532 .57979 Mdot
.66981 .58082 Mdot
.67201 .58184 Mdot
.68048 .58389 Mdot
.68253 .58491 Mdot
.73531 .58593 Mdot
.73817 .58695 Mdot
.75038 .58798 Mdot
.76993 .589 Mdot
.77556 .59002 Mdot
.77885 .59104 Mdot
.7852 .59207 Mdot
.79031 .59309 Mdot
.79131 .59411 Mdot
.82674 .59514 Mdot
.87162 .59616 Mdot
.87569 .59718 Mdot
.90436 .5982 Mdot
.9667 .59923 Mdot
.97619 .60332 Mdot
0 0 0 r
.5 Mabswid
.02381 .0271 m
.12956 .2071 L
.19143 .29982 L
.23532 .35669 L
.26936 .3952 L
.29718 .42302 L
.3207 .44407 L
.34107 .46056 L
.35904 .47383 L
.37512 .48474 L
.38966 .49388 L
.40293 .50165 L
.41515 .50833 L
.42645 .51414 L
.43698 .51924 L
.44683 .52376 L
.45608 .52779 L
.4648 .53141 L
.47304 .53468 L
.48087 .53764 L
.48831 .54034 L
.49541 .54282 L
.50219 .54509 L
.50869 .54719 L
.51492 .54913 L
.5209 .55093 L
.52666 .55261 L
.53221 .55418 L
.53756 .55565 L
.54273 .55702 L
.54774 .55831 L
.55258 .55953 L
.55727 .56068 L
.56183 .56176 L
.56625 .56279 L
.57055 .56376 L
.57473 .56468 L
.5788 .56556 L
.58276 .5664 L
.58662 .5672 L
.59039 .56796 L
.59407 .56868 L
.59766 .56938 L
.60117 .57004 L
.6046 .57068 L
.60795 .57129 L
.61123 .57188 L
.61444 .57245 L
.61759 .57299 L
.62067 .57351 L
Mistroke
.62369 .57402 L
.62665 .5745 L
.62956 .57497 L
.63241 .57542 L
.63521 .57586 L
.63796 .57628 L
.64066 .57669 L
.64331 .57709 L
.64592 .57747 L
.64849 .57784 L
.65101 .5782 L
.65349 .57855 L
.65593 .57888 L
.65833 .57921 L
.6607 .57953 L
.66303 .57984 L
.66532 .58014 L
.66758 .58044 L
.66981 .58072 L
.67201 .581 L
.67417 .58127 L
.6763 .58153 L
.67841 .58179 L
.68048 .58204 L
.68253 .58228 L
.68455 .58252 L
.68655 .58275 L
.68852 .58298 L
.69046 .5832 L
.69238 .58341 L
.69427 .58362 L
.69615 .58383 L
.698 .58403 L
.69982 .58423 L
.70163 .58442 L
.70341 .58461 L
.70518 .58479 L
.70692 .58497 L
.70864 .58515 L
.71035 .58532 L
.71203 .58549 L
.7137 .58566 L
.71535 .58582 L
.71698 .58598 L
.7186 .58614 L
.7202 .58629 L
.72178 .58644 L
.72334 .58659 L
.72489 .58673 L
.72642 .58687 L
Mistroke
.72794 .58701 L
.72945 .58715 L
.73093 .58728 L
.73241 .58742 L
.73387 .58755 L
.73531 .58767 L
.73675 .5878 L
.73817 .58792 L
.73957 .58804 L
.74097 .58816 L
.74235 .58828 L
.74371 .58839 L
.74507 .5885 L
.74642 .58862 L
.74775 .58872 L
.74907 .58883 L
.75038 .58894 L
.75168 .58904 L
.75296 .58914 L
.75424 .58925 L
.75551 .58934 L
.75676 .58944 L
.75801 .58954 L
.75924 .58963 L
.76047 .58973 L
.76168 .58982 L
.76289 .58991 L
.76409 .59 L
.76527 .59009 L
.76645 .59017 L
.76762 .59026 L
.76878 .59034 L
.76993 .59043 L
.77108 .59051 L
.77221 .59059 L
.77334 .59067 L
.77445 .59075 L
.77556 .59083 L
.77667 .5909 L
.77776 .59098 L
.77885 .59105 L
.77992 .59113 L
.78099 .5912 L
.78206 .59127 L
.78311 .59134 L
.78416 .59141 L
.7852 .59148 L
.78624 .59155 L
.78727 .59162 L
.78829 .59168 L
Mistroke
.7893 .59175 L
.79031 .59181 L
.79131 .59188 L
.7923 .59194 L
.79329 .592 L
.79427 .59207 L
.79524 .59213 L
.79621 .59219 L
.79718 .59225 L
.79813 .59231 L
.79908 .59236 L
.80003 .59242 L
.80097 .59248 L
.8019 .59253 L
.80283 .59259 L
.80375 .59265 L
.80467 .5927 L
.80558 .59275 L
.80648 .59281 L
.80738 .59286 L
.80828 .59291 L
.80917 .59296 L
.81005 .59301 L
.81093 .59306 L
.81181 .59311 L
.81267 .59316 L
.81354 .59321 L
.8144 .59326 L
.81525 .59331 L
.8161 .59336 L
.81695 .5934 L
.81779 .59345 L
.81862 .59349 L
.81946 .59354 L
.82028 .59359 L
.82111 .59363 L
.82192 .59367 L
.82274 .59372 L
.82355 .59376 L
.82435 .5938 L
.82515 .59385 L
.82595 .59389 L
.82674 .59393 L
.82753 .59397 L
.82832 .59401 L
.8291 .59405 L
.82987 .59409 L
.83064 .59413 L
.83141 .59417 L
.83218 .59421 L
Mistroke
.83294 .59425 L
.8337 .59429 L
.83445 .59432 L
.8352 .59436 L
.83595 .5944 L
.83669 .59443 L
.83743 .59447 L
.83816 .59451 L
.83889 .59454 L
.83962 .59458 L
.84035 .59461 L
.84107 .59465 L
.84179 .59468 L
.8425 .59472 L
.84321 .59475 L
.84392 .59479 L
.84462 .59482 L
.84533 .59485 L
.84602 .59488 L
.84672 .59492 L
.84741 .59495 L
.8481 .59498 L
.84879 .59501 L
.84947 .59504 L
.85015 .59508 L
.85082 .59511 L
.8515 .59514 L
.85217 .59517 L
.85284 .5952 L
.8535 .59523 L
.85416 .59526 L
.85482 .59529 L
.85548 .59532 L
.85613 .59535 L
.85678 .59537 L
.85743 .5954 L
.85808 .59543 L
.85872 .59546 L
.85936 .59549 L
.85999 .59552 L
.86063 .59554 L
.86126 .59557 L
.86189 .5956 L
.86252 .59562 L
.86314 .59565 L
.86376 .59568 L
.86438 .5957 L
.865 .59573 L
.86561 .59576 L
.86622 .59578 L
Mistroke
.86683 .59581 L
.86744 .59583 L
.86804 .59586 L
.86865 .59588 L
.86924 .59591 L
.86984 .59593 L
.87044 .59596 L
.87103 .59598 L
.87162 .596 L
.87221 .59603 L
.87279 .59605 L
.87338 .59607 L
.87396 .5961 L
.87454 .59612 L
.87511 .59614 L
.87569 .59617 L
.87626 .59619 L
.87683 .59621 L
.8774 .59623 L
.87797 .59626 L
.87853 .59628 L
.87909 .5963 L
.87965 .59632 L
.88021 .59634 L
.88076 .59637 L
.88132 .59639 L
.88187 .59641 L
.88242 .59643 L
.88297 .59645 L
.88351 .59647 L
.88406 .59649 L
.8846 .59651 L
.88514 .59653 L
.88568 .59655 L
.88621 .59657 L
.88675 .59659 L
.88728 .59661 L
.88781 .59663 L
.88834 .59665 L
.88887 .59667 L
.88939 .59669 L
.88992 .59671 L
.89044 .59673 L
.89096 .59675 L
.89148 .59677 L
.89199 .59679 L
.89251 .5968 L
.89302 .59682 L
.89353 .59684 L
.89404 .59686 L
Mistroke
.89455 .59688 L
.89505 .5969 L
.89556 .59691 L
.89606 .59693 L
.89656 .59695 L
.89706 .59697 L
.89756 .59698 L
.89806 .597 L
.89855 .59702 L
.89904 .59704 L
.89953 .59705 L
.90002 .59707 L
.90051 .59709 L
.901 .5971 L
.90148 .59712 L
.90197 .59714 L
.90245 .59715 L
.90293 .59717 L
.90341 .59719 L
.90389 .5972 L
.90436 .59722 L
.90484 .59724 L
.90531 .59725 L
.90578 .59727 L
.90625 .59728 L
.90672 .5973 L
.90719 .59731 L
.90765 .59733 L
.90812 .59735 L
.90858 .59736 L
.90904 .59738 L
.9095 .59739 L
.90996 .59741 L
.91042 .59742 L
.91088 .59744 L
.91133 .59745 L
.91178 .59747 L
.91224 .59748 L
.91269 .59749 L
.91314 .59751 L
.91358 .59752 L
.91403 .59754 L
.91448 .59755 L
.91492 .59757 L
.91536 .59758 L
.91581 .59759 L
.91625 .59761 L
.91668 .59762 L
.91712 .59764 L
.91756 .59765 L
Mistroke
.91799 .59766 L
.91843 .59768 L
.91886 .59769 L
.91929 .5977 L
.91972 .59772 L
.92015 .59773 L
.92058 .59774 L
.92101 .59776 L
.92143 .59777 L
.92186 .59778 L
.92228 .5978 L
.9227 .59781 L
.92312 .59782 L
.92354 .59784 L
.92396 .59785 L
.92438 .59786 L
.9248 .59787 L
.92521 .59789 L
.92562 .5979 L
.92604 .59791 L
.92645 .59792 L
.92686 .59793 L
.92727 .59795 L
.92768 .59796 L
.92809 .59797 L
.92849 .59798 L
.9289 .598 L
.9293 .59801 L
.9297 .59802 L
.93011 .59803 L
.93051 .59804 L
.93091 .59805 L
.93131 .59807 L
.9317 .59808 L
.9321 .59809 L
.9325 .5981 L
.93289 .59811 L
.93328 .59812 L
.93368 .59813 L
.93407 .59815 L
.93446 .59816 L
.93485 .59817 L
.93524 .59818 L
.93563 .59819 L
.93601 .5982 L
.9364 .59821 L
.93678 .59822 L
.93717 .59823 L
.93755 .59824 L
.93793 .59826 L
Mistroke
.93831 .59827 L
.93869 .59828 L
.93907 .59829 L
.93945 .5983 L
.93983 .59831 L
.9402 .59832 L
.94058 .59833 L
.94095 .59834 L
.94133 .59835 L
.9417 .59836 L
.94207 .59837 L
.94244 .59838 L
.94281 .59839 L
.94318 .5984 L
.94355 .59841 L
.94392 .59842 L
.94428 .59843 L
.94465 .59844 L
.94501 .59845 L
.94538 .59846 L
.94574 .59847 L
.9461 .59848 L
.94646 .59849 L
.94682 .5985 L
.94718 .59851 L
.94754 .59852 L
.9479 .59853 L
.94825 .59854 L
.94861 .59855 L
.94897 .59856 L
.94932 .59856 L
.94967 .59857 L
.95003 .59858 L
.95038 .59859 L
.95073 .5986 L
.95108 .59861 L
.95143 .59862 L
.95178 .59863 L
.95213 .59864 L
.95247 .59865 L
.95282 .59866 L
.95317 .59867 L
.95351 .59867 L
.95385 .59868 L
.9542 .59869 L
.95454 .5987 L
.95488 .59871 L
.95522 .59872 L
.95556 .59873 L
.9559 .59874 L
Mistroke
.95624 .59874 L
.95658 .59875 L
.95692 .59876 L
.95725 .59877 L
.95759 .59878 L
.95792 .59879 L
.95826 .59879 L
.95859 .5988 L
.95892 .59881 L
.95926 .59882 L
.95959 .59883 L
.95992 .59884 L
.96025 .59884 L
.96058 .59885 L
.96091 .59886 L
.96123 .59887 L
.96156 .59888 L
.96189 .59888 L
.96221 .59889 L
.96254 .5989 L
.96286 .59891 L
.96318 .59892 L
.96351 .59892 L
.96383 .59893 L
.96415 .59894 L
.96447 .59895 L
.96479 .59896 L
.96511 .59896 L
.96543 .59897 L
.96575 .59898 L
.96607 .59899 L
.96638 .59899 L
.9667 .599 L
.96702 .59901 L
.96733 .59902 L
.96764 .59902 L
.96796 .59903 L
.96827 .59904 L
.96858 .59905 L
.96889 .59905 L
.96921 .59906 L
.96952 .59907 L
.96983 .59908 L
.97014 .59908 L
.97044 .59909 L
.97075 .5991 L
.97106 .5991 L
.97137 .59911 L
.97167 .59912 L
.97198 .59913 L
Mistroke
.97228 .59913 L
.97259 .59914 L
.97289 .59915 L
.97319 .59915 L
.9735 .59916 L
.9738 .59917 L
.9741 .59917 L
.9744 .59918 L
.9747 .59919 L
.975 .59919 L
.9753 .5992 L
.9756 .59921 L
.97589 .59922 L
.97619 .59922 L
Mfstroke
.012 w
.02381 .02951 Mdot
.12956 .20237 Mdot
.19143 .29647 Mdot
.23532 .35477 Mdot
.26936 .39773 Mdot
.29718 .4233 Mdot
.3207 .44683 Mdot
.34107 .46012 Mdot
.35904 .47137 Mdot
.37512 .47649 Mdot
.38966 .48467 Mdot
.40293 .4949 Mdot
.41515 .5041 Mdot
.42645 .50922 Mdot
.43698 .51433 Mdot
.44683 .51536 Mdot
.45608 .52354 Mdot
.4648 .52558 Mdot
.47304 .5307 Mdot
.48087 .53684 Mdot
.48831 .53888 Mdot
.49541 .54399 Mdot
.50869 .54502 Mdot
.51492 .54706 Mdot
.5209 .54809 Mdot
.52666 .5532 Mdot
.53221 .55627 Mdot
.53756 .55729 Mdot
.54273 .56036 Mdot
.54774 .56343 Mdot
.55258 .56445 Mdot
.55727 .56547 Mdot
.56625 .5665 Mdot
.57055 .56752 Mdot
.57473 .56854 Mdot
.5788 .56957 Mdot
.58662 .57161 Mdot
.59039 .57263 Mdot
.59407 .57366 Mdot
.59766 .57468 Mdot
.62067 .5757 Mdot
.62956 .57673 Mdot
.64331 .57775 Mdot
.64849 .57877 Mdot
.66532 .57979 Mdot
.66981 .58082 Mdot
.67201 .58184 Mdot
.68048 .58389 Mdot
.68253 .58491 Mdot
.73531 .58593 Mdot
.73817 .58695 Mdot
.75038 .58798 Mdot
.76993 .589 Mdot
.77556 .59002 Mdot
.77885 .59104 Mdot
.7852 .59207 Mdot
.79031 .59309 Mdot
.79131 .59411 Mdot
.82674 .59514 Mdot
.87162 .59616 Mdot
.87569 .59718 Mdot
.90436 .5982 Mdot
.9667 .59923 Mdot
.97619 .60332 Mdot
.5 Mabswid
[ .015 ] 0 setdash
.02381 .01472 m
.12956 .21381 L
.19143 .31684 L
.23532 .37897 L
.26936 .42009 L
.29718 .44911 L
.3207 .47056 L
.34107 .48701 L
.35904 .49998 L
.37512 .51045 L
.38966 .51906 L
.40293 .52626 L
.41515 .53237 L
.42645 .5376 L
.43698 .54214 L
.44683 .5461 L
.45608 .5496 L
.4648 .5527 L
.47304 .55547 L
.48087 .55797 L
.48831 .56021 L
.49541 .56226 L
.50219 .56412 L
.50869 .56582 L
.51492 .56738 L
.5209 .56882 L
.52666 .57015 L
.53221 .57138 L
.53756 .57253 L
.54273 .5736 L
.54774 .57459 L
.55258 .57553 L
.55727 .5764 L
.56183 .57723 L
.56625 .578 L
.57055 .57873 L
.57473 .57942 L
.5788 .58007 L
.58276 .58069 L
.58662 .58128 L
.59039 .58184 L
.59407 .58237 L
.59766 .58287 L
.60117 .58335 L
.6046 .58381 L
.60795 .58425 L
.61123 .58467 L
.61444 .58507 L
.61759 .58546 L
.62067 .58583 L
Mistroke
.62369 .58618 L
.62665 .58652 L
.62956 .58685 L
.63241 .58716 L
.63521 .58747 L
.63796 .58776 L
.64066 .58804 L
.64331 .58831 L
.64592 .58858 L
.64849 .58883 L
.65101 .58907 L
.65349 .58931 L
.65593 .58954 L
.65833 .58976 L
.6607 .58998 L
.66303 .59018 L
.66532 .59039 L
.66758 .59058 L
.66981 .59077 L
.67201 .59096 L
.67417 .59113 L
.6763 .59131 L
.67841 .59148 L
.68048 .59164 L
.68253 .5918 L
.68455 .59196 L
.68655 .59211 L
.68852 .59226 L
.69046 .5924 L
.69238 .59254 L
.69427 .59268 L
.69615 .59281 L
.698 .59294 L
.69982 .59307 L
.70163 .59319 L
.70341 .59331 L
.70518 .59343 L
.70692 .59354 L
.70864 .59366 L
.71035 .59377 L
.71203 .59387 L
.7137 .59398 L
.71535 .59408 L
.71698 .59418 L
.7186 .59428 L
.7202 .59438 L
.72178 .59447 L
.72334 .59456 L
.72489 .59465 L
.72642 .59474 L
Mistroke
.72794 .59483 L
.72945 .59492 L
.73093 .595 L
.73241 .59508 L
.73387 .59516 L
.73531 .59524 L
.73675 .59532 L
.73817 .59539 L
.73957 .59547 L
.74097 .59554 L
.74235 .59561 L
.74371 .59568 L
.74507 .59575 L
.74642 .59582 L
.74775 .59588 L
.74907 .59595 L
.75038 .59601 L
.75168 .59608 L
.75296 .59614 L
.75424 .5962 L
.75551 .59626 L
.75676 .59632 L
.75801 .59638 L
.75924 .59643 L
.76047 .59649 L
.76168 .59655 L
.76289 .5966 L
.76409 .59665 L
.76527 .59671 L
.76645 .59676 L
.76762 .59681 L
.76878 .59686 L
.76993 .59691 L
.77108 .59696 L
.77221 .59701 L
.77334 .59705 L
.77445 .5971 L
.77556 .59715 L
.77667 .59719 L
.77776 .59724 L
.77885 .59728 L
.77992 .59732 L
.78099 .59737 L
.78206 .59741 L
.78311 .59745 L
.78416 .59749 L
.7852 .59753 L
.78624 .59757 L
.78727 .59761 L
.78829 .59765 L
Mistroke
.7893 .59769 L
.79031 .59772 L
.79131 .59776 L
.7923 .5978 L
.79329 .59783 L
.79427 .59787 L
.79524 .59791 L
.79621 .59794 L
.79718 .59797 L
.79813 .59801 L
.79908 .59804 L
.80003 .59807 L
.80097 .59811 L
.8019 .59814 L
.80283 .59817 L
.80375 .5982 L
.80467 .59823 L
.80558 .59827 L
.80648 .5983 L
.80738 .59833 L
.80828 .59836 L
.80917 .59838 L
.81005 .59841 L
.81093 .59844 L
.81181 .59847 L
.81267 .5985 L
.81354 .59853 L
.8144 .59855 L
.81525 .59858 L
.8161 .59861 L
.81695 .59863 L
.81779 .59866 L
.81862 .59869 L
.81946 .59871 L
.82028 .59874 L
.82111 .59876 L
.82192 .59879 L
.82274 .59881 L
.82355 .59883 L
.82435 .59886 L
.82515 .59888 L
.82595 .59891 L
.82674 .59893 L
.82753 .59895 L
.82832 .59897 L
.8291 .599 L
.82987 .59902 L
.83064 .59904 L
.83141 .59906 L
.83218 .59908 L
Mistroke
.83294 .59911 L
.8337 .59913 L
.83445 .59915 L
.8352 .59917 L
.83595 .59919 L
.83669 .59921 L
.83743 .59923 L
.83816 .59925 L
.83889 .59927 L
.83962 .59929 L
.84035 .59931 L
.84107 .59933 L
.84179 .59935 L
.8425 .59937 L
.84321 .59938 L
.84392 .5994 L
.84462 .59942 L
.84533 .59944 L
.84602 .59946 L
.84672 .59947 L
.84741 .59949 L
.8481 .59951 L
.84879 .59953 L
.84947 .59954 L
.85015 .59956 L
.85082 .59958 L
.8515 .59959 L
.85217 .59961 L
.85284 .59963 L
.8535 .59964 L
.85416 .59966 L
.85482 .59968 L
.85548 .59969 L
.85613 .59971 L
.85678 .59972 L
.85743 .59974 L
.85808 .59975 L
.85872 .59977 L
.85936 .59978 L
.85999 .5998 L
.86063 .59981 L
.86126 .59983 L
.86189 .59984 L
.86252 .59986 L
.86314 .59987 L
.86376 .59989 L
.86438 .5999 L
.865 .59991 L
.86561 .59993 L
.86622 .59994 L
Mistroke
.86683 .59996 L
.86744 .59997 L
.86804 .59998 L
.86865 .6 L
.86924 .60001 L
.86984 .60002 L
.87044 .60003 L
.87103 .60005 L
.87162 .60006 L
.87221 .60007 L
.87279 .60009 L
.87338 .6001 L
.87396 .60011 L
.87454 .60012 L
.87511 .60013 L
.87569 .60015 L
.87626 .60016 L
.87683 .60017 L
.8774 .60018 L
.87797 .60019 L
.87853 .60021 L
.87909 .60022 L
.87965 .60023 L
.88021 .60024 L
.88076 .60025 L
.88132 .60026 L
.88187 .60027 L
.88242 .60029 L
.88297 .6003 L
.88351 .60031 L
.88406 .60032 L
.8846 .60033 L
.88514 .60034 L
.88568 .60035 L
.88621 .60036 L
.88675 .60037 L
.88728 .60038 L
.88781 .60039 L
.88834 .6004 L
.88887 .60041 L
.88939 .60042 L
.88992 .60043 L
.89044 .60044 L
.89096 .60045 L
.89148 .60046 L
.89199 .60047 L
.89251 .60048 L
.89302 .60049 L
.89353 .6005 L
.89404 .60051 L
Mistroke
.89455 .60052 L
.89505 .60053 L
.89556 .60054 L
.89606 .60055 L
.89656 .60056 L
.89706 .60057 L
.89756 .60057 L
.89806 .60058 L
.89855 .60059 L
.89904 .6006 L
.89953 .60061 L
.90002 .60062 L
.90051 .60063 L
.901 .60064 L
.90148 .60065 L
.90197 .60065 L
.90245 .60066 L
.90293 .60067 L
.90341 .60068 L
.90389 .60069 L
.90436 .6007 L
.90484 .6007 L
.90531 .60071 L
.90578 .60072 L
.90625 .60073 L
.90672 .60074 L
.90719 .60074 L
.90765 .60075 L
.90812 .60076 L
.90858 .60077 L
.90904 .60078 L
.9095 .60078 L
.90996 .60079 L
.91042 .6008 L
.91088 .60081 L
.91133 .60081 L
.91178 .60082 L
.91224 .60083 L
.91269 .60084 L
.91314 .60084 L
.91358 .60085 L
.91403 .60086 L
.91448 .60087 L
.91492 .60087 L
.91536 .60088 L
.91581 .60089 L
.91625 .60089 L
.91668 .6009 L
.91712 .60091 L
.91756 .60092 L
Mistroke
.91799 .60092 L
.91843 .60093 L
.91886 .60094 L
.91929 .60094 L
.91972 .60095 L
.92015 .60096 L
.92058 .60096 L
.92101 .60097 L
.92143 .60098 L
.92186 .60098 L
.92228 .60099 L
.9227 .601 L
.92312 .601 L
.92354 .60101 L
.92396 .60101 L
.92438 .60102 L
.9248 .60103 L
.92521 .60103 L
.92562 .60104 L
.92604 .60105 L
.92645 .60105 L
.92686 .60106 L
.92727 .60106 L
.92768 .60107 L
.92809 .60108 L
.92849 .60108 L
.9289 .60109 L
.9293 .60109 L
.9297 .6011 L
.93011 .60111 L
.93051 .60111 L
.93091 .60112 L
.93131 .60112 L
.9317 .60113 L
.9321 .60114 L
.9325 .60114 L
.93289 .60115 L
.93328 .60115 L
.93368 .60116 L
.93407 .60116 L
.93446 .60117 L
.93485 .60117 L
.93524 .60118 L
.93563 .60119 L
.93601 .60119 L
.9364 .6012 L
.93678 .6012 L
.93717 .60121 L
.93755 .60121 L
.93793 .60122 L
Mistroke
.93831 .60122 L
.93869 .60123 L
.93907 .60123 L
.93945 .60124 L
.93983 .60124 L
.9402 .60125 L
.94058 .60125 L
.94095 .60126 L
.94133 .60126 L
.9417 .60127 L
.94207 .60127 L
.94244 .60128 L
.94281 .60128 L
.94318 .60129 L
.94355 .60129 L
.94392 .6013 L
.94428 .6013 L
.94465 .60131 L
.94501 .60131 L
.94538 .60132 L
.94574 .60132 L
.9461 .60133 L
.94646 .60133 L
.94682 .60134 L
.94718 .60134 L
.94754 .60135 L
.9479 .60135 L
.94825 .60136 L
.94861 .60136 L
.94897 .60137 L
.94932 .60137 L
.94967 .60137 L
.95003 .60138 L
.95038 .60138 L
.95073 .60139 L
.95108 .60139 L
.95143 .6014 L
.95178 .6014 L
.95213 .60141 L
.95247 .60141 L
.95282 .60141 L
.95317 .60142 L
.95351 .60142 L
.95385 .60143 L
.9542 .60143 L
.95454 .60144 L
.95488 .60144 L
.95522 .60144 L
.95556 .60145 L
.9559 .60145 L
Mistroke
.95624 .60146 L
.95658 .60146 L
.95692 .60147 L
.95725 .60147 L
.95759 .60147 L
.95792 .60148 L
.95826 .60148 L
.95859 .60149 L
.95892 .60149 L
.95926 .60149 L
.95959 .6015 L
.95992 .6015 L
.96025 .60151 L
.96058 .60151 L
.96091 .60151 L
.96123 .60152 L
.96156 .60152 L
.96189 .60153 L
.96221 .60153 L
.96254 .60153 L
.96286 .60154 L
.96318 .60154 L
.96351 .60154 L
.96383 .60155 L
.96415 .60155 L
.96447 .60156 L
.96479 .60156 L
.96511 .60156 L
.96543 .60157 L
.96575 .60157 L
.96607 .60157 L
.96638 .60158 L
.9667 .60158 L
.96702 .60159 L
.96733 .60159 L
.96764 .60159 L
.96796 .6016 L
.96827 .6016 L
.96858 .6016 L
.96889 .60161 L
.96921 .60161 L
.96952 .60161 L
.96983 .60162 L
.97014 .60162 L
.97044 .60162 L
.97075 .60163 L
.97106 .60163 L
.97137 .60163 L
.97167 .60164 L
.97198 .60164 L
Mistroke
.97228 .60164 L
.97259 .60165 L
.97289 .60165 L
.97319 .60165 L
.9735 .60166 L
.9738 .60166 L
.9741 .60166 L
.9744 .60167 L
.9747 .60167 L
.975 .60167 L
.9753 .60168 L
.9756 .60168 L
.97589 .60168 L
.97619 .60169 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{490.313, 303.063},
  ImageMargins->{{62, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCacheValid->False],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["References", "Section"],

Cell["\<\
L. Boe, T. Tolker-Nielsen, K.-M. Eegholm, H. Spliid, A. Vrang (1994), \
Fluctuation analysis of mutations to nalidixic resistance in Escherichia \
coli, Journal of Bacteriology, 176: 2781-2787.

M. Demerec (1945), production of staphylococcus strains resistant to various \
concentrations of penicillin, proceedings of the National Academic of \
Science, 31: 16-24.\
\>", "Text"],

Cell["\<\
Q. Zheng (1999), Progress of a half century in the study of the \
Luria-Delbruck distribution. Mathematical Biosciences, 162: 1-32.

Q. Zheng (2002), Statistical and algorithmic methods for fluctuation analysis \
with SALVADOR as an implementation,  Mathematical Bioscences, 176: 237-252.\
\>", "Text"],

Cell["\<\
Q. Zheng (2005), New algorithms for Luria-Delbruck fluctuation analysis, \
Mathematical Biosciences , to appear.\
\>", "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 685}},
ScreenStyleEnvironment->"Presentation",
WindowSize->{948, 651},
WindowMargins->{{Automatic, 18}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{"u05", "vancouver"}, \
"good2.nb.ps", CharacterEncoding -> "WindowsANSI"],
"Magnification"->1}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Info3317216010-4785616"->{
    Cell[4410, 143, 193, 3, 51, "Print",
      CellTags->"Info3317216010-4785616"]},
  "Info3317216011-1867243"->{
    Cell[4851, 163, 203, 3, 51, "Print",
      CellTags->"Info3317216011-1867243"]},
  "Info3317216011-7491825"->{
    Cell[5299, 183, 443, 7, 97, "Print",
      CellTags->"Info3317216011-7491825"]},
  "Info3317216011-4729632"->{
    Cell[6960, 234, 219, 4, 51, "Print",
      CellTags->"Info3317216011-4729632"]},
  "Info3317216011-4887619"->{
    Cell[7811, 264, 291, 5, 74, "Print",
      CellTags->"Info3317216011-4887619"]},
  "Info3317216011-5180669"->{
    Cell[34886, 1111, 461, 7, 97, "Print",
      CellTags->"Info3317216011-5180669"]},
  "Info3317216011-5924977"->{
    Cell[36038, 1145, 338, 5, 74, "Print",
      CellTags->"Info3317216011-5924977"]},
  "Info3317216011-5580427"->{
    Cell[37546, 1194, 280, 4, 51, "Print",
      CellTags->"Info3317216011-5580427"]},
  "Info3317216011-4936587"->{
    Cell[39505, 1254, 163, 3, 51, "Print",
      CellTags->"Info3317216011-4936587"]},
  "Info3317216012-4476652"->{
    Cell[41783, 1350, 173, 3, 51, "Print",
      CellTags->"Info3317216012-4476652"]},
  "Info3317216013-2836232"->{
    Cell[42934, 1395, 430, 6, 97, "Print",
      CellTags->"Info3317216013-2836232"]},
  "Info3317216015-6655442"->{
    Cell[46287, 1517, 260, 4, 58, "Print",
      CellTags->"Info3317216015-6655442"]},
  "Info3317216057-4785192"->{
    Cell[47427, 1563, 186, 3, 51, "Print",
      CellTags->"Info3317216057-4785192"]},
  "Info3317216058-9081352"->{
    Cell[48161, 1588, 183, 3, 51, "Print",
      CellTags->"Info3317216058-9081352"]},
  "Info3317216076-5808766"->{
    Cell[52477, 1762, 167, 3, 51, "Print",
      CellTags->"Info3317216076-5808766"]},
  "Info3317216076-9348376"->{
    Cell[52916, 1782, 326, 5, 74, "Print",
      CellTags->"Info3317216076-9348376"]},
  "Info3317216076-1494160"->{
    Cell[53497, 1804, 421, 6, 120, "Print",
      CellTags->"Info3317216076-1494160"]},
  "Info3317216076-2304299"->{
    Cell[54153, 1827, 302, 5, 81, "Print",
      CellTags->"Info3317216076-2304299"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Info3317216010-4785616", 206450, 8776},
  {"Info3317216011-1867243", 206565, 8779},
  {"Info3317216011-7491825", 206680, 8782},
  {"Info3317216011-4729632", 206795, 8785},
  {"Info3317216011-4887619", 206910, 8788},
  {"Info3317216011-5180669", 207025, 8791},
  {"Info3317216011-5924977", 207142, 8794},
  {"Info3317216011-5580427", 207259, 8797},
  {"Info3317216011-4936587", 207376, 8800},
  {"Info3317216012-4476652", 207493, 8803},
  {"Info3317216013-2836232", 207610, 8806},
  {"Info3317216015-6655442", 207727, 8809},
  {"Info3317216057-4785192", 207844, 8812},
  {"Info3317216058-9081352", 207961, 8815},
  {"Info3317216076-5808766", 208078, 8818},
  {"Info3317216076-9348376", 208195, 8821},
  {"Info3317216076-1494160", 208312, 8824},
  {"Info3317216076-2304299", 208430, 8827}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 52, 1, 59, "Title"],
Cell[1831, 56, 166, 6, 145, "Text"],

Cell[CellGroupData[{
Cell[2022, 66, 31, 0, 77, "Section"],
Cell[2056, 68, 450, 11, 95, "Text"],

Cell[CellGroupData[{
Cell[2531, 83, 58, 1, 38, "Input"],
Cell[2592, 86, 64, 1, 38, "Output"]
}, Open  ]],
Cell[2671, 90, 48, 1, 38, "Input"],
Cell[2722, 93, 168, 3, 45, "Text"],
Cell[2893, 98, 37, 0, 36, "Input"],
Cell[2933, 100, 282, 5, 70, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3252, 110, 45, 1, 67, "Section"],
Cell[3300, 113, 935, 20, 145, "Text"],
Cell[4238, 135, 93, 1, 38, "Input"],

Cell[CellGroupData[{
Cell[4356, 140, 51, 1, 38, "Input"],
Cell[4410, 143, 193, 3, 51, "Print",
  CellTags->"Info3317216010-4785616"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4640, 151, 66, 1, 38, "Input"],
Cell[4709, 154, 52, 1, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4798, 160, 50, 1, 38, "Input"],
Cell[4851, 163, 203, 3, 51, "Print",
  CellTags->"Info3317216011-1867243"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5091, 171, 65, 1, 38, "Input"],
Cell[5159, 174, 51, 1, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5247, 180, 49, 1, 38, "Input"],
Cell[5299, 183, 443, 7, 97, "Print",
  CellTags->"Info3317216011-7491825"]
}, Open  ]],
Cell[5757, 193, 85, 2, 45, "Text"],

Cell[CellGroupData[{
Cell[5867, 199, 77, 1, 38, "Input"],
Cell[5947, 202, 105, 2, 38, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6101, 210, 58, 1, 67, "Section"],
Cell[6162, 213, 722, 14, 120, "Text"],

Cell[CellGroupData[{
Cell[6909, 231, 48, 1, 38, "Input"],
Cell[6960, 234, 219, 4, 51, "Print",
  CellTags->"Info3317216011-4729632"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7216, 243, 53, 1, 38, "Input"],
Cell[7272, 246, 315, 5, 61, "Output"]
}, Open  ]],
Cell[7602, 254, 138, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[7765, 261, 43, 1, 38, "Input"],
Cell[7811, 264, 291, 5, 74, "Print",
  CellTags->"Info3317216011-4887619"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8139, 274, 84, 1, 38, "Input"],
Cell[8226, 277, 280, 4, 61, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8543, 286, 82, 1, 38, "Input"],
Cell[8628, 289, 276, 4, 61, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8941, 298, 128, 2, 38, "Input"],
Cell[9072, 302, 25487, 791, 186, 7429, 564, "GraphicsData", "PostScript", \
"Graphics"],
Cell[34562, 1095, 130, 3, 38, "Output"]
}, Open  ]],
Cell[34707, 1101, 109, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[34841, 1108, 42, 1, 38, "Input"],
Cell[34886, 1111, 461, 7, 97, "Print",
  CellTags->"Info3317216011-5180669"]
}, Open  ]],

Cell[CellGroupData[{
Cell[35384, 1123, 52, 1, 38, "Input"],
Cell[35439, 1126, 326, 5, 61, "Output"]
}, Open  ]],
Cell[35780, 1134, 184, 4, 70, "Text"],

Cell[CellGroupData[{
Cell[35989, 1142, 46, 1, 38, "Input"],
Cell[36038, 1145, 338, 5, 74, "Print",
  CellTags->"Info3317216011-5924977"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36413, 1155, 56, 1, 38, "Input"],
Cell[36472, 1158, 323, 5, 61, "Output"]
}, Open  ]],
Cell[36810, 1166, 112, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[36947, 1173, 60, 1, 38, "Input"],
Cell[37010, 1176, 324, 5, 61, "Output"]
}, Open  ]],
Cell[37349, 1184, 122, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[37496, 1191, 47, 1, 38, "Input"],
Cell[37546, 1194, 280, 4, 51, "Print",
  CellTags->"Info3317216011-5580427"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37863, 1203, 59, 1, 38, "Input"],
Cell[37925, 1206, 317, 5, 61, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[38291, 1217, 63, 1, 67, "Section"],
Cell[38357, 1220, 513, 9, 120, "Text"],
Cell[38873, 1231, 73, 1, 38, "Input"],
Cell[38949, 1234, 86, 2, 45, "Text"],
Cell[39038, 1238, 198, 3, 61, "Input"],
Cell[39239, 1243, 192, 4, 70, "Text"],

Cell[CellGroupData[{
Cell[39456, 1251, 46, 1, 38, "Input"],
Cell[39505, 1254, 163, 3, 51, "Print",
  CellTags->"Info3317216011-4936587"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39705, 1262, 50, 1, 38, "Input"],
Cell[39758, 1265, 53, 1, 38, "Output"]
}, Open  ]],
Cell[39826, 1269, 231, 4, 70, "Text"],

Cell[CellGroupData[{
Cell[40082, 1277, 34, 0, 36, "Input"],
Cell[40119, 1279, 169, 2, 80, "Output"]
}, Open  ]],
Cell[40303, 1284, 292, 5, 70, "Text"],

Cell[CellGroupData[{
Cell[40620, 1293, 67, 0, 36, "Input"],
Cell[40690, 1295, 34, 1, 28, "Print"],
Cell[40727, 1298, 52, 1, 28, "Print"],
Cell[40782, 1301, 50, 1, 28, "Print"],
Cell[40835, 1304, 52, 1, 28, "Print"],
Cell[40890, 1307, 52, 1, 28, "Print"],
Cell[40945, 1310, 52, 1, 28, "Print"],
Cell[41000, 1313, 53, 1, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41090, 1319, 68, 0, 36, "Input"],
Cell[41161, 1321, 35, 1, 28, "Print"],
Cell[41199, 1324, 52, 1, 28, "Print"],
Cell[41254, 1327, 52, 1, 28, "Print"],
Cell[41309, 1330, 52, 1, 28, "Print"],
Cell[41364, 1333, 52, 1, 28, "Print"],
Cell[41419, 1336, 53, 1, 38, "Output"]
}, Open  ]],
Cell[41487, 1340, 243, 4, 70, "Text"],

Cell[CellGroupData[{
Cell[41755, 1348, 25, 0, 36, "Input"],
Cell[41783, 1350, 173, 3, 51, "Print",
  CellTags->"Info3317216012-4476652"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41993, 1358, 33, 0, 36, "Input"],
Cell[42029, 1360, 75, 1, 38, "Output"]
}, Open  ]],
Cell[42119, 1364, 68, 0, 45, "Text"],

Cell[CellGroupData[{
Cell[42212, 1368, 33, 0, 36, "Input"],
Cell[42248, 1370, 186, 3, 80, "Output"]
}, Open  ]],
Cell[42449, 1376, 85, 2, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42571, 1383, 56, 0, 77, "Section"],
Cell[42630, 1385, 246, 4, 70, "Text"],

Cell[CellGroupData[{
Cell[42901, 1393, 30, 0, 36, "Input"],
Cell[42934, 1395, 430, 6, 97, "Print",
  CellTags->"Info3317216013-2836232"]
}, Open  ]],
Cell[43379, 1404, 93, 2, 45, "Text"],

Cell[CellGroupData[{
Cell[43497, 1410, 38, 0, 36, "Input"],
Cell[43538, 1412, 75, 1, 38, "Output"]
}, Open  ]],
Cell[43628, 1416, 291, 5, 70, "Text"],

Cell[CellGroupData[{
Cell[43944, 1425, 49, 0, 36, "Input"],
Cell[43996, 1427, 75, 1, 38, "Output"]
}, Open  ]],
Cell[44086, 1431, 68, 0, 45, "Text"],

Cell[CellGroupData[{
Cell[44179, 1435, 38, 0, 36, "Input"],
Cell[44220, 1437, 266, 4, 80, "Output"]
}, Open  ]],
Cell[44501, 1444, 638, 9, 145, "Text"],

Cell[CellGroupData[{
Cell[45164, 1457, 104, 3, 36, "Input"],
Cell[45271, 1462, 64, 1, 28, "Print"],
Cell[45338, 1465, 34, 1, 28, "Print"],
Cell[45375, 1468, 51, 1, 28, "Print"],
Cell[45429, 1471, 51, 1, 28, "Print"],
Cell[45483, 1474, 51, 1, 28, "Print"],
Cell[45537, 1477, 50, 1, 28, "Print"],
Cell[45590, 1480, 50, 1, 28, "Print"],
Cell[45643, 1483, 64, 1, 28, "Print"],
Cell[45710, 1486, 35, 1, 28, "Print"],
Cell[45748, 1489, 52, 1, 28, "Print"],
Cell[45803, 1492, 52, 1, 28, "Print"],
Cell[45858, 1495, 51, 1, 28, "Print"],
Cell[45912, 1498, 52, 1, 28, "Print"],
Cell[45967, 1501, 52, 1, 28, "Print"],
Cell[46022, 1504, 75, 1, 38, "Output"]
}, Open  ]],
Cell[46112, 1508, 117, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[46254, 1515, 30, 0, 36, "Input"],
Cell[46287, 1517, 260, 4, 58, "Print",
  CellTags->"Info3317216015-6655442"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46584, 1526, 39, 0, 36, "Input"],
Cell[46626, 1528, 53, 1, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46716, 1534, 46, 0, 36, "Input"],
Cell[46765, 1536, 55, 1, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46857, 1542, 75, 0, 36, "Input"],
Cell[46935, 1544, 75, 1, 38, "Output"]
}, Open  ]],
Cell[47025, 1548, 113, 3, 45, "Text"],
Cell[47141, 1553, 224, 4, 70, "Text"],

Cell[CellGroupData[{
Cell[47390, 1561, 34, 0, 36, "Input"],
Cell[47427, 1563, 186, 3, 51, "Print",
  CellTags->"Info3317216057-4785192"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47650, 1571, 42, 0, 36, "Input"],
Cell[47695, 1573, 76, 1, 38, "Output"]
}, Open  ]],
Cell[47786, 1577, 313, 5, 95, "Text"],

Cell[CellGroupData[{
Cell[48124, 1586, 34, 0, 36, "Input"],
Cell[48161, 1588, 183, 3, 51, "Print",
  CellTags->"Info3317216058-9081352"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48381, 1596, 47, 0, 36, "Input"],
Cell[48431, 1598, 75, 1, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48543, 1604, 59, 0, 36, "Input"],
Cell[48605, 1606, 135, 2, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48777, 1613, 63, 0, 36, "Input"],
Cell[48843, 1615, 129, 2, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[49009, 1622, 70, 0, 36, "Input"],
Cell[49082, 1624, 74, 1, 38, "Output"]
}, Open  ]],
Cell[49171, 1628, 113, 3, 45, "Text"],
Cell[49287, 1633, 68, 0, 45, "Text"],

Cell[CellGroupData[{
Cell[49380, 1637, 54, 0, 36, "Input"],
Cell[49437, 1639, 75, 1, 38, "Output"]
}, Open  ]],
Cell[49527, 1643, 112, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[49664, 1650, 42, 0, 36, "Input"],
Cell[49709, 1652, 76, 1, 38, "Output"]
}, Open  ]],
Cell[49800, 1656, 115, 3, 45, "Text"],
Cell[49918, 1661, 116, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[50059, 1668, 42, 0, 36, "Input"],
Cell[50104, 1670, 342, 5, 106, "Output"]
}, Open  ]],
Cell[50461, 1678, 132, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[50618, 1685, 111, 3, 54, "Input"],
Cell[50732, 1690, 64, 1, 28, "Print"],
Cell[50799, 1693, 59, 1, 28, "Print"],
Cell[50861, 1696, 76, 1, 28, "Print"],
Cell[50940, 1699, 73, 1, 28, "Print"],
Cell[51016, 1702, 75, 1, 28, "Print"],
Cell[51094, 1705, 73, 1, 28, "Print"],
Cell[51170, 1708, 74, 1, 28, "Print"],
Cell[51247, 1711, 72, 1, 28, "Print"],
Cell[51322, 1714, 74, 1, 28, "Print"],
Cell[51399, 1717, 74, 1, 28, "Print"],
Cell[51476, 1720, 64, 1, 28, "Print"],
Cell[51543, 1723, 59, 1, 28, "Print"],
Cell[51605, 1726, 74, 1, 28, "Print"],
Cell[51682, 1729, 75, 1, 28, "Print"],
Cell[51760, 1732, 75, 1, 28, "Print"],
Cell[51838, 1735, 74, 1, 28, "Print"],
Cell[51915, 1738, 75, 1, 28, "Print"],
Cell[51993, 1741, 76, 1, 38, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[52118, 1748, 65, 1, 67, "Section"],
Cell[52186, 1751, 215, 4, 70, "Text"],

Cell[CellGroupData[{
Cell[52426, 1759, 48, 1, 38, "Input"],
Cell[52477, 1762, 167, 3, 51, "Print",
  CellTags->"Info3317216076-5808766"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52681, 1770, 52, 1, 38, "Input"],
Cell[52736, 1773, 89, 1, 28, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52862, 1779, 51, 1, 38, "Input"],
Cell[52916, 1782, 326, 5, 74, "Print",
  CellTags->"Info3317216076-9348376"]
}, Open  ]],

Cell[CellGroupData[{
Cell[53279, 1792, 69, 1, 38, "Input"],
Cell[53351, 1795, 53, 1, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[53441, 1801, 53, 1, 38, "Input"],
Cell[53497, 1804, 421, 6, 120, "Print",
  CellTags->"Info3317216076-1494160"]
}, Open  ]],

Cell[CellGroupData[{
Cell[53955, 1815, 57, 1, 38, "Input"],
Cell[54015, 1818, 53, 1, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[54105, 1824, 45, 1, 38, "Input"],
Cell[54153, 1827, 302, 5, 81, "Print",
  CellTags->"Info3317216076-2304299"]
}, Open  ]],

Cell[CellGroupData[{
Cell[54492, 1837, 49, 1, 38, "Input"],
Cell[54544, 1840, 53, 1, 38, "Output"]
}, Open  ]],
Cell[54612, 1844, 135, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[54772, 1851, 46, 3, 54, "Input"],
Cell[54821, 1856, 52, 1, 38, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[54922, 1863, 34, 0, 77, "Section"],
Cell[54959, 1865, 147, 3, 45, "Text"],
Cell[55109, 1870, 2441, 32, 540, "Input"],
Cell[57553, 1904, 110, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[57688, 1911, 30, 0, 36, "Input"],
Cell[57721, 1913, 53, 1, 38, "Output"]
}, Open  ]],
Cell[57789, 1917, 137, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[57951, 1924, 43, 0, 36, "Input"],
Cell[57997, 1926, 33601, 1284, 186, 14938, 1049, "GraphicsData", \
"PostScript", "Graphics"],
Cell[91601, 3212, 130, 3, 38, "Output"]
}, Open  ]],
Cell[91746, 3218, 108, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[91879, 3225, 29, 0, 36, "Input"],
Cell[91911, 3227, 76, 1, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[92024, 3233, 68, 0, 36, "Input"],
Cell[92095, 3235, 32904, 1275, 186, 14946, 1049, "GraphicsData", \
"PostScript", "Graphics"],
Cell[125002, 4512, 130, 3, 38, "Output"]
}, Open  ]],
Cell[125147, 4518, 114, 3, 45, "Text"],

Cell[CellGroupData[{
Cell[125286, 4525, 33, 0, 36, "Input"],
Cell[125322, 4527, 277, 4, 84, "Output"]
}, Open  ]],
Cell[125614, 4534, 344, 6, 95, "Text"],

Cell[CellGroupData[{
Cell[125983, 4544, 153, 3, 54, "Input"],
Cell[126139, 4549, 34357, 1293, 219, 14953, 1049, "GraphicsData", \
"PostScript", "Graphics"],
Cell[160499, 5844, 130, 3, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[160666, 5852, 145, 4, 72, "Input"],
Cell[160814, 5858, 15073, 1053, 186, 14950, 1049, "GraphicsData", \
"PostScript", "Graphics",
  ImageCacheValid->False],
Cell[175890, 6913, 130, 3, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[176057, 6921, 116, 3, 54, "Input"],
Cell[176176, 6926, 28285, 1790, 312, 28158, 1786, "GraphicsData", \
"PostScript", "Graphics",
  ImageCacheValid->False],
Cell[204464, 8718, 130, 3, 38, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[204643, 8727, 29, 0, 77, "Section"],
Cell[204675, 8729, 389, 8, 145, "Text"],
Cell[205067, 8739, 312, 6, 120, "Text"],
Cell[205382, 8747, 136, 3, 45, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

